@@ -1,5 +1,6 @@
 /******************************************************************************
-* Copyright 2015-2021 Xilinx, Inc.
+* Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -43,11 +44,19 @@ bool AuthenticationContext::zynpmpVerEs1 = false;
 *****************************************************   F U N C T I O N S   ***
 -------------------------------------------------------------------------------
 */
-void AuthenticationContext::SetPresignFile(const std::string& filename) 
+
+/******************************************************************************/
+void AuthenticationContext::SetPresignFile(const std::string& filename)
 {
     presignFile = filename;
 }
 
+/******************************************************************************/
+void AuthenticationContext::SetACFile(const std::string& filename)
+{
+    acFile = filename;
+}
+
 /******************************************************************************/
 void AuthenticationContext::SetUdfFile(const std::string& filename)
 {
@@ -216,12 +225,6 @@ void AuthenticationContext::GetPresign(const std::string& presignFilename, uint8
     std::string filename(presignFilename);
     std::string baseFile = StringUtils::BaseName(filename);
 
-    if (index > 9) 
-    {
-        LOG_DEBUG(DEBUG_STAMP, "Partition index count %d too high",index);
-        LOG_ERROR("Presign file should have proper index (0-9)");
-    }
-
     if (index != 0)
     {
         size_t x = filename.find(".0.");
@@ -229,8 +232,9 @@ void AuthenticationContext::GetPresign(const std::string& presignFilename, uint8
         {
             LOG_ERROR("Presign file %s does not have partition index (*.0.*)", baseFile.c_str());
         }
-        // nudge the '0' to '1' ... '9'.
-        filename[x+1] = (char)(filename[x+1] + index); 
+        // nudge the '0' to index number
+        std::string sindex = std::to_string(index);
+        filename.replace(x+1,1,sindex);
     }
 
     LOG_TRACE("Reading presign file - %s", filename.c_str());
@@ -274,6 +278,54 @@ void AuthenticationContext::GetPresign(const std::string& presignFilename, uint8
     }
 }
 
+/******************************************************************************/
+void AuthenticationContext::GetAC(const std::string& acFilename, uint8_t* ac, uint32_t index)
+{
+    std::string filename(acFilename);
+    std::string baseFile = StringUtils::BaseName(filename);
+
+    if (index != 0)
+    {
+        size_t x = filename.find(".0.");
+        if (x == std::string::npos)
+        {
+            LOG_ERROR("AC file %s does not have partition index (*.0.*)", baseFile.c_str());
+        }
+        // nudge the '0' to index number
+        std::string sindex = std::to_string(index);
+        filename.replace(x + 1, 1, sindex);
+    }
+
+    LOG_TRACE("Reading AC from - %s", filename.c_str());
+    FILE* filePtr;
+    filePtr = fopen(filename.c_str(), "rb");
+    if (filePtr)
+    {
+        fseek(filePtr, 0, SEEK_END);
+        long size = ftell(filePtr);
+        fclose(filePtr);
+        if (size == GetCertificateSize())
+        {
+            // read binary
+            filePtr = fopen(filename.c_str(), "rb");
+            long read_size = fread(ac, 1, GetCertificateSize(), filePtr);
+            if (read_size != GetCertificateSize())
+            {
+                LOG_ERROR("Authentication Error !!!\n           AC file %s should be of %d bytes", baseFile.c_str(), GetCertificateSize());
+            }
+            fclose(filePtr);
+        }
+        else
+        {
+            LOG_ERROR("Authentication Error !!!\n           AC file %s should be of %d bytes", baseFile.c_str(), GetCertificateSize());
+        }
+    }
+    else
+    {
+        LOG_ERROR("Failure opening AC file - %s", baseFile.c_str());
+    }
+}
+
 /******************************************************************************/
 void AuthenticationContext::LoadUdfData(const std::string& udfFilename, uint8_t* signature)
 {
