@@ -1,5 +1,6 @@
 /******************************************************************************
-* Copyright 2015-2021 Xilinx, Inc.
+* Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -471,12 +472,12 @@ void VersalKey::Parse(const std::string& filename, bool isSecret0)
 
                     fseek(f, 0, 0); // rewind
 
-                    /* If the file starts with 'N', then it is Xilinx Format
+                    /* If the file starts with 'N', then it is AMD Format
                        If it starts with '-', then it is OpenSSL format */
                     if (name == 'N')
                     {
-                        LOG_INFO("Parsing RSA key (Xilinx Format)");
-                        errCode = ParseXilinxRsaKey(f);
+                        LOG_INFO("Parsing RSA key (AMD Format)");
+                        errCode = ParseAMDRsaKey(f);
                     }
                     else if (name == '-')
                     {
@@ -485,7 +486,7 @@ void VersalKey::Parse(const std::string& filename, bool isSecret0)
                     }
                     else
                     {
-                        LOG_ERROR("Unsupported key file - %s\n           Supported key formats: Xilinx Format & OpenSSL format", basefile.c_str());
+                        LOG_ERROR("Unsupported key file - %s\n           Supported key formats: AMD Format & OpenSSL format", basefile.c_str());
                     }
                     fclose(f);
 
@@ -576,15 +577,15 @@ void VersalKey::Parse(const std::string& filename, bool isSecret0)
                     do
                     {
                         name = getc(f);
-                    } while (name >= 0 && isspace(name));
+                    } while (name >= 0 && isspace(name) && name != EOF);
 
                     fseek(f, 0, 0); // rewind
 
-                    /* If the file starts with 'N', then it is Xilinx Format. If it starts with '-', then it is OpenSSL format */
+                    /* If the file starts with 'N', then it is AMD Format. If it starts with '-', then it is OpenSSL format */
                     if (name == 'N')
                     {
-                        LOG_INFO("Parsing RSA key (Xilinx Format)");
-                        errCode = ParseXilinxRsaKey(f);
+                        LOG_INFO("Parsing RSA key (AMD Format)");
+                        errCode = ParseAMDRsaKey(f);
                     }
                     else if (name == '-')
                     {
@@ -593,7 +594,7 @@ void VersalKey::Parse(const std::string& filename, bool isSecret0)
                     }
                     else
                     {
-                        LOG_ERROR("Unsupported key file - %s\n           Supported key formats: Xilinx Format & OpenSSL format", basefile.c_str());
+                        LOG_ERROR("Unsupported key file - %s\n           Supported key formats: AMD Format & OpenSSL format", basefile.c_str());
                     }
                     fclose(f);
 
