@@ -42,7 +42,7 @@
 
 /* "%code requires" blocks.  */
 /* Line 33 of lalr1.cc  */
-#line 36 "../s/bif.y"
+#line 37 "../s/bif.y"
 
        namespace BIF {
         class BisonScanner;
@@ -68,7 +68,7 @@
 #endif
 
 /* Line 33 of lalr1.cc  */
-#line 23 "../s/bif.y"
+#line 24 "../s/bif.y"
 namespace BIF {
 /* Line 33 of lalr1.cc  */
 #line 75 "../bisonflex/bif.tab.hpp"
@@ -82,7 +82,7 @@ namespace BIF {
     union semantic_type
     {
 /* Line 33 of lalr1.cc  */
-#line 72 "../s/bif.y"
+#line 73 "../s/bif.y"
 
     uint32_t token;
     uint64_t number;
@@ -175,109 +175,118 @@ namespace BIF {
      AUTHBLOCKS = 306,
      BOOTVECTORS = 307,
      PRESIGN = 308,
-     BIF_SECTION = 309,
-     UDF_DATA = 310,
-     MCS = 311,
-     BIN = 312,
-     SLR_NUM = 313,
-     CLUSTER_NUM = 314,
-     DICE = 315,
-     PARENT_ID = 316,
-     ID_CODE = 317,
-     EXT_ID_CODE = 318,
-     BYPASS_IDCODE_CHECK = 319,
-     A_HWROT = 320,
-     S_HWROT = 321,
-     UNIQUE_ID = 322,
-     PARENT_UNIQUE_ID = 323,
-     FUNCTION_ID = 324,
-     IMAGE = 325,
-     ID = 326,
-     NAME = 327,
-     DELAY_HANDOFF = 328,
-     DELAY_LOAD = 329,
-     COPY = 330,
-     INCLUDE = 331,
-     DELAY_AUTH = 332,
-     PARTITION = 333,
-     PFILE = 334,
-     METAHEADER = 335,
-     WORD = 336,
-     HEXWORD = 337,
-     FILENAME = 338,
-     QFILENAME = 339,
-     NONE = 340,
-     DECVALUE = 341,
-     HEXVALUE = 342,
-     KEYSRC_ENCRYPTION = 343,
-     FSBL_CONFIG = 344,
-     AUTH_PARAMS = 345,
-     AUTHJTAG_CONFIG = 346,
-     DEVICE_DNA = 347,
-     JTAG_TIMEOUT = 348,
-     PUF4KMODE = 349,
-     PUFROSWAP = 350,
-     SHUTTER = 351,
-     SPLIT = 352,
-     SMAP_WIDTH = 353,
-     PUF_HELPER_FILE = 354,
-     BH_KEY_FILE = 355,
-     BH_KEY_IV = 356,
-     BH_KEK_IV = 357,
-     BBRAM_KEK_IV = 358,
-     EFUSE_KEK_IV = 359,
-     EFUSE_USER_KEK0_IV = 360,
-     EFUSE_USER_KEK1_IV = 361,
-     USER_KEYS = 362,
-     PMCDATA = 363,
-     BOOTIMAGE = 364,
-     UDF_BH = 365,
-     INIT = 366,
-     PMUFW_IMAGE = 367,
-     AES_KEY_FILE = 368,
-     FAMILY_KEY = 369,
-     PPK_FILE = 370,
-     PSK_FILE = 371,
-     SPK_FILE = 372,
-     SSK_FILE = 373,
-     SPK_SIGNATURE_FILE = 374,
-     BH_SIGNATURE_FILE = 375,
-     HEADER_SIGNATURE_FILE = 376,
-     AUTHVALUE = 377,
-     ENCRVALUE = 378,
-     CHECKSUMVALUE = 379,
-     POWNERVALUE = 380,
-     PTYPEVALUE = 381,
-     KEY_SRC = 382,
-     CORE = 383,
-     BH_RSA = 384,
-     AUTH_HASH = 385,
-     INT_HASH = 386,
-     PUFHD_LOC = 387,
-     OPT_KEY = 388,
-     AUTH_ONLY = 389,
-     BOOT_DEVICE_TYPE = 390,
-     DEST_DEVICE_TYPE = 391,
-     DEST_CPU_TYPE = 392,
-     EXCEPTION_LEVEL_TYPE = 393,
-     TRUSTZONE_TYPE = 394,
-     SPLITMODE = 395,
-     DPA_CM = 396,
-     SPKSELECT = 397,
-     OR = 398,
-     XOR = 399,
-     AND = 400,
-     MULT = 401,
-     DIVIDE = 402,
-     MODULO = 403,
-     PLUS = 404,
-     MINUS = 405,
-     LSHIFT = 406,
-     RSHIFT = 407,
-     NEGATION = 408,
-     LPAREN = 409,
-     RPAREN = 410,
-     ASTERISK = 411
+     AC = 309,
+     BIF_SECTION = 310,
+     UDF_DATA = 311,
+     MCS = 312,
+     BIN = 313,
+     SLR_NUM = 314,
+     CLUSTER_NUM = 315,
+     DICE = 316,
+     PCR_NUMBER = 317,
+     PCR_MEASUREMENT_INDEX = 318,
+     IMAGE_STORE = 319,
+     PARENT_ID = 320,
+     ID_CODE = 321,
+     EXT_ID_CODE = 322,
+     BYPASS_IDCODE_CHECK = 323,
+     A_HWROT = 324,
+     S_HWROT = 325,
+     UNIQUE_ID = 326,
+     PARENT_UNIQUE_ID = 327,
+     FUNCTION_ID = 328,
+     IMAGE = 329,
+     ID = 330,
+     NAME = 331,
+     DELAY_HANDOFF = 332,
+     DELAY_LOAD = 333,
+     COPY = 334,
+     INCLUDE = 335,
+     DELAY_AUTH = 336,
+     PARTITION = 337,
+     PFILE = 338,
+     OPTIONAL_DATA = 339,
+     METAHEADER = 340,
+     TCM_BOOT = 341,
+     TCM_A_REGION = 342,
+     TCM_B_REGION = 343,
+     TCM_C_REGION = 344,
+     WORD = 345,
+     HEXWORD = 346,
+     FILENAME = 347,
+     QFILENAME = 348,
+     NONE = 349,
+     DECVALUE = 350,
+     HEXVALUE = 351,
+     KEYSRC_ENCRYPTION = 352,
+     FSBL_CONFIG = 353,
+     AUTH_PARAMS = 354,
+     AUTHJTAG_CONFIG = 355,
+     DEVICE_DNA = 356,
+     JTAG_TIMEOUT = 357,
+     PUF4KMODE = 358,
+     PUFROSWAP = 359,
+     SHUTTER = 360,
+     SPLIT = 361,
+     SMAP_WIDTH = 362,
+     PUF_HELPER_FILE = 363,
+     BH_KEY_FILE = 364,
+     BH_KEY_IV = 365,
+     BH_KEK_IV = 366,
+     BBRAM_KEK_IV = 367,
+     EFUSE_KEK_IV = 368,
+     EFUSE_USER_KEK0_IV = 369,
+     EFUSE_USER_KEK1_IV = 370,
+     USER_KEYS = 371,
+     PMCDATA = 372,
+     BOOTIMAGE = 373,
+     UDF_BH = 374,
+     INIT = 375,
+     PMUFW_IMAGE = 376,
+     AES_KEY_FILE = 377,
+     FAMILY_KEY = 378,
+     PPK_FILE = 379,
+     PSK_FILE = 380,
+     SPK_FILE = 381,
+     SSK_FILE = 382,
+     SPK_SIGNATURE_FILE = 383,
+     BH_SIGNATURE_FILE = 384,
+     HEADER_SIGNATURE_FILE = 385,
+     AUTHVALUE = 386,
+     ENCRVALUE = 387,
+     CHECKSUMVALUE = 388,
+     POWNERVALUE = 389,
+     PTYPEVALUE = 390,
+     KEY_SRC = 391,
+     CORE = 392,
+     BH_RSA = 393,
+     AUTH_HASH = 394,
+     INT_HASH = 395,
+     PUFHD_LOC = 396,
+     OPT_KEY = 397,
+     AUTH_ONLY = 398,
+     BOOT_DEVICE_TYPE = 399,
+     DEST_DEVICE_TYPE = 400,
+     DEST_CPU_TYPE = 401,
+     EXCEPTION_LEVEL_TYPE = 402,
+     TRUSTZONE_TYPE = 403,
+     SPLITMODE = 404,
+     DPA_CM = 405,
+     SPKSELECT = 406,
+     OR = 407,
+     XOR = 408,
+     AND = 409,
+     MULT = 410,
+     DIVIDE = 411,
+     MODULO = 412,
+     PLUS = 413,
+     MINUS = 414,
+     LSHIFT = 415,
+     RSHIFT = 416,
+     NEGATION = 417,
+     LPAREN = 418,
+     RPAREN = 419,
+     ASTERISK = 420
    };
 
     };
@@ -451,10 +460,10 @@ namespace BIF {
     Options& options;
   };
 /* Line 33 of lalr1.cc  */
-#line 23 "../s/bif.y"
+#line 24 "../s/bif.y"
 } // BIF
 /* Line 33 of lalr1.cc  */
-#line 458 "../bisonflex/bif.tab.hpp"
+#line 467 "../bisonflex/bif.tab.hpp"
 
 
 
