@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -96,6 +97,8 @@ BifOptions::BifOptions(Arch::Type architecture, const char* name)
     , pmcdataBlocks(0)
     , dice(DICE::DiceDisable)
     , slrNum(0xFF)
+    , pdiType(PartitionType::RESERVED)
+    , revokeId(0x00000000)
 {
     arch = architecture;
     SetGroupName(name);
@@ -131,6 +134,7 @@ PartitionBifOptions::PartitionBifOptions()
     , pmuFwImage(false)
     , pmcData(false)
     , presignFile("")
+    , acFile("")
     , udfDataFile("")
     , encryptType(Encryption::None)
     , authType(Authentication::None)
@@ -176,6 +180,12 @@ PartitionBifOptions::PartitionBifOptions()
     , updateReserveInPh(false)
     , bifSection("")
     , delayAuth(false)
+    , imageStoreId(0xFF)
+    , erasePartition(false)
+    , tcmBoot(false)
+    , tcmARegionAddr(0)
+    , tcmBRegionAddr(0)
+    , tcmCRegionAddr(0)
 { }
 
 /******************************************************************************/
@@ -266,12 +276,12 @@ void BifOptions::Add(PartitionBifOptions* currentPartitionBifOptions, ImageBifOp
         }
         if ((currentPartitionBifOptions)->aesKeyFile != "")
         {
-            pmcDataAesFile = (currentPartitionBifOptions)->aesKeyFile;
+            SetPmcDataAesFile((currentPartitionBifOptions)->aesKeyFile);
         }
         //If no key file found in partition specific attributes - Generate aeskeyfile with partition_name.nky
         else
         {
-            pmcDataAesFile = StringUtils::RemoveExtension(StringUtils::BaseName((currentPartitionBifOptions)->filename)) + ".nky";
+            SetPmcDataAesFile(StringUtils::RemoveExtension(StringUtils::BaseName((currentPartitionBifOptions)->filename)) + ".nky");
         }
 
         if(!(currentPartitionBifOptions->pmcData))
@@ -290,6 +300,7 @@ void BifOptions::Add(PartitionBifOptions* currentPartitionBifOptions, ImageBifOp
             if (currentImageBifOptions->GetImageType() == PartitionType::SLR_CONFIG)
             {
                 slrConfigCnt++;
+                currentImageBifOptions->slrConfigCnt++;
                 currentPartitionBifOptions->partitionType = PartitionType::SLR_CONFIG;
             }
             if (currentPartitionBifOptions->partitionType == PartitionType::RESERVED)
@@ -1136,6 +1147,12 @@ void BifOptions::SetMetaHeaderEncryptType(Encryption::Type type)
     metaHdrAttributes.encrypt = type;
 }
 
+/******************************************************************************/
+void BifOptions::SetMetaHeaderEncryptionKeyFile(std::string file)
+{
+    metaHdrAttributes.encrKeyFile = file;
+}
+
 /******************************************************************************/
 void BifOptions::SetMetaHeaderAuthType(Authentication::Type type)
 {
@@ -1439,6 +1456,22 @@ void PartitionBifOptions::SetClusterNum(uint8_t id)
     }
 }
 
+/******************************************************************************/
+void BifOptions::SetRevokeId(uint32_t id)
+{
+    if (id > 0xFF)
+    {
+        LOG_ERROR("revoke_id can only take values from 0x0 to 0xFF.");
+    }
+    revokeId = id;
+}
+
+/******************************************************************************/
+uint32_t BifOptions::GetRevokeId()
+{
+    return revokeId;
+}
+
 /******************************************************************************/
 void PartitionBifOptions::SetLockStepFlag()
 {
@@ -1452,6 +1485,45 @@ void PartitionBifOptions::SetLockStepFlag()
     }
 }
 
+/******************************************************************************/
+void PartitionBifOptions::SetTcmARegion(uint64_t address)
+{
+    if ((arch == Arch::VERSAL && versalNetSeries) && (destCPUType == DestinationCPU::R5_0 || destCPUType == DestinationCPU::R5_1 || destCPUType == DestinationCPU::R5_lockstep))
+    {
+        tcmARegionAddr = address;
+    }
+    else
+    {
+        LOG_ERROR("BIF attribute error !!!\n\t  'tcm_a_region' is supported only in VersalNet architecture");
+    }
+}
+
+/******************************************************************************/
+void PartitionBifOptions::SetTcmBRegion(uint64_t address)
+{
+    if ((arch == Arch::VERSAL && versalNetSeries) && (destCPUType == DestinationCPU::R5_0 || destCPUType == DestinationCPU::R5_1 || destCPUType == DestinationCPU::R5_lockstep))
+    {
+        tcmBRegionAddr = address;
+    }
+    else
+    {
+        LOG_ERROR("BIF attribute error !!!\n\t  'tcm_b_region' is supported only in VersalNet architecture");
+    }
+}
+
+/******************************************************************************/
+void PartitionBifOptions::SetTcmCRegion(uint64_t address)
+{
+    if ((arch == Arch::VERSAL && versalNetSeries) && (destCPUType == DestinationCPU::R5_0 || destCPUType == DestinationCPU::R5_1 || destCPUType == DestinationCPU::R5_lockstep))
+    {
+        tcmCRegionAddr = address;
+    }
+    else
+    {
+        LOG_ERROR("BIF attribute error !!!\n\t  'tcm_c_region' is supported only in VersalNet architecture");
+    }
+}
+
 /******************************************************************************/
 std::vector<uint32_t>& PartitionBifOptions::GetEncryptionBlocks(void)
 {
@@ -1469,11 +1541,11 @@ void PartitionBifOptions::SetAuthBlockAttr(size_t authBlockAttr)
 {
     if (authBlockAttr != 4 && authBlockAttr != 8 && authBlockAttr != 16 && authBlockAttr != 32 && authBlockAttr != 64)
     {
-        LOG_ERROR("'-authblock' option supports only 4,8,16,32,64 sizes (taken in MB)");
+        LOG_ERROR("'authblock' option supports only 4,8,16,32,64 sizes (taken in MB)");
     }
     if (arch != Arch::ZYNQMP)
     {
-        LOG_ERROR("BIF attribute error !!!\n\t\t'-authblock' option supported only for ZYNQMP architecture '-arch zynqmp'");
+        LOG_ERROR("BIF attribute error !!!\n\t\t'authblock' option supported only for ZYNQMP architecture '-arch zynqmp'");
     }
     authblockattr = authBlockAttr;
 }
@@ -1506,6 +1578,19 @@ void PartitionBifOptions::SetDelayAuth(bool flag)
     delayAuth = flag;
 }
 
+/******************************************************************************/
+void PartitionBifOptions::SetTcmBootFlag()
+{
+    if (arch == Arch::VERSAL && versalNetSeries)
+    {
+        tcmBoot = true;
+    }
+    else
+    {
+        LOG_ERROR("BIF attribute error !!!\n\t  'tcmboot' is supported only in VersalNet architecture");
+    }
+}
+
 /******************************************************************************/
 uint32_t PartitionBifOptions::GetDefaultEncryptionBlockSize(void)
 {
@@ -1599,6 +1684,12 @@ void BifOptions::SetHeaderAuthentication()
     LOG_TRACE("Setting Header Authentication");
 }
 
+/******************************************************************************/
+void BifOptions::SetSmapWidth(uint32_t value)
+{
+    smapWidth = value;
+}
+
 /******************************************************************************/
 std::string BifOptions::GetFamilyKeyFileName()
 {
@@ -1695,6 +1786,12 @@ std::vector<std::string> BifOptions::GetPmcCdoFileList(void)
     return pmcCdoFileList;
 }
 
+/******************************************************************************/
+void BifOptions::SetPmcDataAesFile(std::string filename)
+{
+    pmcDataAesFile = filename;
+}
+
 /******************************************************************************/
 std::string BifOptions::GetPmcDataAesFile(void)
 {
