@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -25,7 +26,7 @@
 -------------------------------------------------------------------------------
 */
 #include "options.h"
-#include "imageheadertable-versal.h"
+#include "imageheadertable.h"
 
 /* Forward Class References */
 class Options;
@@ -58,6 +59,7 @@ typedef struct
     Checksum::Type checksum;
     DpaCM::Type dpaCM;
     PufHdLoc::Type pufHdLoc;
+    std::vector<std::pair<std::string, uint32_t>> ihtOptionalDataInfo;
 } MetaHdrInfo;
 
 typedef struct
@@ -106,6 +108,10 @@ public:
     void SetPufHdLocation(PufHdLoc::Type);
     void SetReserveLength(uint64_t length, bool flag);
     void SetDelayAuth(bool flag);
+    void SetTcmBootFlag();
+    void SetTcmARegion(uint64_t tcm_a_region);
+    void SetTcmBRegion(uint64_t tcm_b_region);
+    void SetTcmCRegion(uint64_t tcm_c_region);
 
     std::string GetUdfDataFile(void);
     std::vector<uint32_t>& GetEncryptionBlocks(void);
@@ -148,6 +154,7 @@ public:
     bool pmuFwImage;
     bool pmcData;
     std::string presignFile;
+    std::string acFile;
     std::string udfDataFile;
     Encryption::Type encryptType;
     Authentication::Type authType;
@@ -162,6 +169,10 @@ public:
     PufHdLoc::Type pufHdLoc;
     uint8_t slrNum;
     uint8_t clusterNum;
+    bool tcmBoot;
+    uint64_t tcmARegionAddr;
+    uint64_t tcmBRegionAddr;
+    uint64_t tcmCRegionAddr;
 
     //no default declared
     Override<int> alignment;
@@ -169,11 +180,11 @@ public:
     Override<Binary::Length_t> reserve;
     Override<Binary::Address_t> load;
     Override<Binary::Address_t> startup;
-    Override<Binary::Address_t> pmcDataLoadAddress;
 
     bool bigEndian;
     bool a32Mode;
     uint32_t partitionId;
+    uint32_t imageStoreId;
     bool boot;
     bool user;
     bool Static;
@@ -204,6 +215,7 @@ public:
     bool updateReserveInPh;
     Arch::Type arch;
     bool delayAuth;
+    bool erasePartition;
 
 private:
     bool versalNetSeries;
@@ -223,6 +235,10 @@ public:
         , uniqueId(0xFFFFFFFF)
         , parentUniqueId(0xFFFFFFFF)
         , functionId(0xFFFFFFFF)
+        , pcrNumber(0xFFFF)
+        , pcrMeasurementIndex(0xFFFF)
+        , slrConfigCnt(0)
+        , slrConfigPartitionIndex(0)
     {
         partitionBifOptionsList.clear();
     }
@@ -258,6 +274,18 @@ public:
     {
         imageType = type;
     }
+    void SetPcrNumber(uint16_t num)
+    {
+        if ((2 > num) || (num > 7))
+        {
+            LOG_ERROR("'pcr' is specified as '%d' for the Image:'%s'\n\t   The permitted values range from 2 to 7.", num, GetImageName().c_str());
+        }
+        pcrNumber = num;
+    }
+    void SetPcrMeasurementIndex(uint16_t pcrIndex)
+    {
+        pcrMeasurementIndex = pcrIndex;
+    }
 
     uint32_t GetImageId(void)
     {
@@ -303,9 +331,20 @@ public:
     {
         return imageType;
     }
-
+    uint16_t GetPcrNumber(void)
+    {
+        return pcrNumber;
+    }
+    uint16_t GetPcrMeasurementIndex(void)
+    {
+        return pcrMeasurementIndex;
+    }
     std::list<PartitionBifOptions*> partitionBifOptionsList;
 
+    uint32_t slrConfigCnt;
+    uint32_t slrConfigPartitionIndex;
+    std::list<SlrPdiInfo*> slrConfigPdiInfo;
+
 private:
     uint32_t imageId;
     std::string imageName;
@@ -316,6 +355,8 @@ private:
     uint32_t uniqueId;
     uint32_t parentUniqueId;
     uint32_t functionId;
+    uint32_t pcrNumber;
+    uint32_t pcrMeasurementIndex;
 };
 
 /******************************************************************************/
@@ -362,6 +403,7 @@ public:
     void SetSPKSelection(uint32_t spkSelection);
     void SetSpkId(uint32_t spkId);
     void SetHeaderAuthentication();
+    void SetSmapWidth(uint32_t);
     void SetSplitMode(SplitMode::Type type);
     void SetSplitFmt(File::Type type);
     void SetPmcdataFile(const std::string & filename);
@@ -386,11 +428,14 @@ public:
     void SetCore(Core::Type type);
     void SetMetaHeaderEncryptionKeySource(KeySource::Type type, bool versalNetSeries);
     void SetMetaHeaderEncryptType(Encryption::Type type);
+    void SetMetaHeaderEncryptionKeyFile(std::string file);
     void SetMetaHeaderAuthType(Authentication::Type type);
     void SetPufHdinBHFlag();
     void SetAuthJtagRevokeID(uint32_t value);
     void SetAuthJtagDeviceDna(std::string string);
     void SetAuthJtagTimeOut(uint32_t value);
+    void SetPmcDataAesFile(std::string filename);
+    void SetRevokeId(uint32_t id);
 
     std::string GetGroupName(void);
     std::string GetAESKeyFileName(void);
@@ -466,10 +511,10 @@ public:
     std::list<ImageBifOptions*> imageBifOptionList;
     std::list<PartitionBifOptions*> partitionBifOptionList;
     Core::Type GetCore(void);
+    uint32_t GetRevokeId();
 
     //Versal
     Binary::Address_t pmcCdoLoadAddress;
-    std::string pmcDataAesFile;
     uint32_t idCode;
     uint32_t extendedIdCode;
     PartitionType::Type pdiType;
@@ -479,7 +524,6 @@ public:
     uint32_t pmcdataSize;
     uint32_t totalpmcdataSize;
     uint8_t* pmcDataBuffer;
-    uint32_t smapWidth;
     MetaHdrInfo metaHdrAttributes;
     AuthJtagInfo authJtagInfo;
     uint32_t slrBootCnt;
@@ -489,6 +533,7 @@ public:
     std::vector<uint32_t> pmcdataBlocks;
     uint8_t slrNum;
 private:
+    std::string pmcDataAesFile;
     std::string regInitFile;
     std::string udfBhFile;
     std::string pmuFwImageFile;
@@ -544,5 +589,7 @@ private:
     // Versal
     uint32_t pdiId;
     uint32_t parentId;
+    uint32_t revokeId;
+    uint32_t smapWidth;
 };
 #endif
