@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -23,7 +24,9 @@
 #include "bootheader-versal.h"
 #include "authentication-versal.h"
 
-#define PLM_MAX_SIZE            0x60000 //384KB
+#define PLM_MAX_SIZE               0x60000 //384KB
+#define PMCDATA_MAX_SIZE_VERSAL    0x14000 //80KB
+#define PMCDATA_MAX_SIZE_VERSALNET 0x10000 //64KB
 
 /*
 -------------------------------------------------------------------------------
@@ -118,6 +121,10 @@ void VersalBootHeader::Build(BootImage& bi, Binary& cache)
     /* If the boot header is imported from a bootimage file, no need to build */
     if (prebuilt)
     {
+        if (bi.bifOptions->GetRegInitFileName() != "")
+        {
+            BuildRegInitTable(bi.options);
+        }
         return;
     }
 
@@ -244,6 +251,14 @@ void VersalBootHeader::Link(BootImage& bi)
     {
         LOG_ERROR("Total PLM size : %dKB > maximum allowable size (384KB)", bHTable->totalPlmLength / 1024);
     }
+    if (bHTable->totalPmcCdoLength > PMCDATA_MAX_SIZE_VERSAL && !bi.options.IsVersalNetSeries())
+    {
+        LOG_ERROR("Total PMC DATA size : %dKB > maximum allowable size (80KB)", bHTable->totalPlmLength / 1024);
+    }
+    if (bHTable->totalPmcCdoLength > PMCDATA_MAX_SIZE_VERSALNET && bi.options.IsVersalNetSeries())
+    {
+        LOG_ERROR("Total PMC DATA size : %dKB > maximum allowable size (64KB)", bHTable->totalPlmLength / 1024);
+    }
 }
 
 /******************************************************************************/
@@ -268,23 +283,26 @@ void VersalBootHeader::LinkPrebuiltBH(BootImage& bi)
         if (!(presignedLocal))
         {
             uint32_t acSize = sizeof(AuthCertificate4096Sha3PaddingStructure);
-            bHTable->totalPlmLength += acSize;
             bHTable->sourceOffset += acSize;
-
-            /* The AC should be appended to a 64-byte aligned FSBL + PMU partition
-               So find the pad length required for alignment and then add AC size */
-            //uint32_t padLength = (64 - (((bHTable->totalFsblLength + bHTable->totalPmuFwLength) & 63) & 63));
-            Binary::Length_t shaPadOnLength = bHTable->totalPlmLength + bHTable->totalPmcCdoLength - SIGN_LENGTH_VERSAL;
-            uint8_t shaPadLength = SHA3_PAD_LENGTH - (shaPadOnLength % SHA3_PAD_LENGTH);
-            if (bHTable->totalPmcCdoLength == 0)
+            if (bi.options.IsVersalNetSeries())
             {
-                bHTable->totalPlmLength += shaPadLength;
+                SetTotalPlmLength(bi.GetTotalFsblFwSize());
+                SetTotalPmcCdoLength(bi.bifOptions->GetTotalPmcFwSize());
             }
             else
             {
-                bHTable->totalPmcCdoLength += shaPadLength;
+                bHTable->totalPlmLength += acSize;
+                Binary::Length_t shaPadOnLength = bHTable->totalPlmLength + bHTable->totalPmcCdoLength - SIGN_LENGTH_VERSAL;
+                uint8_t shaPadLength = SHA3_PAD_LENGTH - (shaPadOnLength % SHA3_PAD_LENGTH);
+                if (bHTable->totalPmcCdoLength == 0)
+                {
+                    bHTable->totalPlmLength += shaPadLength;
+                }
+                else
+                {
+                    bHTable->totalPmcCdoLength += shaPadLength;
+                }
             }
-
             SetHeaderChecksum();
         }
         bi.SetTotalFsblFwSize(bHTable->totalPlmLength);
