@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -44,6 +45,8 @@
 ------------------------------------------------------------------------------
 */
 
+std::vector<std::string> BootImage::encryptionKeyFileVec;
+
 /******************************************************************************/
 void BIF_File::Process(Options& options)
 {
@@ -85,24 +88,30 @@ void BIF_File::Process(Options& options)
             }
         }
 
-        ReplaceFiles();
+        AppendAndReplaceFilesinBIF();
         options.bifOptionsList = bifOptionList = includeBifOptionList;
     }
 
     BootImage* currentbi = NULL;
     uint8_t index = 0;
+    size_t itr = 0;
+    std::string lastPmcDataAesFile = bifOptionList.back()->GetPmcDataAesFile();
 
-    for (std::vector<BifOptions*>::iterator bifoptions = bifOptionList.begin(); bifoptions != bifOptionList.end(); bifoptions++)
+    for (std::vector<BifOptions*>::iterator bifoptions = bifOptionList.begin(); bifoptions != bifOptionList.end(); bifoptions++, itr++)
     {
         if (((*bifoptions)->slrBootCnt == 0) && ((*bifoptions)->slrConfigCnt == 0) && options.IsSsitBif())
         {
             (*bifoptions)->pdiType = PartitionType::SLR_SLAVE;
-            if ((*bifoptions)->smapWidth != 0)
+            if ((*bifoptions)->GetSmapWidth() != 0)
             {
-                LOG_WARNING("smap_width for SSIT SLAVE PDIs is identified as %d. Setting it to the default value : 0.", (*bifoptions)->smapWidth);
-                (*bifoptions)->smapWidth = 0;
+                LOG_WARNING("smap_width for SSIT SLAVE PDIs is identified as %d. Setting it to the default value '0'.", (*bifoptions)->GetSmapWidth());
+                (*bifoptions)->SetSmapWidth(0);
             }
         }
+        else if ((*bifoptions)->pdiType == PartitionType::SLR_SLAVE_CONFIG)
+        {
+            (*bifoptions)->SetSmapWidth(0);
+        }
 
         if (options.archType == Arch::ZYNQMP)
         {
@@ -111,6 +120,11 @@ void BIF_File::Process(Options& options)
         else if (options.archType == Arch::VERSAL)
         {
             currentbi = new VersalBootImage(options, index);
+            currentbi->pmcDataAesFile = (*bifoptions)->GetPmcDataAesFile();
+            if ((itr+1) == bifOptionList.size())
+            {
+                currentbi->pmcDataAesFile = lastPmcDataAesFile;
+            }
         }
         else
         {
@@ -120,13 +134,19 @@ void BIF_File::Process(Options& options)
         if (currentbi != NULL)
         {
             currentbi->Add(*bifoptions);
-            if (currentbi->bootloaderFound && (*bifoptions)->pdiType == PartitionType::SLR_SLAVE)
+            if (((*bifoptions)->pdiType != PartitionType::SLR_SLAVE_BOOT) && ((*bifoptions)->pdiType != PartitionType::SLR_SLAVE_CONFIG))
             {
-                (*bifoptions)->pdiType = PartitionType::SLR_SLAVE_BOOT;
-            }
-            else
-            {
-                (*bifoptions)->pdiType = PartitionType::SLR_SLAVE_CONFIG;
+                if ((*bifoptions)->pdiType == PartitionType::SLR_SLAVE)
+                {
+                    if (currentbi->bootloaderFound)
+                    {
+                        (*bifoptions)->pdiType = PartitionType::SLR_SLAVE_BOOT;
+                    }
+                    else
+                    {
+                        (*bifoptions)->pdiType = PartitionType::SLR_SLAVE_CONFIG;
+                    }
+                }
             }
             bootImages.push_back(currentbi);
         }
@@ -140,6 +160,7 @@ void BIF_File::Output(Options& options, uint8_t index)
 {
     File::Type splitFileType = options.GetSplitType();
     std::list<std::string> outFilename = options.GetOutputFileNames();
+    std::string fName = "";
     BootImage* bi = bootImages.at(index);
     {
         if (options.GetAuthKeyGeneration() != GenAuthKeys::None)
@@ -168,17 +189,36 @@ void BIF_File::Output(Options& options, uint8_t index)
         OutputMode::Type outputMode = options.GetOutputMode();
         bi->ValidateOutputModes(splitFileType, outputMode);
 
+        if (options.GetOutType() != File::Unknown)
+        {
+            if (!outFilename.empty())
+            {
+                std::string outFile = outFilename.front();
+                fName = StringUtils::RemoveExtension(outFile);
+                outFilename.clear();
+            }
+            else
+            {
+                fName = StringUtils::RemoveExtension(biffilename);
+            }
+            fName += (options.GetOutType() == File::MCS) ? ".mcs" : ".bin";
+
+        }
+
         if (outputMode == OutputMode::OUT_SPLIT_NORMAL || outputMode == OutputMode::OUT_SPLIT_SLAVEBOOT || splitFileType != File::Unknown)
         {
             outFilename.clear();
-            std::string fName = StringUtils::RemoveExtension(biffilename);
+            if (options.GetOutType() != File::Unknown)
+            {
+                fName.clear();
+            }    
+            fName = StringUtils::RemoveExtension(biffilename);
             fName = fName.substr(fName.find_last_of("\\")+1, std::string::npos);
             if(splitFileType == File::Unknown)
             {
                 splitFileType = options.GetOutputFormat();
             }
             fName += (splitFileType == File::MCS) ? ".mcs" : ".bin";
-            outFilename.push_back(fName);
         }
 
         /* Process Bitstream Mode
@@ -186,13 +226,17 @@ void BIF_File::Output(Options& options, uint8_t index)
         if (outputMode == OutputMode::OUT_BITSTREAM) 
         {
             outFilename.clear();
-            std::string fName = bi->bitFilename;
+            fName = bi->bitFilename;
             if(bi->bitFilename.empty())
             {
                 LOG_ERROR("No bitstream in the BIF file - %s ",options.GetBifFilename().c_str());
             }
             fName += (options.GetProcessBitstreamType() == File::MCS) ? ".mcs" : ".bin";
-            outFilename.push_back(fName);
+        }
+
+        if (fName != "")
+        {
+           outFilename.push_back(fName);
         }
 
         bi->OutputPartitionFiles(options, *bi->cache);
@@ -266,8 +310,15 @@ BootImage::BootImage(Options& options, uint8_t index)
     , firstOptKey(NULL)
     , overlayCDO(NULL)
     , globalSlrId(0)
+    , iht_optional_data(NULL)
+    , iht_optional_data_length(0)
+    , pmcDataAesFile("")
+    , xplm_modules_data_length(0)
+    , xplm_modules_data(NULL)
+    , authOnPartitionFound(false)
+    , numHashTableEntries(0)
 {
-    bifOptions = options.bifOptionsList.at(index);
+    options.bifOptions = bifOptions = options.bifOptionsList.at(index);
     Name = bifOptions->GetGroupName();
     cache = new Binary();
     checksumTable = new ChecksumTable();
@@ -670,13 +721,13 @@ void BootImage::SetAssumeEncryptionFlag(bool flag)
 /******************************************************************************/
 std::vector<std::string>& BootImage::GetEncryptionKeyFileVec()
 { 
-    return encryptionKeyFileVec; 
+    return encryptionKeyFileVec;
 }
 
 /******************************************************************************/
 void BootImage::InsertEncryptionKeyFile(std::string filename)
 {
-    encryptionKeyFileVec.push_back(filename); 
+    encryptionKeyFileVec.push_back(filename);
 }
 
 /******************************************************************************/
@@ -710,7 +761,7 @@ void BIF_File::ParseBifFile(Options& options)
 }
 
 /******************************************************************************/
-void BIF_File::ReplaceFiles()
+void BIF_File::AppendAndReplaceFilesinBIF()
 {
     /* If slr number matches between partitions - replace the files */
     for (size_t i = 0; i < includeBifOptionList.size(); i++)
@@ -734,6 +785,7 @@ void BIF_File::ReplaceFiles()
 
                                 (*itr2)->filename = (*itr1)->filename;
                                 (*itr2)->filelist = (*itr1)->filelist;
+                                (*itr1)->erasePartition = true;
                                 break;
                             }
                         }
@@ -757,6 +809,7 @@ void BIF_File::ReplaceFiles()
                                 includeBifOptionList[i]->ClearPmcCdoFileList();
                                 for (size_t listSize = 0; listSize < (*itr1)->filelist.size(); listSize++)
                                     includeBifOptionList[i]->SetPmcCdoFileList((*itr1)->filelist[listSize]);
+                                (*itr1)->erasePartition = true;
                                 break;
                             }
                         }
@@ -775,12 +828,65 @@ void BIF_File::ReplaceFiles()
 
                                 (*itr2)->filename = (*itr1)->filename;
                                 (*itr2)->filelist = (*itr1)->filelist;
+                                (*itr1)->erasePartition = true;
                                 break;
                             }
                         }
                     }
                 }
             }
+            for (std::list<ImageBifOptions*>::iterator itr1 = bifOptionList[j]->imageBifOptionList.begin(); itr1 != bifOptionList[j]->imageBifOptionList.end(); )
+            {
+                bool incremented = false;
+                for (std::list<PartitionBifOptions*>::iterator itr2 = (*itr1)->partitionBifOptionsList.begin(); itr2 != (*itr1)->partitionBifOptionsList.end(); )
+                {
+                    if ((*itr2)->erasePartition)
+                    {
+                        (*itr1)->partitionBifOptionsList.remove(*itr2++);
+                        if ((*itr1)->partitionBifOptionsList.size() == 0)
+                        {
+                            bifOptionList[j]->imageBifOptionList.remove(*itr1++);
+                            incremented = true;
+                            break;
+                        }
+                        //break;
+                    }
+                    else
+                    {
+                        itr2++;
+                    }
+                }
+                if(!incremented)
+                    itr1++;
+            }
+        }
+    }
+
+    for (size_t i = 0; i < includeBifOptionList.size(); i++)
+    {
+        if ((includeBifOptionList[i]->pdiType == PartitionType::SLR_CONFIG) || (includeBifOptionList[i]->slrNum == 0))
+        {
+            for (size_t j = 0; j < bifOptionList.size(); j++)
+            {
+                for (std::list<ImageBifOptions*>::iterator itr1 = bifOptionList[j]->imageBifOptionList.begin(); itr1 != bifOptionList[j]->imageBifOptionList.end(); itr1++)
+                {
+                    std::list<PartitionBifOptions*>::iterator itr2 = (*itr1)->partitionBifOptionsList.begin();
+                    {
+                        if (includeBifOptionList[i]->slrNum == (*itr2)->slrNum)
+                        {
+                            includeBifOptionList[i]->imageBifOptionList.push_back((*itr1));
+                            includeBifOptionList[i]->partitionBifOptionList.insert(includeBifOptionList[i]->partitionBifOptionList.end(), (*itr1)->partitionBifOptionsList.begin(), (*itr1)->partitionBifOptionsList.end());
+                        }
+                    }
+                }
+            }
+        }
+        else if(includeBifOptionList[i]->slrNum == 0xFF)
+        {
+            for (size_t j = 0; j < bifOptionList.size(); j++)
+            {
+                includeBifOptionList[i]->imageBifOptionList.splice(includeBifOptionList[i]->imageBifOptionList.end(), bifOptionList[j]->imageBifOptionList);
+            }
         }
     }
 }
