@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2019-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -24,6 +25,7 @@
 static uint32_t verbose;
 static uint32_t auto_align;
 static uint32_t add_offset;
+uint32_t id_code_binary;
 
 #define u32xe(v) (be ? u32be(v) : u32le(v))
 #define u32xe_lo(v) u32xe((uint32_t)(v))
@@ -108,6 +110,14 @@ enum {
     CMD2_PLM_UPDATE	 = 0x120U,
     CMD2_SCATTER_WRITE	 = 0x121U,
     CMD2_SCATTER_WRITE2	 = 0x122U,
+    CMD2_TAMPER_TRIGGER  = 0x123U,
+    CMD2_SET_IPI_ACCESS  = 0x125U,
+    CMD2_RUN_PROC	 = 0x126U,
+    CMD2_LIST_SET	 = 0x127U,
+    CMD2_LIST_WRITE	 = 0x128U,
+    CMD2_LIST_MASK_WRITE = 0x129U,
+    CMD2_LIST_MASK_POLL  = 0x12AU,
+    CMD2_CDO_SEQUENCE    = 0x12CU,
 
     /* PM Commands */
     CMD2_PM_GET_API_VERSION	= 0x201U,
@@ -165,14 +175,22 @@ enum {
     CMD2_PM_APPLY_TRIM = 0x244,
     CMD2_PM_NOC_CLOCK_ENABLE	= 0x245,
     CMD2_PM_IF_NOC_CLOCK_ENABLE	= 0x246,
+    CMD2_PM_FORCE_HC =0x247,
+    CMD2_PM_HNICX_NPI_DATA_XFER	= 0x24AU,
 
     /* NPI Commands */
+    /* Version 1.50 and later, but not version 2.00 */
     CMD2_NPI_SEQ	 = 0x301U,
     CMD2_NPI_PRECFG	 = 0x302U,
     CMD2_NPI_WRITE	 = 0x303U,
     CMD2_NPI_SHUTDN	 = 0x304U,
 
+    /* SEM Commands */
+    /* Version 2.00 and later */
+    CMD2_SEM_NPI_TABLE	 = 0x300U,
+
     /* CFU Commands */
+    /* Version 1.50 and later, but not version 2.00 */
     CMD2_CFU_SET_CRC32	 = 0x401U,
     CMD2_CFU_DECOMPRESS	 = 0x402U,
     CMD2_CFU_CRAM_RW	 = 0x403U,
@@ -193,6 +211,69 @@ enum {
     CMD2_END             = 0xffffffffU
 };
 
+struct cdo_device {
+    const char* name;
+    uint32_t idcode;
+} binary_device_list[] = {
+    { "xcvm1402", 0x04C08093 },
+    { "xcvm1302", 0x04C09093 },
+    { "xcv20", 0x04C0F093 },
+    { "xcvp1052", 0x04C18093 },
+    { "xcvp1002", 0x04C1B093 },
+    { "xcvm2102", 0x04C1C093 },
+    { "xcvp1402", 0x04C20093 },
+    { "xcvp1102", 0x04C22093 },
+    { "xcvm2902", 0x04C23093 },
+    { "xcvm2302", 0x04C24093 },
+    { "xcvr1352", 0x04C90093 },
+    { "xcvr1302", 0x04C91093 },
+    { "xcvc1352", 0x04C93093 },
+    { "xcvr1402", 0x04C94093 },
+    { "xcvc1702", 0x04C98093 },
+    { "xcvm1502", 0x04C99093 },
+    { "xcve1752", 0x04C9A093 },
+    { "xcvc1502", 0x04C9B093 },
+    { "xcvr1702", 0x04CA0093 },
+    { "xcvr1502", 0x04CA2093 },
+    { "xcvr1602", 0x04CA3093 },
+    { "xcvr1802", 0x04CA5093 },
+    { "xcvc1802", 0x14CA9093 },
+    { "xcvm1802", 0x14CAA093 },
+    { "xcv65", 0x14CAF093 },
+    { "xcve2102", 0x04CC0093 },
+    { "xcve2002", 0x04CC1093 },
+    { "xcve2302", 0x14CC8093 },
+    { "xcve2202", 0x14CC9093 },
+    { "xcvm1102", 0x14CCA093 },
+    { "xcvc2802", 0x14CD0093 },
+    { "xcvc2602", 0x14CD1093 },
+    { "xcve2602", 0x14CD2093 },
+    { "xcve2802", 0x14CD3093 },
+    { "xcvm2202", 0x14CD4093 },
+    { "xcv70", 0x14CD7093 },
+    { "xcvp1202", 0x14D00093 },
+    { "xcvm2502", 0x14D01093 },
+    { "xcvp1502", 0x14D08093 },
+    { "xcvp1702", 0x14D10093 },
+    { "xcvp1802", 0x14D14093 },
+    { "xcvp2502", 0x14D1C093 },
+    { "xcvp2802", 0x14D20093 },
+    { "xcvh1582", 0x14D28093 },
+    { "xcvh1542", 0x14D29093 },
+    { "xcvh1522", 0x14D2A093 },
+    { "xcvh1782", 0x14D2C093 },
+    { "xcvh1742", 0x14D2D093 },
+    { "xcv80", 0x14D2F093 },
+    { "xcvp1552", 0x14D34093 },
+    { "xcvp1752", 0x14D38093 },
+    { "xcvn3716", 0x14D80093 },
+    { "xcvn3516", 0x14D81093 },
+    { "xcvn3408", 0x14D82093 },
+    { "xcvn3708", 0x14D83093 },
+    { "xcvc1902", 0x14CA8093 },
+    { NULL, 0 }
+};
+
 void cdobinary_set_verbose(uint32_t level) {
     verbose = level;
 }
@@ -214,6 +295,25 @@ static void byte_swap_buffer(uint32_t * p, uint32_t count, uint32_t be) {
     }
 }
 
+uint32_t idcode_from_binary(uint32_t id)
+{
+    if (id_code_binary != 0)
+    {
+        id = id_code_binary;
+        id_code_binary = 0;
+    }
+    return id;
+}
+
+static uint32_t  find_device(char * name) {
+    struct cdo_device * device = binary_device_list;
+    while (device->name != NULL) {
+        if (strncmp(device->name, name, sizeof(char*)) == 0) break;
+        device++;
+    }
+    return device->idcode;
+}
+
 static uint32_t decode_v1(CdoSequence * seq, uint32_t * p, uint32_t l, uint32_t be) {
     uint32_t i = u32xe(p[0]) + 1;
     uint32_t sections = u32xe(p[3]);
@@ -432,6 +532,8 @@ static uint32_t decode_v2_cmd(CdoSequence * seq, uint32_t * p, uint32_t * ip, ui
                 cdocmd_add_mask_poll(seq, u32xe(p[i+0]), u32xe(p[i+1]), u32xe(p[i+2]), u32xe(p[i+3]), 0);
             } else if (args == 5) {
                 cdocmd_add_mask_poll(seq, u32xe(p[i+0]), u32xe(p[i+1]), u32xe(p[i+2]), u32xe(p[i+3]), u32xe(p[i+4]));
+            } else if (args == 6) {
+                cdocmd_add_mask_poll_w_err(seq, u32xe(p[i+0]), u32xe(p[i+1]), u32xe(p[i+2]), u32xe(p[i+3]), u32xe(p[i+4]), u32xe(p[i+5]));
             } else {
                 goto unexpected;
             }
@@ -444,6 +546,31 @@ static uint32_t decode_v2_cmd(CdoSequence * seq, uint32_t * p, uint32_t * ip, ui
             if (args != 2) goto unexpected;
             cdocmd_add_write(seq, u32xe(p[i+0]), u32xe(p[i+1]));
             break;
+	case CMD2_RUN_PROC:
+		if (args != 1) goto unexpected;
+		cdocmd_add_run_proc(seq, u32xe(p[i+0]));
+		break;
+	case CMD2_LIST_SET:
+		if (args < 2) goto unexpected;
+		cdocmd_add_list_set(seq, u32xe(p[i+0]), args - 1, &p[i+1], be);
+		break;
+	case CMD2_LIST_WRITE:
+		if (args != 2) goto unexpected;
+		cdocmd_add_list_write(seq, u32xe(p[i+0]), u32xe(p[i+1]));
+		break;
+	case CMD2_LIST_MASK_WRITE:
+		if (args != 3) goto unexpected;
+		cdocmd_add_list_mask_write(seq, u32xe(p[i+0]), u32xe(p[i+1]), u32xe(p[i+2]));
+		break;
+	case CMD2_LIST_MASK_POLL:
+            if (args == 4) {
+                cdocmd_add_list_mask_poll(seq, u32xe(p[i+0]), u32xe(p[i+1]), u32xe(p[i+2]), u32xe(p[i+3]), 0);
+            } else if (args == 5) {
+                cdocmd_add_list_mask_poll(seq, u32xe(p[i+0]), u32xe(p[i+1]), u32xe(p[i+2]), u32xe(p[i+3]), u32xe(p[i+4]));
+            } else {
+                goto unexpected;
+            }
+            break;
         case CMD2_DELAY:
             if (args != 1) goto unexpected;
             cdocmd_add_delay(seq, u32xe(p[i+0]));
@@ -473,6 +600,8 @@ static uint32_t decode_v2_cmd(CdoSequence * seq, uint32_t * p, uint32_t * ip, ui
                 cdocmd_add_mask_poll(seq, u32pair(u32xe(p[i+0]), u32xe(p[i+1])), u32xe(p[i+2]), u32xe(p[i+3]), u32xe(p[i+4]), 0);
             } else if (args == 6) {
                 cdocmd_add_mask_poll(seq, u32pair(u32xe(p[i+0]), u32xe(p[i+1])), u32xe(p[i+2]), u32xe(p[i+3]), u32xe(p[i+4]), u32xe(p[i+5]));
+            } else if (args == 7) {
+                cdocmd_add_mask_poll_w_err(seq, u32pair(u32xe(p[i+0]), u32xe(p[i+1])), u32xe(p[i+2]), u32xe(p[i+3]), u32xe(p[i+4]), u32xe(p[i+5]), u32xe(p[i+6]));
             } else {
                 goto unexpected;
             }
@@ -592,6 +721,16 @@ static uint32_t decode_v2_cmd(CdoSequence * seq, uint32_t * p, uint32_t * ip, ui
             cdocmd_add_end(seq);
             break;
         }
+        case CMD2_CDO_SEQUENCE: {
+            uint32_t i2 = i + 3;
+            if (args < 3) goto unexpected;
+            cdocmd_add_cdo_sequence(seq, u32pair(u32xe(p[i+0]), u32xe(p[i+1])), u32xe(p[i+2]));
+            if (decode_v2_cmd(seq, p, &i2, i2 + args - 3, be)) {
+                goto error;
+            }
+            cdocmd_add_end(seq);
+            break;
+        }
         case CMD2_SCATTER_WRITE:
             if (args < 2) goto unexpected;
             cdocmd_add_scatter_write(seq, u32xe(p[i+0]), args - 1, &p[i+1], be);
@@ -601,22 +740,40 @@ static uint32_t decode_v2_cmd(CdoSequence * seq, uint32_t * p, uint32_t * ip, ui
             cdocmd_add_scatter_write2(seq, u32xe(p[i+0]), u32xe(p[i+1]), args - 2, &p[i+2], be);
             break;
 
+        case CMD2_TAMPER_TRIGGER:
+            if (args != 1) goto unexpected;
+            cdocmd_add_tamper_trigger(seq, u32xe(p[i+0]));
+            break;
+
+        case CMD2_SET_IPI_ACCESS:
+            if (args != 2) goto unexpected;
+            cdocmd_add_set_ipi_access(seq, u32xe(p[i+0]), u32xe(p[i+1]));
+            break;
+
         case CMD2_NPI_SEQ:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args != 2) goto unexpected;
             cdocmd_add_npi_seq(seq, u32xe(p[i+0]), u32xe(p[i+1]));
             break;
         case CMD2_NPI_PRECFG:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args != 2) goto unexpected;
             cdocmd_add_npi_precfg(seq, u32xe(p[i+0]), u32xe(p[i+1]));
             break;
         case CMD2_NPI_WRITE:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args < 2) goto unexpected;
             cdocmd_add_npi_write(seq, u32xe(p[i+0]), u32xe(p[i+1]), args - 2, &p[i+2], be);
             break;
         case CMD2_NPI_SHUTDN:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args != 2) goto unexpected;
             cdocmd_add_npi_shutdown(seq, u32xe(p[i+0]), u32xe(p[i+1]));
             break;
+	case CMD2_PM_HNICX_NPI_DATA_XFER:
+		if (args != 2) goto unexpected;
+		cdocmd_add_pm_hnicx_npi_data_xfer(seq, u32xe(p[i+0]), u32xe(p[i+1]));
+		break;
         case CMD2_PM_GET_API_VERSION:
             if (args != 0) goto unexpected;
             cdocmd_add_pm_get_api_version(seq);
@@ -840,8 +997,13 @@ static uint32_t decode_v2_cmd(CdoSequence * seq, uint32_t * p, uint32_t * ip, ui
             cdocmd_add_pm_if_noc_clock_enable(seq, u32xe(p[i+0]), u32xe(p[i+1]),
                                               args == 3 ? u32xe(p[i+2]) : 1);
             break;
+        case CMD2_PM_FORCE_HC:
+            if (args != 1) goto unexpected;
+            cdocmd_add_pm_force_hc(seq, u32xe(p[i+0]));
+            break;
 
         case CMD2_CFU_SET_CRC32:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args < 1) goto unexpected;
             if (u32xe(p[i+0]) == 0) {
                 cdocmd_add_cfu_set_crc32(seq, 0, 0);
@@ -851,34 +1013,42 @@ static uint32_t decode_v2_cmd(CdoSequence * seq, uint32_t * p, uint32_t * ip, ui
             cdocmd_add_cfu_set_crc32(seq, u32xe(p[i+0]), u32xe(p[i+1]));
             break;
         case CMD2_CFU_DECOMPRESS:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args < 1) goto unexpected;
             cdocmd_add_cfu_decompress(seq, u32xe(p[i+0]));
             break;
         case CMD2_CFU_CRAM_RW:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args < 1) goto unexpected;
             cdocmd_add_cfu_cram_rw(seq, u32xe(p[i+0]));
             break;
         case CMD2_CFU_SEU_GO:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args < 1) goto unexpected;
             cdocmd_add_cfu_seu_go(seq, u32xe(p[i+0]));
             break;
         case CMD2_CFU_CRC8_DIS:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args < 1) goto unexpected;
             cdocmd_add_cfu_crc8_dis(seq, u32xe(p[i+0]));
             break;
         case CMD2_CFU_SSI_PER_SLR_PR:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args < 1) goto unexpected;
             cdocmd_add_cfu_ssi_per_slr_pr(seq, u32xe(p[i+0]));
             break;
         case CMD2_CFU_GSR_GSC:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args < 1) goto unexpected;
             cdocmd_add_cfu_gsr_gsc(seq, u32xe(p[i+0]));
             break;
         case CMD2_CFU_GCAP_CLK_EN:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args < 1) goto unexpected;
             cdocmd_add_cfu_gcap_clk_en(seq, u32xe(p[i+0]));
             break;
         case CMD2_CFU_CFI_TYPE:
+            if (seq->version >= CDO_VERSION_2_00) goto unexpected;
             if (args < 1) goto unexpected;
             cdocmd_add_cfu_cfi_type(seq, u32xe(p[i+0]));
             break;
@@ -894,6 +1064,12 @@ static uint32_t decode_v2_cmd(CdoSequence * seq, uint32_t * p, uint32_t * ip, ui
             if (args != 1) goto unexpected;
             cdocmd_add_ldr_cframe_clear_check(seq, u32xe(p[i+0]));
             break;
+
+        case CMD2_SEM_NPI_TABLE:
+            if (args < 2) goto unexpected;
+            cdocmd_add_sem_npi_table(seq, u32xe(p[i+0]), u32xe(p[i+1]), args - 2, &p[i+2], be);
+            break;
+
         default:
             cdocmd_add_generic_command(seq, hdr & 0xffff, p + i, args, be);
             break;
@@ -911,7 +1087,7 @@ error:
     return 1;
 }
 
-uint32_t check_cdo_commands(void* data, uint32_t l, uint32_t * xplm_data, uint32_t xplm_length) {
+int check_cdo_commands(void* data, uint32_t l, uint32_t * xplm_data, uint32_t xplm_length) {
     uint32_t be = 0;
     uint32_t * p = (uint32_t *)data;
     uint32_t hdrlen = u32xe(p[0]);
@@ -943,31 +1119,34 @@ uint32_t check_cdo_commands(void* data, uint32_t l, uint32_t * xplm_data, uint32
             if (hdr == CMD2_END) goto found_end_marker;
             if (i >= l) {
                 fprintf(stderr, "incomplete CDO command %#"PRIx32" at %"PRIu32"\n", hdr, cmdpos);
-                return -1;
             }
             args = u32xe(p[i++]);
         }
         if (l - i < args) {
             fprintf(stderr, "incomplete CDO command %#"PRIx32" at %"PRIu32"\n", hdr, cmdpos);
-            return -1;
         }
         cmd_id = hdr & 0xff;
         module_id = (hdr & 0xff00) >> 0x8;
         // TODO: only PLM commands are supported for now
-        if (module_id != 0x1) continue;
-        cmd_found = 0;
-        size_t cmdInfo_size = sizeof(XPlmi_CmdInfo);
-        for (index = 0; index < (xplm_length/ cmdInfo_size); index++) {
-            uint8_t* cmdInfo = (uint8_t*)malloc(cmdInfo_size);
-            memset(cmdInfo, 0, cmdInfo_size);
-            memcpy(cmdInfo, xplm_data + (index * (cmdInfo_size/4)), cmdInfo_size);
-            XPlmi_CmdInfo* cmd_info = (XPlmi_CmdInfo*)(cmdInfo);
-            if (cmd_info->cmd_id == cmd_id && cmd_info->module_id == module_id && cmd_info->min_arg_cnt >= args && cmd_info->max_arg_cnt <= args) {
-                cmd_found = 1;
-                break;
+        if (module_id == 0x1 && xplm_length != 0){
+           cmd_found = 0;
+           size_t cmdInfo_size = sizeof(XPlmi_CmdInfo);
+           for (index = 0; index < (xplm_length/ cmdInfo_size); index++) {
+               uint8_t* cmdInfo = (uint8_t*)malloc(cmdInfo_size);
+               memset(cmdInfo, 0, cmdInfo_size);
+               memcpy(cmdInfo, xplm_data + (index * (cmdInfo_size/4)), cmdInfo_size);
+               XPlmi_CmdInfo* cmd_info = (XPlmi_CmdInfo*)(cmdInfo);
+               if (cmd_info->cmd_id == cmd_id && cmd_info->module_id == module_id && args >= cmd_info->min_arg_cnt && args <= cmd_info->max_arg_cnt) {
+                  cmd_found = 1;
+                  break;
+               }
+            }
+            if (cmd_found == 0 && cmd_id != 5 && cmd_id != 13 && cmd_id != 0)
+            {
+               return 1;
             }
         }
-        if (cmd_found == 0) return -1;
+        i += args;
     }
 found_end_marker:
 return 0;
@@ -1056,13 +1235,13 @@ CdoSequence * decode_cdo_binary(const void * data, size_t size) {
     }
     identification = u32xe(p[1]);
     version = u32xe(p[2]);
-    if (version < 0x200) {
+    if (version < CDO_VERSION_1_50) {
         if (identification != 0x584C4E58) {
             fprintf(stderr, "invalid identification word %#"PRIx32"\n", identification);
             return NULL;
         }
         decode = decode_v1;
-    } else if (version < 0x300) {
+    } else if (version < CDO_VERSION_3_00) {
         if (identification != 0x004F4443) {
             fprintf(stderr, "invalid identification word %#"PRIx32"\n", identification);
             return NULL;
@@ -1342,10 +1521,10 @@ static void * encode_v1(CdoSequence * seq, size_t * sizep, uint32_t be) {
     return p;
 }
 
-static void hdr2(uint32_t ** bufp, uint32_t * posp, uint32_t cmd, uint32_t len, uint32_t be) {
+static void hdr2l(uint32_t ** bufp, uint32_t * posp, uint32_t cmd, uint32_t len, uint32_t be, uint32_t long_header) {
     uint32_t * p = *bufp;
     uint32_t pos = *posp;
-    if (len < 255) {
+    if (len < 255 && !long_header) {
         p = grow_cdobuf(pos + len + 1);
         p[pos++] = u32xe((len << 16) | cmd);
     } else {
@@ -1357,6 +1536,10 @@ static void hdr2(uint32_t ** bufp, uint32_t * posp, uint32_t cmd, uint32_t len,
     *posp = pos;
 }
 
+static void hdr2(uint32_t ** bufp, uint32_t * posp, uint32_t cmd, uint32_t len, uint32_t be) {
+    hdr2l(bufp, posp, cmd, len, be, 0);
+}
+
 static LINK * find_block_end(LINK * l, LINK * lh) {
     uint32_t level = 0;
     while (l != lh) {
@@ -1379,9 +1562,10 @@ static LINK * find_block_end(LINK * l, LINK * lh) {
     return lh;
 }
 
-static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
+static void * encode_v2_cmd(uint32_t version, LINK * l, LINK * lh, uint32_t * posp, uint32_t be, uint32_t * aligned_commandp) {
     uint32_t pos = *posp;
     uint32_t * p = grow_cdobuf(pos);
+    uint32_t aligned_command = 0;
     while (l != lh) {
         CdoCommand * cmd = all2cmds(l);
         l = l->next;
@@ -1455,6 +1639,7 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
                     p[pos++] = u32xe_hi(dstaddr);
                     p[pos++] = u32xe_lo(dstaddr);
                     if (!auto_align) break;
+                    aligned_command = 1;
                     if ((pos & 3) == ((dstaddr >> 2) & 3)) break;
                     padding++;
                     pos = pos_save;
@@ -1499,6 +1684,7 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
                 p[pos++] = u32xe_lo(cmd->dstaddr);
                 p[pos++] = u32xe(cmd->value);
                 if (!auto_align) break;
+                aligned_command = 1;
                 if ((pos & 3) == ((cmd->dstaddr >> 2) & 3)) break;
                 padding++;
                 pos = pos_save;
@@ -1527,22 +1713,60 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
             p[pos++] = u32xe(cmd->mask);
             p[pos++] = u32xe(cmd->value);
             break;
+	case CdoCmdListSet:
+		hdr2(&p, &pos, CMD2_LIST_SET, 1 + cmd->count, be);
+		p[pos++] = u32xe_lo(cmd->id);
+		memcpy(p + pos, cmd->buf, cmd->count * sizeof(uint32_t));
+		byte_swap_buffer(p + pos, cmd->count, be);
+		pos += cmd->count;
+		break;
+	case CdoCmdRunProc:
+		hdr2(&p, &pos, CMD2_RUN_PROC, 1, be);
+		p[pos++] = u32xe_lo(cmd->id);
+		break;
+	case CdoCmdListWrite:
+		hdr2(&p, &pos, CMD2_LIST_WRITE, 2, be);
+		p[pos++] = u32xe_lo(cmd->id);
+		p[pos++] = u32xe(cmd->value);
+		break;
+	case CdoCmdListMaskWrite:
+		hdr2(&p, &pos, CMD2_LIST_MASK_WRITE, 3, be);
+		p[pos++] = u32xe_lo(cmd->id);
+		p[pos++] = u32xe(cmd->mask);
+		p[pos++] = u32xe(cmd->value);
+		break;
+	case CdoCmdListMaskPoll: {
+		uint32_t has_flags = cmd->flags != 0;
+		hdr2(&p, &pos, CMD2_LIST_MASK_POLL, has_flags  ? 5 : 4, be);
+		p[pos++] = u32xe_lo(cmd->id);
+		p[pos++] = u32xe(cmd->mask);
+		p[pos++] = u32xe(cmd->value);
+		p[pos++] = u32xe(cmd->count);
+		if (has_flags) {
+			p[pos++] = u32xe(cmd->flags);
+		}
+		break;
+	}
         case CdoCmdMaskPoll: {
             uint32_t has_flags = cmd->flags != 0;
+            uint32_t has_errorcode = cmd->errorcode != 0;
             if ((cmd->dstaddr >> 32) == 0) {
-                hdr2(&p, &pos, CMD2_MASK_POLL, has_flags ? 5 : 4, be);
+                hdr2(&p, &pos, CMD2_MASK_POLL, has_errorcode ? 6: (has_flags  ? 5 : 4), be);
                 p[pos++] = u32xe_lo(cmd->dstaddr);
             } else {
-                hdr2(&p, &pos, CMD2_MASK_POLL64, has_flags ? 6 : 5, be);
+                hdr2(&p, &pos, CMD2_MASK_POLL64, has_errorcode ? 7: (has_flags  ? 6 : 5), be);
                 p[pos++] = u32xe_hi(cmd->dstaddr);
                 p[pos++] = u32xe_lo(cmd->dstaddr);
             }
             p[pos++] = u32xe(cmd->mask);
             p[pos++] = u32xe(cmd->value);
             p[pos++] = u32xe(cmd->count);
-            if (has_flags) {
+            if (has_flags || has_errorcode) {
                 p[pos++] = u32xe(cmd->flags);
             }
+            if (has_errorcode) {
+                p[pos++] = u32xe(cmd->errorcode);
+            }
             break;
         }
 #if 0
@@ -1585,6 +1809,7 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
                 p[pos++] = u32xe_lo(cmd->dstaddr);
                 p[pos++] = u32xe(cmd->value);
                 if (!auto_align) break;
+                aligned_command = 1;
                 if ((pos & 3) == 0) break;
                 padding++;
                 pos = pos_save;
@@ -1670,6 +1895,14 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
         case CdoCmdMarker: {
             uint32_t len = strlen((char *)cmd->buf);
             uint32_t count = (len + 3)/4;
+            if (cmd->value == MARKER_DEVICE || cmd->value == MARKER_DATE)
+            {
+                break;
+            }
+            if (cmd->value == MARKER_PART)
+            {
+                id_code_binary = find_device((char*)cmd->buf);
+            }
             /* Search for the START markers with string "NOC Start up", only for SSIT devices */
             if (search_sync_points && (cmd->value == 0x64 || cmd->value == 0x65) && (strcmp((char*)cmd->buf, "NOC Startup") == 0)) {
                 /* Store the offset of each of the START and END marker with "NOC Start up" string */
@@ -1693,20 +1926,107 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
         }
         case CdoCmdProc: {
             uint32_t pos_save = pos;
-            uint32_t payload_start;
+            uint32_t long_header = 0;
+            uint32_t aligned_proc_command = 0;
             LINK * blockend = find_block_end(l, lh);
-            hdr2(&p, &pos, CMD2_PROC, 1, be);
-            payload_start = pos;
-            p[pos++] = u32xe(cmd->value);
-            p = encode_v2_cmd(l, blockend, &pos, be);
-            hdr2(&p, &pos_save, CMD2_PROC, pos - payload_start, be);
-            if (pos_save != payload_start) {
-                /* Header grew, regenerate payload */
+            for (;;) {
+                uint32_t hdr_len = long_header ? 2 : 1;
+                uint32_t payload_start;
+                uint32_t restart = 0;
+                uint32_t pos_hdr;
                 pos = pos_save;
+
+                if (aligned_proc_command && auto_align && ((pos + hdr_len + 1) & 3) != 0) {
+                    /* Align proc if it contains commands that require alignment */
+                    uint32_t padding = 4 - ((pos + hdr_len + 1) & 3);
+                    hdr2(&p, &pos, CMD2_NOP, padding - 1, be);
+                    memset(p + pos, 0, (padding - 1)*4);
+                    pos += padding - 1;
+                }
+
+                /* Remember header location and reserve space */
+                pos_hdr = pos;
+                hdr2l(&p, &pos, CMD2_PROC, 1, be, long_header);
+                payload_start = pos;
                 p[pos++] = u32xe(cmd->value);
-                p = encode_v2_cmd(l, blockend, &pos, be);
-                /* Update payload size incase it changed due to alignment */
-                p[pos_save - 1] = u32xe(pos - pos_save);
+
+                /* Store commands as payload */
+                p = encode_v2_cmd(version, l, blockend, &pos, be, &aligned_proc_command);
+
+                /* Store header in reserved space. */
+                hdr2l(&p, &pos_hdr, CMD2_PROC, pos - payload_start, be, long_header);
+                if (pos_hdr != payload_start) {
+                    /* Header grew, restart with long header */
+                    assert(!long_header);
+                    assert(pos_hdr - payload_start == 1);
+                    long_header = 1;
+                    restart = 1;
+                }
+
+                if (aligned_proc_command) {
+                    aligned_command = 1;
+                    if (auto_align && ((pos_hdr + 1) & 3) != 0) {
+                        /* Update padding. */
+                        restart = 1;
+                    }
+                }
+
+                if (!restart) break;
+            }
+            l = blockend;
+            if (l != lh) l = l->next;
+            break;
+        }
+        case CdoCmdCdoSequence: {
+            uint32_t pos_save = pos;
+            uint32_t long_header = 0;
+            uint32_t aligned_cdo_command = 0;
+            LINK * blockend = find_block_end(l, lh);
+            for (;;) {
+                uint32_t hdr_len = long_header ? 2 : 1;
+                uint32_t payload_start;
+                uint32_t restart = 0;
+                uint32_t pos_hdr;
+                pos = pos_save;
+
+                if (aligned_cdo_command && auto_align && ((pos + hdr_len + 3) & 3) != 0) {
+                    /* Align command if it contains commands that require alignment */
+                    uint32_t padding = 4 - ((pos + hdr_len + 3) & 3);
+                    hdr2(&p, &pos, CMD2_NOP, padding - 1, be);
+                    memset(p + pos, 0, (padding - 1)*4);
+                    pos += padding - 1;
+                }
+
+                /* Remember header location and reserve space */
+                pos_hdr = pos;
+                hdr2l(&p, &pos, CMD2_CDO_SEQUENCE, 3, be, long_header);
+                payload_start = pos;
+                p[pos++] = u32xe_hi(cmd->dstaddr);
+                p[pos++] = u32xe_lo(cmd->dstaddr);
+                p[pos++] = u32xe(cmd->value);
+
+                /* Store commands as payload */
+                p = encode_v2_cmd(version, l, blockend, &pos, be, &aligned_cdo_command);
+
+                /* Store header in reserved space. */
+                hdr2l(&p, &pos_hdr, CMD2_CDO_SEQUENCE, pos - payload_start, be, long_header);
+                if (pos_hdr != payload_start) {
+                    /* Header grew, restart with long header */
+                    assert(!long_header);
+                    assert(pos_hdr - payload_start == 1);
+                    long_header = 1;
+                    restart = 1;
+                }
+
+                if (aligned_cdo_command) {
+                    aligned_command = 1;
+                    if (auto_align && ((pos_hdr + 3) & 3) != 0) {
+                        /* Update padding. */
+                        restart = 1;
+                    }
+                }
+
+                if (!restart) break;
             }
             l = blockend;
             if (l != lh) l = l->next;
@@ -1723,7 +2043,7 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
             memcpy(p + pos, cmd->buf, len);
             byte_swap_buffer(p + pos, count, be);
             pos += count;
-            p = encode_v2_cmd(l, blockend, &pos, be);
+            p = encode_v2_cmd(version, l, blockend, &pos, be, &aligned_command);
             p[payload_start] = u32xe(pos - payload_start - 1);
             l = blockend;
             break;
@@ -1749,12 +2069,12 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
             LINK * blockend = find_block_end(l, lh);
             hdr2(&p, &pos, CMD2_PSM_SEQUENCE, 0, be);
             payload_start = pos;
-            p = encode_v2_cmd(l, blockend, &pos, be);
+            p = encode_v2_cmd(version, l, blockend, &pos, be, &aligned_command);
             hdr2(&p, &pos_save, CMD2_PSM_SEQUENCE, pos - payload_start, be);
             if (pos_save != payload_start) {
                 /* Header grew, regenerate payload */
                 pos = pos_save;
-                p = encode_v2_cmd(l, blockend, &pos, be);
+                p = encode_v2_cmd(version, l, blockend, &pos, be, &aligned_command);
             }
             l = blockend;
             if (l != lh) l = l->next;
@@ -1775,18 +2095,29 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
             byte_swap_buffer(p + pos, cmd->count, be);
             pos += cmd->count;
             break;
-
+        case CdoCmdTamperTrigger:
+            hdr2(&p, &pos, CMD2_TAMPER_TRIGGER, 1, be);
+            p[pos++] = u32xe(cmd->value);
+            break;
+        case CdoCmdSetIpiAccess:
+            hdr2(&p, &pos, CMD2_SET_IPI_ACCESS, 2, be);
+            p[pos++] = u32xe(cmd->value);
+            p[pos++] = u32xe(cmd->mask);
+            break;
         case CdoCmdNpiSeq:
+            if (version >= CDO_VERSION_2_00) goto npi_error;
             hdr2(&p, &pos, CMD2_NPI_SEQ, 2, be);
             p[pos++] = u32xe_lo(cmd->dstaddr);
             p[pos++] = u32xe(cmd->flags);
             break;
         case CdoCmdNpiPreCfg:
+            if (version >= CDO_VERSION_2_00) goto npi_error;
             hdr2(&p, &pos, CMD2_NPI_PRECFG, 2, be);
             p[pos++] = u32xe_lo(cmd->dstaddr);
             p[pos++] = u32xe(cmd->flags);
             break;
         case CdoCmdNpiWrite:
+            if (version >= CDO_VERSION_2_00) goto npi_error;
             hdr2(&p, &pos, CMD2_NPI_WRITE, 2 + cmd->count, be);
             p[pos++] = u32xe_lo(cmd->dstaddr);
             p[pos++] = u32xe(cmd->flags);
@@ -1795,11 +2126,17 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
             pos += cmd->count;
             break;
         case CdoCmdNpiShutdown:
+            if (version >= CDO_VERSION_2_00) goto npi_error;
             hdr2(&p, &pos, CMD2_NPI_SHUTDN, 2, be);
             p[pos++] = u32xe_lo(cmd->dstaddr);
             p[pos++] = u32xe(cmd->flags);
             break;
 
+        case CdoCmdPmHnicxNpiDataXfer:
+            hdr2(&p, &pos, CMD2_PM_HNICX_NPI_DATA_XFER, 2, be);
+            p[pos++] = u32xe_lo(cmd->dstaddr);
+            p[pos++] = u32xe(cmd->value);
+            break;
         case CdoCmdPmGetApiVersion:
             hdr2(&p, &pos, CMD2_PM_GET_API_VERSION, 0, be);
             break;
@@ -2098,7 +2435,13 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
             byte_swap_buffer(p + pos, cmd->count, be);
             pos += cmd->count;
             break;
+        case CdoCmdPmForceHc:
+            hdr2(&p, &pos, CMD2_PM_FORCE_HC, 1, be);
+            p[pos++] = u32xe(cmd->id);
+            break;
+
         case CdoCmdCfuSetCrc32:
+            if (version >= CDO_VERSION_2_00) goto cfu_error;
             if (cmd->flags != 0) {
                 hdr2(&p, &pos, CMD2_CFU_SET_CRC32, 2, be);
                 p[pos++] = u32xe(cmd->flags);
@@ -2109,34 +2452,42 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
             }
             break;
         case CdoCmdCfuDecompress:
+            if (version >= CDO_VERSION_2_00) goto cfu_error;
             hdr2(&p, &pos, CMD2_CFU_DECOMPRESS, 1, be);
             p[pos++] = u32xe(cmd->flags);
             break;
         case CdoCmdCfuCramRW:
+            if (version >= CDO_VERSION_2_00) goto cfu_error;
             hdr2(&p, &pos, CMD2_CFU_CRAM_RW, 1, be);
             p[pos++] = u32xe(cmd->value);
             break;
         case CdoCmdCfuSeuGo:
+            if (version >= CDO_VERSION_2_00) goto cfu_error;
             hdr2(&p, &pos, CMD2_CFU_SEU_GO, 1, be);
             p[pos++] = u32xe(cmd->flags);
             break;
         case CdoCmdCfuCrc8Dis:
+            if (version >= CDO_VERSION_2_00) goto cfu_error;
             hdr2(&p, &pos, CMD2_CFU_CRC8_DIS, 1, be);
             p[pos++] = u32xe(cmd->flags);
             break;
         case CdoCmdCfuSsiPerSlrPr:
+            if (version >= CDO_VERSION_2_00) goto cfu_error;
             hdr2(&p, &pos, CMD2_CFU_SSI_PER_SLR_PR, 1, be);
             p[pos++] = u32xe(cmd->flags);
             break;
         case CdoCmdCfuGsrGsc:
+            if (version >= CDO_VERSION_2_00) goto cfu_error;
             hdr2(&p, &pos, CMD2_CFU_GSR_GSC, 1, be);
             p[pos++] = u32xe(cmd->flags);
             break;
         case CdoCmdCfuGcapClkEn:
+            if (version >= CDO_VERSION_2_00) goto cfu_error;
             hdr2(&p, &pos, CMD2_CFU_GCAP_CLK_EN, 1, be);
             p[pos++] = u32xe(cmd->flags);
             break;
         case CdoCmdCfuCfiType:
+            if (version >= CDO_VERSION_2_00) goto cfu_error;
             hdr2(&p, &pos, CMD2_CFU_CFI_TYPE, 1, be);
             p[pos++] = u32xe(cmd->value);
             break;
@@ -2157,11 +2508,28 @@ static void * encode_v2_cmd(LINK * l, LINK * lh, uint32_t * posp, uint32_t be) {
             hdr2(&p, &pos, CMD2_LDR_CFRAME_CLEAR_CHECK, 1, be);
             p[pos++] = u32xe(cmd->id);
             break;
+
+        case CdoCmdSemNpiTable:
+            hdr2(&p, &pos, CMD2_SEM_NPI_TABLE, 2 + cmd->count, be);
+            p[pos++] = u32xe(cmd->id);
+            p[pos++] = u32xe(cmd->flags);
+            memcpy(p + pos, cmd->buf, cmd->count * sizeof(uint32_t));
+            byte_swap_buffer(p + pos, cmd->count, be);
+            pos += cmd->count;
+            break;
+
         default:
             fprintf(stderr, "unknown command (%u)\n", cmd->type);
             break;
+        npi_error:
+            fprintf(stderr, "npi related commands are not supported in CDO version 2 or newer\n");
+            break;
+        cfu_error:
+            fprintf(stderr, "cfu related commands are not supported in CDO version 2 or newer\n");
+            break;
         }
     }
+    if (aligned_commandp) *aligned_commandp = aligned_command;
     *posp = pos;
     return p;
 }
@@ -2174,7 +2542,7 @@ static void * encode_v2(CdoSequence * seq, size_t * sizep, uint32_t be) {
     p[0] = u32xe(pos - 1);
     p[1] = u32xe(0x004F4443);
     p[2] = u32xe(seq->version);
-    p = encode_v2_cmd(l, &seq->cmds, &pos, be);
+    p = encode_v2_cmd(seq->version, l, &seq->cmds, &pos, be, NULL);
     p[3] = u32xe(pos - hdrlen - 1);
     p[4] = u32xe(checksum32(p, hdrlen, be));
     clear_cdobuf();
@@ -2182,12 +2550,54 @@ static void * encode_v2(CdoSequence * seq, size_t * sizep, uint32_t be) {
     return p;
 }
 
+void cdoseq_write_header_raw(FILE * f, CdoSequence ** seq) {
+    LINK * l = (*seq)->cmds.next;
+    char * buf;
+    fprintf(f, "Xilinx ASCII NPI Deviceimage\n");
+
+    while (l != &((*seq)->cmds)) {
+        CdoCommand * cmd = all2cmds(l);
+        l = l->next;
+        if (cmd->type != CdoCmdMarker) {
+            /* header markers are at the top */
+            break;
+        }
+        switch (cmd->value) {
+        case 1:
+            buf = "Created by";
+            break;
+        case 2:
+            buf = "Design name:";
+            break;
+        case 3:
+            buf = "Architecture:";
+            break;
+        case 4:
+            buf = "Part:";
+            break;
+        case 5:
+            buf = "SLR:";
+            break;
+        case 6:
+            buf = "Date:";
+            break;
+        default:
+            buf = NULL;
+            break;
+        }
+        if (buf) {
+            fprintf(f, "%s %s\n", buf, (char *)cmd->buf);
+            cdocmd_free(cmd);
+        }
+    }
+}
+
 void * cdoseq_to_binary(CdoSequence * seq, size_t * sizep, uint32_t be) {
     void * (*encode)(CdoSequence * seq, size_t * sizep, uint32_t be) = NULL;
     *sizep = 0;
-    if (seq->version < 0x200) {
+    if (seq->version < CDO_VERSION_1_50) {
         encode = encode_v1;
-    } else if (seq->version < 0x300) {
+    } else if (seq->version < CDO_VERSION_3_00) {
         encode = encode_v2;
     } else {
         fprintf(stderr, "unsupported version %#"PRIx32"\n", seq->version);
