@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2019-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -155,9 +156,24 @@ void cdocmd_add_random_command(CdoSequence * seq, uint32_t * levelp) {
              type == CdoCmdSetBaseAddress ||
              (type == CdoCmdProc && level != 0) ||
              (type == CdoCmdPsmSequence && level != 0) ||
+             (type == CdoCmdCdoSequence && level != 0) ||
              (type == CdoCmdBegin && level >= (randu32() & 3)) ||
              (type == CdoCmdEnd && level == 0) ||
-             (type == CdoCmdBreak && level == 0));
+             (type == CdoCmdBreak && level == 0) ||
+	     (seq->version >= CDO_VERSION_2_00 && (
+                 type == CdoCmdNpiPreCfg ||
+                 type == CdoCmdNpiSeq ||
+                 type == CdoCmdNpiWrite ||
+                 type == CdoCmdNpiShutdown ||
+                 type == CdoCmdCfuSetCrc32 ||
+                 type == CdoCmdCfuDecompress ||
+                 type == CdoCmdCfuCramRW ||
+                 type == CdoCmdCfuSeuGo ||
+                 type == CdoCmdCfuCrc8Dis ||
+                 type == CdoCmdCfuSsiPerSlrPr ||
+                 type == CdoCmdCfuGsrGsc ||
+                 type == CdoCmdCfuGcapClkEn ||
+                 type == CdoCmdCfuCfiType)));
     cmd = cdocmd_alloc(type);
     cmd->id = randu32();
     cmd->dstaddr = randu64();
@@ -192,7 +208,9 @@ void cdocmd_add_random_command(CdoSequence * seq, uint32_t * levelp) {
         cmd->type == CdoCmdPmAddRequirement ||
         cmd->type == CdoCmdPmInitNode ||
         cmd->type == CdoCmdPmSetNodeAccess ||
-        cmd->type == CdoCmdPmNocClockEnable) {
+        cmd->type == CdoCmdPmNocClockEnable ||
+	cmd->type == CdoCmdListSet ||
+        cmd->type == CdoCmdSemNpiTable) {
         uint32_t * p;
         uint32_t i;
         cmd->count = (cmd->count & 15) + 1;
@@ -227,6 +245,7 @@ void cdocmd_add_random_command(CdoSequence * seq, uint32_t * levelp) {
     }
     if (cmd->type == CdoCmdProc) level++;
     if (cmd->type == CdoCmdPsmSequence) level++;
+    if (cmd->type == CdoCmdCdoSequence) level++;
     if (cmd->type == CdoCmdBegin) level++;
     if (cmd->type == CdoCmdEnd) level--;
     add_command(seq, cmd);
@@ -244,7 +263,7 @@ void cdocmd_add_generic_command(CdoSequence * seq, uint32_t id, void * buf, uint
 void cdocmd_add_comment(CdoSequence * seq, const char * comment, ...) {
     CdoCommand * cmd = cdocmd_alloc(CdoCmdComment);
     va_list ap;
-    int size = 1;
+    int size = 4;	/* Align on 4 to avoid issues in copy_buf() */
     cmd->buf = (char *)myalloc(size);
     while (1) {
         int n;
@@ -253,7 +272,7 @@ void cdocmd_add_comment(CdoSequence * seq, const char * comment, ...) {
         va_end(ap);
         if (n >= 0) {
             if (n < size) break;
-            size = n + 1;
+            size = (n + 4) & ~3;
         } else {
             size *= 2;
         }
@@ -272,7 +291,7 @@ void cdocmd_add_section(CdoSequence * seq, uint32_t id) {
 void cdocmd_add_include(CdoSequence * seq, const char * name) {
     CdoCommand * cmd = cdocmd_alloc(CdoCmdInclude);
     cmd->count = strlen(name) / 4 + 1;
-    cmd->buf = strdup(name);
+    cmd->buf = my_strdup(name);
     add_command(seq, cmd);
 }
 
@@ -318,6 +337,17 @@ void cdocmd_add_mask_poll(CdoSequence * seq, uint64_t addr, uint32_t mask, uint3
     add_command(seq, cmd);
 }
 
+void cdocmd_add_mask_poll_w_err(CdoSequence * seq, uint64_t addr, uint32_t mask, uint32_t value, uint32_t count, uint32_t flags, uint32_t errorcode) {
+    CdoCommand * cmd = cdocmd_alloc(CdoCmdMaskPoll);
+    cmd->dstaddr = addr;
+    cmd->mask = mask;
+    cmd->value = value;
+    cmd->count = count;
+    cmd->flags = flags;
+    cmd->errorcode = errorcode;
+    add_command(seq, cmd);
+}
+
 void cdocmd_add_mask_write(CdoSequence * seq, uint64_t addr, uint32_t mask, uint32_t value) {
     CdoCommand * cmd = cdocmd_alloc(CdoCmdMaskWrite);
     cmd->dstaddr = addr;
@@ -330,6 +360,45 @@ void cdocmd_add_write(CdoSequence * seq, uint64_t addr, uint32_t value) {
     cdocmd_add_block_write(seq, addr, 1, &value, is_be_host());
 }
 
+void cdocmd_add_list_set(CdoSequence * seq, uint32_t id, uint32_t count, void * buf, uint32_t be) {
+	CdoCommand * cmd = cdocmd_alloc(CdoCmdListSet);
+	cmd->id = id;
+	cmd->count = count;
+	cmd->buf = copy_buf(buf, count, be);
+	add_command(seq, cmd);
+}
+
+void cdocmd_add_list_write(CdoSequence * seq, uint32_t id, uint32_t value) {
+	CdoCommand * cmd = cdocmd_alloc(CdoCmdListWrite);
+	cmd->id = id;
+	cmd->value = value;
+	add_command(seq, cmd);
+}
+
+void cdocmd_add_list_mask_write(CdoSequence * seq, uint32_t id, uint32_t mask, uint32_t value) {
+	CdoCommand * cmd = cdocmd_alloc(CdoCmdListMaskWrite);
+	cmd->id = id;
+	cmd->mask = mask;
+	cmd->value = value;
+	add_command(seq, cmd);
+}
+
+void cdocmd_add_list_mask_poll(CdoSequence * seq, uint32_t id, uint32_t mask, uint32_t value, uint32_t count, uint32_t flags) {
+    CdoCommand * cmd = cdocmd_alloc(CdoCmdListMaskPoll);
+    cmd->id = id;
+    cmd->mask = mask;
+    cmd->value = value;
+    cmd->count = count;
+    cmd->flags = flags;
+    add_command(seq, cmd);
+}
+
+void cdocmd_add_run_proc(CdoSequence * seq, uint32_t id) {
+	CdoCommand * cmd = cdocmd_alloc(CdoCmdRunProc);
+	cmd->id = id;
+	add_command(seq, cmd);
+}
+
 void cdocmd_add_delay(CdoSequence * seq, uint32_t delay) {
     CdoCommand * cmd = cdocmd_alloc(CdoCmdDelay);
     cmd->value = delay;
@@ -446,6 +515,13 @@ void cdocmd_add_pm_request_wakeup(CdoSequence * seq, uint32_t nodeid, uint64_t a
     add_command(seq, cmd);
 }
 
+void cdocmd_add_pm_hnicx_npi_data_xfer(CdoSequence * seq, uint32_t addr, uint32_t value) {
+	CdoCommand * cmd = cdocmd_alloc(CdoCmdPmHnicxNpiDataXfer);
+	cmd->dstaddr = addr;
+	cmd->value = value;
+	add_command(seq, cmd);
+}
+
 void cdocmd_add_pm_set_wakeup_source(CdoSequence * seq, uint32_t target, uint32_t source, uint32_t enable) {
     CdoCommand * cmd = cdocmd_alloc(CdoCmdPmSetWakeupSource);
     cmd->id = target;
@@ -686,7 +762,7 @@ void cdocmd_add_pm_add_node_name(CdoSequence * seq, uint32_t nodeid, const char
     CdoCommand * cmd = cdocmd_alloc(CdoCmdPmAddNodeName);
     cmd->id = nodeid;
     cmd->count = strlen(name) / 4 + 1;
-    cmd->buf = strdup(name);
+    cmd->buf = my_strdup(name);
     add_command(seq, cmd);
 }
 
@@ -770,6 +846,12 @@ void cdocmd_add_pm_if_noc_clock_enable(CdoSequence * seq, uint32_t index, uint32
     add_command(seq, cmd);
 }
 
+void cdocmd_add_pm_force_hc(CdoSequence * seq, uint32_t nodeid) {
+    CdoCommand * cmd = cdocmd_alloc(CdoCmdPmForceHc);
+    cmd->id = nodeid;
+    add_command(seq, cmd);
+}
+
 void cdocmd_add_cfu_set_crc32(CdoSequence * seq, uint32_t type, uint32_t value) {
     CdoCommand * cmd = cdocmd_alloc(CdoCmdCfuSetCrc32);
     cmd->flags = type;
@@ -872,7 +954,7 @@ void cdocmd_add_event_logging(CdoSequence * seq, uint32_t subcmd, uint32_t count
 void cdocmd_add_set_board(CdoSequence * seq, const char * name) {
     CdoCommand * cmd = cdocmd_alloc(CdoCmdSetBoard);
     cmd->count = strlen(name) / 4 + 1;
-    cmd->buf = strdup(name);
+    cmd->buf = my_strdup(name);
     add_command(seq, cmd);
 }
 
@@ -886,7 +968,7 @@ void cdocmd_add_set_plm_wdt(CdoSequence * seq, uint32_t nodeid, uint32_t periodi
 void cdocmd_add_log_string(CdoSequence * seq, const char * name) {
     CdoCommand * cmd = cdocmd_alloc(CdoCmdLogString);
     cmd->count = strlen(name) / 4 + 1;
-    cmd->buf = strdup(name);
+    cmd->buf = my_strdup(name);
     add_command(seq, cmd);
 }
 
@@ -900,7 +982,7 @@ void cdocmd_add_marker(CdoSequence * seq, uint32_t value, const char * name) {
     CdoCommand * cmd = cdocmd_alloc(CdoCmdMarker);
     cmd->value = value;
     cmd->count = strlen(name) / 4 + 1;
-    cmd->buf = strdup(name);
+    cmd->buf = my_strdup(name);
     add_command(seq, cmd);
 }
 
@@ -913,7 +995,7 @@ void cdocmd_add_proc(CdoSequence * seq, uint32_t value) {
 void cdocmd_add_begin(CdoSequence * seq, const char * name) {
     CdoCommand * cmd = cdocmd_alloc(CdoCmdBegin);
     cmd->count = strlen(name) / 4 + 1;
-    cmd->buf = strdup(name);
+    cmd->buf = my_strdup(name);
     add_command(seq, cmd);
 }
 
@@ -939,6 +1021,13 @@ void cdocmd_add_psm_sequence(CdoSequence * seq) {
     add_command(seq, cmd);
 }
 
+void cdocmd_add_cdo_sequence(CdoSequence * seq, uint64_t addr, uint32_t keyhole_size) {
+    CdoCommand * cmd = cdocmd_alloc(CdoCmdCdoSequence);
+    cmd->dstaddr = addr;
+    cmd->value = keyhole_size;
+    add_command(seq, cmd);
+}
+
 void cdocmd_add_scatter_write(CdoSequence * seq, uint32_t value, uint32_t count, void * buf, uint32_t be) {
     CdoCommand * cmd = cdocmd_alloc(CdoCmdScatterWrite);
     cmd->value = value;
@@ -956,6 +1045,19 @@ void cdocmd_add_scatter_write2(CdoSequence * seq, uint32_t value1, uint32_t valu
     add_command(seq, cmd);
 }
 
+void cdocmd_add_tamper_trigger(CdoSequence * seq, uint32_t value) {
+    CdoCommand * cmd = cdocmd_alloc(CdoCmdTamperTrigger);
+    cmd->value = value;
+    add_command(seq, cmd);
+}
+
+void cdocmd_add_set_ipi_access(CdoSequence * seq, uint32_t value, uint32_t mask) {
+    CdoCommand * cmd = cdocmd_alloc(CdoCmdSetIpiAccess);
+    cmd->value = value;
+    cmd->mask = mask;
+    add_command(seq, cmd);
+}
+
 void cdocmd_add_pm_register_notifier(CdoSequence * seq, uint32_t nodeid, uint32_t mask, uint32_t arg1, uint32_t arg2) {
     CdoCommand * cmd = cdocmd_alloc(CdoCmdPmRegisterNotifier);
     cmd->id = nodeid;
@@ -988,6 +1090,15 @@ void cdocmd_add_ldr_cframe_clear_check(CdoSequence * seq, uint32_t block_type) {
     add_command(seq, cmd);
 }
 
+void cdocmd_add_sem_npi_table(CdoSequence * seq, uint32_t nodeid, uint32_t flags, uint32_t count, void * buf, uint32_t be) {
+    CdoCommand * cmd = cdocmd_alloc(CdoCmdSemNpiTable);
+    cmd->id = nodeid;
+    cmd->flags = flags;
+    cmd->count = count;
+    cmd->buf = copy_buf(buf, count, be);
+    add_command(seq, cmd);
+}
+
 void cdocmd_insert_seq(CdoCommand * cmd, CdoSequence * seq) {
     list_concat(&cmd->link_all, &seq->cmds);
     list_init(&seq->cmds);
