@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2019-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -30,8 +31,7 @@ struct CdoOverlayEntry {
 
 struct CdoOverlayInfo {
     CdoSequence * seq;
-    size_t count;
-    CdoOverlayEntry * list;
+    LINK actions;
 };
 
 CdoOverlayInfo * cdooverlay_open(CdoSequence * seq);
