@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2019-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,7 +18,7 @@
 #ifndef D_cdo_raw
 #define D_cdo_raw
 
-#include "cdo-command.h"
+#include "cdo-metadata.h"
 
 typedef struct CdoRawInfo CdoRawInfo;
 
@@ -29,11 +30,18 @@ typedef enum {
 
 struct CdoRawInfo {
     CdoRawType type;
+    CdoMetadata meta;
+    uint32_t data_free_disable:1;
     size_t size;
-    uint32_t data[1];                   /* Must be last */
+    uint32_t *data;
 };
 
-CdoRawInfo * decode_raw(CdoSequence ** seq, void * data, size_t size);
+CdoRawInfo * cdoraw_create(CdoRawType type);
+void cdoraw_delete(CdoRawInfo * raw);
+CdoRawInfo * cdoraw_decode(void * data, size_t size);
+void cdoraw_encode(CdoRawInfo * raw, FILE * f);
+void cdoraw_encode_seq(FILE * f, CdoSequence * seq, uint32_t be);
+CdoRawInfo * decode_raw(void * data, size_t size);
 void encode_raw(FILE * f, void * data, size_t size);
 
 #endif /* D_cdo_raw */
