@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2019-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -21,10 +22,10 @@
 #include <inttypes.h>
 #include "cdo-load.h"
 #include "cdo-source.h"
-#include <stdbool.h>
 
 static char* slr_id_ptr;
 static char slr_id;
+uint32_t id_code_source;
 
 #if defined(_WIN32)
 #define strcasecmp(x,y) stricmp(x,y)
@@ -77,12 +78,21 @@ struct command_info {
     { "break", CdoCmdBreak },
     { "ot_check", CdoCmdOtCheck },
     { "psm_sequence", CdoCmdPsmSequence },
+    { "cdo_sequence", CdoCmdCdoSequence },
     { "scatter_write", CdoCmdScatterWrite },
     { "scatter_write2", CdoCmdScatterWrite2 },
+    { "tamper_trigger", CdoCmdTamperTrigger },
+    { "set_ipi_access", CdoCmdSetIpiAccess },
+    { "list_set", CdoCmdListSet},
+    { "list_write", CdoCmdListWrite},
+    { "list_mask_write", CdoCmdListMaskWrite},
+    { "list_mask_poll", CdoCmdListMaskPoll},
+    { "run_proc", CdoCmdRunProc},
     { "npi_seq", CdoCmdNpiSeq },
     { "npi_precfg", CdoCmdNpiPreCfg },
     { "npi_write", CdoCmdNpiWrite },
     { "npi_shutdown", CdoCmdNpiShutdown },
+    { "pm_hnicx_npi_data_xfer", CdoCmdPmHnicxNpiDataXfer },
     { "pm_get_api_version", CdoCmdPmGetApiVersion },
     { "pm_get_device_status", CdoCmdPmGetDeviceStatus },
     { "pm_register_notifier", CdoCmdPmRegisterNotifier },
@@ -137,6 +147,7 @@ struct command_info {
     { "pm_if_noc_clock_enable", CdoCmdPmIfNocClockEnable },
     { "pm_activate_subsystem", CdoCmdPmActivateSubsystem },
     { "pm_set_node_access", CdoCmdPmSetNodeAccess },
+    { "pm_force_hc", CdoCmdPmForceHc},
     { "cfu_set_crc32", CdoCmdCfuSetCrc32 },
     { "cfu_decompress", CdoCmdCfuDecompress },
     { "cfu_cram_rw", CdoCmdCfuCramRW },
@@ -149,6 +160,7 @@ struct command_info {
     { "em_set_action", CdoCmdEmSetAction },
     { "ldr_set_image_info", CdoCmdLdrSetImageInfo },
     { "cframe_clear_check", CdoCmdLdrCframeClearCheck },
+    { "sem_npi_table", CdoCmdSemNpiTable },
     { NULL, 0 }
 };
 
@@ -162,6 +174,69 @@ char* marker_list[] = {
 };
 int marker_count[sizeof(marker_list) / sizeof(marker_list[0])];
 
+struct cdo_device {
+    const char* name;
+    uint32_t idcode;
+} device_list[] = {
+    { "xcvm1402", 0x04C08093 },
+    { "xcvm1302", 0x04C09093 },
+    { "xcv20", 0x04C0F093 },
+    { "xcvp1052", 0x04C18093 },
+    { "xcvp1002", 0x04C1B093 },
+    { "xcvm2102", 0x04C1C093 },
+    { "xcvp1402", 0x04C20093 },
+    { "xcvp1102", 0x04C22093 },
+    { "xcvm2902", 0x04C23093 },
+    { "xcvm2302", 0x04C24093 },
+    { "xcvr1352", 0x04C90093 },
+    { "xcvr1302", 0x04C91093 },
+    { "xcvc1352", 0x04C93093 },
+    { "xcvr1402", 0x04C94093 },
+    { "xcvc1702", 0x04C98093 },
+    { "xcvm1502", 0x04C99093 },
+    { "xcve1752", 0x04C9A093 },
+    { "xcvc1502", 0x04C9B093 },
+    { "xcvr1702", 0x04CA0093 },
+    { "xcvr1502", 0x04CA2093 },
+    { "xcvr1602", 0x04CA3093 },
+    { "xcvr1802", 0x04CA5093 },
+    { "xcvc1802", 0x14CA9093 },
+    { "xcvm1802", 0x14CAA093 },
+    { "xcv65", 0x14CAF093 },
+    { "xcve2102", 0x04CC0093 },
+    { "xcve2002", 0x04CC1093 },
+    { "xcve2302", 0x14CC8093 },
+    { "xcve2202", 0x14CC9093 },
+    { "xcvm1102", 0x14CCA093 },
+    { "xcvc2802", 0x14CD0093 },
+    { "xcvc2602", 0x14CD1093 },
+    { "xcve2602", 0x14CD2093 },
+    { "xcve2802", 0x14CD3093 },
+    { "xcvm2202", 0x14CD4093 },
+    { "xcv70", 0x14CD7093 },
+    { "xcvp1202", 0x14D00093 },
+    { "xcvm2502", 0x14D01093 },
+    { "xcvp1502", 0x14D08093 },
+    { "xcvp1702", 0x14D10093 },
+    { "xcvp1802", 0x14D14093 },
+    { "xcvp2502", 0x14D1C093 },
+    { "xcvp2802", 0x14D20093 },
+    { "xcvh1582", 0x14D28093 },
+    { "xcvh1542", 0x14D29093 },
+    { "xcvh1522", 0x14D2A093 },
+    { "xcvh1782", 0x14D2C093 },
+    { "xcvh1742", 0x14D2D093 },
+    { "xcv80", 0x14D2F093 },
+    { "xcvp1552", 0x14D34093 },
+    { "xcvp1752", 0x14D38093 },
+    { "xcvn3716", 0x14D80093 },
+    { "xcvn3516", 0x14D81093 },
+    { "xcvn3408", 0x14D82093 },
+    { "xcvn3708", 0x14D83093 },
+    { "xcvc1902", 0x14CA8093 },
+    { NULL, 0 }
+};
+
 static uint32_t iseol(char ** sp) {
     char * s = *sp;
     skipsp(s);
@@ -179,6 +254,15 @@ static command_info * find_command(char * name, uint32_t len) {
     return NULL;
 }
 
+static uint32_t  find_device(char * name) {
+    struct cdo_device * device = device_list;
+    while (device->name != NULL) {
+        if (strncmp(device->name, name, sizeof(char*)) == 0) break;
+        device++;
+    }
+    return device->idcode;
+}
+
 static uint32_t parse_u32(char ** sp, uint32_t * valuep) {
     char * s = *sp;
     skipsp(s);
@@ -283,6 +367,16 @@ char slr_id_from_source(char ch)
     return ch;
 }
 
+uint32_t idcode_from_source(uint32_t id)
+{
+    if (id_code_source != 0)
+    {
+        id = id_code_source;
+        id_code_source = 0;
+    }
+    return id;
+}
+
 static void check_redundant_markers(char * marker_string)
 {
     bool same_marker_found = false;
@@ -366,7 +460,7 @@ CdoSequence * cdoseq_from_source(FILE * f) {
         switch(cmdinfo->id) {
         case CdoCmdSection: {
             uint32_t id;
-            if (seq->version >= 0x200) goto syntax_error;
+            if (seq->version >= CDO_VERSION_1_50) goto syntax_error;
             if (parse_u32(&s, &id)) goto syntax_error;
             cdocmd_add_section(seq, id);
             break;
@@ -412,6 +506,57 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             free(buf);
             break;
         }
+	case CdoCmdListSet: {
+		uint32_t id;
+		uint32_t * buf;
+		uint32_t count;
+		if (parse_u32(&s, &id)) goto syntax_error;
+		if (parse_buf(&s, &buf, &count)) goto syntax_error;
+		cdocmd_add_list_set(seq, id, count, buf, is_be_host());
+		free(buf);
+		break;
+	}
+	case CdoCmdListWrite: {
+		uint32_t id;
+		uint32_t value;
+		if (parse_u32(&s, &id)) goto syntax_error;
+		if (parse_u32(&s, &value)) goto syntax_error;
+		cdocmd_add_list_write(seq, id, value);
+		break;
+	}
+	case CdoCmdListMaskWrite: {
+		uint32_t id;
+		uint32_t mask;
+		uint32_t value;
+		if (parse_u32(&s, &id)) goto syntax_error;
+		if (parse_u32(&s, &mask)) goto syntax_error;
+		if (parse_u32(&s, &value)) goto syntax_error;
+		cdocmd_add_list_mask_write(seq, id, mask, value);
+		break;
+	}
+	case CdoCmdListMaskPoll: {
+            uint32_t id;
+            uint32_t mask;
+            uint32_t value;
+            uint32_t count = 0;
+            uint32_t flags = 0;
+            if (parse_u32(&s, &id)) goto syntax_error;
+            if (parse_u32(&s, &mask)) goto syntax_error;
+            if (parse_u32(&s, &value)) goto syntax_error;
+            skipsp(s);
+            if (istok(*s) && parse_u32(&s, &count)) goto syntax_error;
+            skipsp(s);
+            if (istok(*s) && parse_u32(&s, &flags)) goto syntax_error;
+            skipsp(s);
+            cdocmd_add_list_mask_poll(seq, id, mask, value, count, flags);
+            break;
+	}
+	case CdoCmdRunProc: {
+            uint32_t id;
+            if (parse_u32(&s, &id)) goto syntax_error;
+		cdocmd_add_run_proc(seq, id);
+		break;
+	}
         case CdoCmdSetBlock: {
             uint64_t addr;
             uint32_t count;
@@ -450,6 +595,7 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             uint32_t value;
             uint32_t count = 0;
             uint32_t flags = 0;
+            uint32_t errorcode = 0;
             if (parse_u64(&s, &addr)) goto syntax_error;
             if (parse_u32(&s, &mask)) goto syntax_error;
             if (parse_u32(&s, &value)) goto syntax_error;
@@ -457,7 +603,12 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             if (istok(*s) && parse_u32(&s, &count)) goto syntax_error;
             skipsp(s);
             if (istok(*s) && parse_u32(&s, &flags)) goto syntax_error;
-            cdocmd_add_mask_poll(seq, addr, mask, value, count, flags);
+            skipsp(s);
+            if (istok(*s) && parse_u32(&s, &errorcode)) goto syntax_error;
+            if (errorcode)
+                cdocmd_add_mask_poll_w_err(seq, addr, mask, value, count, flags, errorcode);
+            else
+                cdocmd_add_mask_poll(seq, addr, mask, value, count, flags);
             break;
         }
         case CdoCmdDelay: {
@@ -476,7 +627,7 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             if (parse_u64(&s, &dstaddr)) goto syntax_error;
             if (parse_u32(&s, &count)) goto syntax_error;
             if (parse_u32(&s, &flags)) goto syntax_error;
-            if (seq->version < 0x200 && srcaddr == 0) {
+            if (seq->version < CDO_VERSION_2_00 && srcaddr == 0) {
                 uint32_t count2 = 0;
                 if (parse_buf(&s, &buf, &count2)) goto syntax_error;
                 if (count != count2) {
@@ -500,7 +651,7 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             if (parse_buf(&s, &buf, &count)) goto syntax_error;
             cdocmd_add_cframe_read(seq, param, addr, read_count, count, buf, is_be_host());
             free(buf);
-            if (seq->version < 0x200 && count != 16) goto syntax_error;
+            if (seq->version < CDO_VERSION_2_00 && count != 16) goto syntax_error;
             break;
         }
         case CdoCmdSsitSyncMaster: {
@@ -587,6 +738,10 @@ CdoSequence * cdoseq_from_source(FILE * f) {
                 slr_id_ptr = name;
                 slr_id = *slr_id_ptr;
             }
+            if (value == MARKER_PART)
+            {
+                id_code_source = find_device(name);
+            }   
             cdocmd_add_marker(seq, value, name);
             free(name);
             break;
@@ -633,6 +788,15 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             cdocmd_add_psm_sequence(seq);
             level++;
             break;
+        case CdoCmdCdoSequence: {
+            uint64_t addr;
+            uint32_t value;
+            if (parse_u64(&s, &addr)) goto syntax_error;
+            if (parse_u32(&s, &value)) goto syntax_error;
+            cdocmd_add_cdo_sequence(seq, addr, value);
+            level++;
+            break;
+        }
         case CdoCmdScatterWrite: {
             uint32_t * buf;
             uint32_t count;
@@ -655,12 +819,26 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             free(buf);
             break;
         }
-
+        case CdoCmdTamperTrigger: {
+            uint32_t value;
+            if (parse_u32(&s, &value)) goto syntax_error;
+            cdocmd_add_tamper_trigger(seq, value);
+            break;
+        }
+        case CdoCmdSetIpiAccess: {
+            uint32_t value;
+            uint32_t mask;
+            if (parse_u32(&s, &value)) goto syntax_error;
+            if (parse_u32(&s, &mask)) goto syntax_error;
+            cdocmd_add_set_ipi_access(seq, value, mask);
+            break;
+        }
         case CdoCmdNpiSeq:
         case CdoCmdNpiPreCfg:
         case CdoCmdNpiShutdown: {
             uint32_t addr;
             uint32_t flags;
+            if (seq->version >= CDO_VERSION_2_00) goto syntax_error;
             if (parse_u32(&s, &addr)) goto syntax_error;
             if (parse_u32(&s, &flags)) goto syntax_error;
             if (cmdinfo->id == CdoCmdNpiSeq) {
@@ -677,6 +855,7 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             uint32_t flags;
             uint32_t count;
             uint32_t * buf;
+            if (seq->version >= CDO_VERSION_2_00) goto syntax_error;
             if (parse_u32(&s, &addr)) goto syntax_error;
             if (parse_u32(&s, &flags)) goto syntax_error;
             if (parse_buf(&s, &buf, &count)) goto syntax_error;
@@ -684,6 +863,14 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             free(buf);
             break;
         }
+        case CdoCmdPmHnicxNpiDataXfer: {
+            uint32_t addr;
+            uint32_t value;
+            if (parse_u32(&s, &addr)) goto syntax_error;
+            if (parse_u32(&s, &value)) goto syntax_error;
+            cdocmd_add_pm_hnicx_npi_data_xfer(seq, addr, value);
+            break;
+        }
         case CdoCmdPmGetApiVersion: {
             cdocmd_add_pm_get_api_version(seq);
             break;
@@ -1133,9 +1320,16 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             free(buf);
             break;
         }
+        case CdoCmdPmForceHc: {
+            uint32_t id;
+            if (parse_u32(&s, &id)) goto syntax_error;
+            cdocmd_add_pm_force_hc(seq, id);
+            break;
+        }
         case CdoCmdCfuSetCrc32: {
             uint32_t flags;
             uint32_t value;
+            if (seq->version >= CDO_VERSION_2_00) goto syntax_error;
             if (parse_u32(&s, &flags)) goto syntax_error;
             if (flags != 0) {
                 if (parse_u32(&s, &value)) goto syntax_error;
@@ -1147,48 +1341,56 @@ CdoSequence * cdoseq_from_source(FILE * f) {
         }
         case CdoCmdCfuDecompress: {
             uint32_t flags;
+            if (seq->version >= CDO_VERSION_2_00) goto syntax_error;
             if (parse_u32(&s, &flags)) goto syntax_error;
             cdocmd_add_cfu_decompress(seq, flags);
             break;
         }
         case CdoCmdCfuCramRW: {
             uint32_t value;
+            if (seq->version >= CDO_VERSION_2_00) goto syntax_error;
             if (parse_u32(&s, &value)) goto syntax_error;
             cdocmd_add_cfu_cram_rw(seq, value);
             break;
         }
         case CdoCmdCfuSeuGo: {
             uint32_t value;
+            if (seq->version >= CDO_VERSION_2_00) goto syntax_error;
             if (parse_u32(&s, &value)) goto syntax_error;
             cdocmd_add_cfu_seu_go(seq, value);
             break;
         }
         case CdoCmdCfuCrc8Dis: {
             uint32_t flags;
+            if (seq->version >= CDO_VERSION_2_00) goto syntax_error;
             if (parse_u32(&s, &flags)) goto syntax_error;
             cdocmd_add_cfu_crc8_dis(seq, flags);
             break;
         }
         case CdoCmdCfuSsiPerSlrPr: {
             uint32_t value;
+            if (seq->version >= CDO_VERSION_2_00) goto syntax_error;
             if (parse_u32(&s, &value)) goto syntax_error;
             cdocmd_add_cfu_ssi_per_slr_pr(seq, value);
             break;
         }
         case CdoCmdCfuGsrGsc: {
             uint32_t value;
+            if (seq->version >= CDO_VERSION_2_00) goto syntax_error;
             if (parse_u32(&s, &value)) goto syntax_error;
             cdocmd_add_cfu_gsr_gsc(seq, value);
             break;
         }
         case CdoCmdCfuGcapClkEn: {
             uint32_t value;
+            if (seq->version >= CDO_VERSION_2_00) goto syntax_error;
             if (parse_u32(&s, &value)) goto syntax_error;
             cdocmd_add_cfu_gcap_clk_en(seq, value);
             break;
         }
         case CdoCmdCfuCfiType: {
             uint32_t value;
+            if (seq->version >= CDO_VERSION_2_00) goto syntax_error;
             if (parse_u32(&s, &value)) goto syntax_error;
             cdocmd_add_cfu_cfi_type(seq, value);
             break;
@@ -1221,6 +1423,18 @@ CdoSequence * cdoseq_from_source(FILE * f) {
             cdocmd_add_ldr_cframe_clear_check(seq, id);
             break;
         }
+        case CdoCmdSemNpiTable: {
+            uint32_t id;
+            uint32_t flags;
+            uint32_t count;
+            uint32_t * buf;
+            if (parse_u32(&s, &id)) goto syntax_error;
+            if (parse_u32(&s, &flags)) goto syntax_error;
+            if (parse_buf(&s, &buf, &count)) goto syntax_error;
+            cdocmd_add_sem_npi_table(seq, id, flags, count, buf, is_be_host());
+            free(buf);
+            break;
+        }
         default:
             goto syntax_error;
         }
@@ -1242,6 +1456,14 @@ error:
     return NULL;
 }
 
+static void print_x32(FILE * f, uint32_t v) {
+    if (v == 0) {
+        fprintf(f, "0");
+    } else {
+        fprintf(f, "%#"PRIx32, v);
+    }
+}
+
 static void print_x64(FILE * f, uint64_t v) {
     if (v == 0) {
         fprintf(f, "0");
@@ -1349,10 +1571,14 @@ void cdoseq_to_source(FILE * f, CdoSequence * seq) {
             print_x64(f, cmd->value);
             fprintf(f, " ");
             print_x64(f, cmd->count);
-            if (cmd->flags != 0) {
+            if (cmd->flags != 0 || cmd->errorcode != 0) {
                 fprintf(f, " ");
                 print_x64(f, cmd->flags);
             }
+            if (cmd->errorcode != 0) {
+                fprintf(f, " ");
+                print_x64(f, cmd->errorcode);
+            }
             fprintf(f, "\n");
             break;
         case CdoCmdDelay:
@@ -1367,7 +1593,7 @@ void cdoseq_to_source(FILE * f, CdoSequence * seq) {
             print_x64(f, cmd->count);
             fprintf(f, " ");
             print_x64(f, cmd->flags);
-            if (seq->version < 0x200 && cmd->buf) {
+            if (seq->version < CDO_VERSION_2_00 && cmd->buf) {
                 print_buf(f, cmd->buf, cmd->count);
             }
             fprintf(f, "\n");
@@ -1476,6 +1702,14 @@ void cdoseq_to_source(FILE * f, CdoSequence * seq) {
             fprintf(f, "psm_sequence\n");
             level++;
             break;
+        case CdoCmdCdoSequence:
+            fprintf(f, "cdo_sequence ");
+            print_x64(f, cmd->dstaddr);
+            fprintf(f, " ");
+            print_x64(f, cmd->value);
+            fprintf(f, "\n");
+            level++;
+            break;
         case CdoCmdScatterWrite:
             fprintf(f, "scatter_write ");
             print_x64(f, cmd->value);
@@ -1490,7 +1724,18 @@ void cdoseq_to_source(FILE * f, CdoSequence * seq) {
             print_buf(f, cmd->buf, cmd->count);
             fprintf(f, "\n");
             break;
-
+        case CdoCmdTamperTrigger:
+            fprintf(f, "tamper_trigger ");
+            print_x64(f, cmd->value);
+            fprintf(f, "\n");
+            break;
+        case CdoCmdSetIpiAccess:
+            fprintf(f, "set_ipi_access ");
+            print_x64(f, cmd->value);
+            fprintf(f, " ");
+            print_x64(f, cmd->mask);
+            fprintf(f, "\n");
+            break;
         case CdoCmdNpiSeq:
             fprintf(f, "npi_seq ");
             print_x64(f, cmd->dstaddr);
@@ -1520,6 +1765,13 @@ void cdoseq_to_source(FILE * f, CdoSequence * seq) {
             print_x64(f, cmd->flags);
             fprintf(f, "\n");
             break;
+        case CdoCmdPmHnicxNpiDataXfer:
+            fprintf(f, "pm_hnicx_npi_data_xfer ");
+            print_x32(f, cmd->dstaddr);
+            fprintf(f, " ");
+            print_x32(f, cmd->value);
+            fprintf(f, "\n");
+            break;
         case CdoCmdPmGetApiVersion:
             fprintf(f, "pm_get_api_version");
             fprintf(f, "\n");
@@ -1885,6 +2137,11 @@ void cdoseq_to_source(FILE * f, CdoSequence * seq) {
             print_buf(f, cmd->buf, cmd->count);
             fprintf(f, "\n");
             break;
+        case CdoCmdPmForceHc:
+            fprintf(f, "pm_force_hc ");
+            print_x64(f, cmd->id);
+            fprintf(f, "\n");
+            break;
         case CdoCmdCfuSetCrc32:
             fprintf(f, "cfu_set_crc32 ");
             print_x64(f, cmd->flags);
@@ -1959,6 +2216,57 @@ void cdoseq_to_source(FILE * f, CdoSequence * seq) {
             print_x64(f, cmd->id);
             fprintf(f, "\n");
             break;
+        case CdoCmdSemNpiTable:
+            fprintf(f, "sem_npi_table ");
+            print_x64(f, cmd->id);
+            fprintf(f, " ");
+            print_x64(f, cmd->flags);
+            print_buf(f, cmd->buf, cmd->count);
+            fprintf(f, "\n");
+            break;
+	case CdoCmdListSet:
+		fprintf(f, "list_set ");
+		print_x64(f, cmd->id);
+		fprintf(f, " ");
+		print_buf(f, cmd->buf, cmd->count);
+		fprintf(f, "\n");
+		break;
+	case CdoCmdListWrite:
+		fprintf(f, "list_write ");
+		print_x64(f, cmd->id);
+		fprintf(f, " ");
+		print_x64(f, cmd->value);
+		fprintf(f, "\n");
+		break;
+	case CdoCmdListMaskWrite:
+		fprintf(f, "list_mask_write ");
+		print_x64(f, cmd->id);
+		fprintf(f, " ");
+		print_x64(f, cmd->mask);
+		fprintf(f, " ");
+		print_x64(f, cmd->value);
+		fprintf(f, "\n");
+		break;
+	case CdoCmdListMaskPoll:
+		fprintf(f, "list_mask_poll ");
+		print_x64(f, cmd->id);
+		fprintf(f, " ");
+		print_x64(f, cmd->mask);
+		fprintf(f, " ");
+		print_x64(f, cmd->value);
+		fprintf(f, " ");
+		print_x64(f, cmd->count);
+		if (cmd->flags != 0) {
+			fprintf(f, " ");
+			print_x64(f, cmd->flags);
+		}
+		fprintf(f, "\n");
+		break;
+	case CdoCmdRunProc:
+		fprintf(f, "run_proc ");
+		print_x64(f, cmd->id);
+		fprintf(f, "\n");
+		break;
         default:
             fprintf(f, "unknown command (%u)\n", cmd->type);
             break;
