@@ -42,7 +42,7 @@
 
 /* "%code requires" blocks.  */
 /* Line 33 of lalr1.cc  */
-#line 33 "../../s/cmdoptions.y"
+#line 34 "../../s/cmdoptions.y"
 
     namespace CO {
         class BisonScanner;
@@ -69,7 +69,7 @@
 #endif
 
 /* Line 33 of lalr1.cc  */
-#line 23 "../../s/cmdoptions.y"
+#line 24 "../../s/cmdoptions.y"
 namespace CO {
 /* Line 33 of lalr1.cc  */
 #line 76 "cmdoptions.tab.hpp"
@@ -83,7 +83,7 @@ namespace CO {
     union semantic_type
     {
 /* Line 33 of lalr1.cc  */
-#line 63 "../../s/cmdoptions.y"
+#line 64 "../../s/cmdoptions.y"
 
     uint64_t number;
     char* cstring;
@@ -122,167 +122,176 @@ namespace CO {
      _PROCESS_BITSTREAM = 275,
      MCS = 276,
      BIN = 277,
-     _DUMP = 278,
-     DUMP_PLM = 279,
-     DUMP_PMC_CDO = 280,
-     DUMP_BOOT_FILES = 281,
-     _DUMP_DIR = 282,
-     DUMP_SLAVE_PDIS = 283,
-     _ARCH = 284,
-     ZYNQ = 285,
-     ZYNQMP = 286,
-     VERSAL = 287,
-     _R = 288,
-     FPGA = 289,
-     VERSALNET = 290,
-     _DUAL_QSPI_MODE = 291,
-     _DUAL_OSPI_MODE = 292,
-     PARALLEL = 293,
-     STACKED = 294,
-     _W = 295,
-     ON = 296,
-     OFF = 297,
-     _NOAUTHBLOCKS = 298,
-     _ZYNQMPES1 = 299,
-     _OVERLAYCDO = 300,
-     _EFUSEPPKBITS = 301,
-     _GENERATE_HASHES = 302,
-     _PADIMAGEHEADER = 303,
-     _SPKSIGNATURE = 304,
-     _GENERATE_KEYS = 305,
-     PEM = 306,
-     RSA = 307,
-     ECDSAP521 = 308,
-     AUTH = 309,
-     GREY = 310,
-     METAL = 311,
-     _SECUREDEBUG = 312,
-     ECDSA = 313,
-     _AUTHJTAG = 314,
-     _ENCRYPT = 315,
-     BBRAM = 316,
-     EFUSE = 317,
-     _P_TOK = 318,
-     _INTERFACE = 319,
-     SMAPx8 = 320,
-     SMAPx16 = 321,
-     SMAPx32 = 322,
-     SPI = 323,
-     BPIx8 = 324,
-     BPIx16 = 325,
-     _READ = 326,
-     READ_BH = 327,
-     READ_IHT = 328,
-     READ_IH = 329,
-     READ_PHT = 330,
-     READ_AC = 331,
-     _VERIFY = 332,
-     _VERIFYKDF = 333,
-     _ZYNQMPENCRDUMP = 334,
-     HEXVALUE = 335,
-     IDENTIFIER = 336,
-     FILENAME = 337,
-     QFILENAME = 338,
-     HEXSTRING = 339,
-     EQUALS = 340,
-     HMAC = 341,
-     STARTCBC = 342,
-     KEY0 = 343,
-     COMMA = 344,
-     HBIFHELP = 345,
-     HARCH = 346,
-     HIMAGE = 347,
-     HFILL = 348,
-     HO = 349,
-     HP = 350,
-     HW = 351,
-     HEFUSEPPKBITS = 352,
-     HGENHASHES = 353,
-     HLEGACY = 354,
-     HPADHDR = 355,
-     H_SPKSIGN = 356,
-     HPACKAGE = 357,
-     HENCRYPT = 358,
-     HGENKEYS = 359,
-     HDQSPI = 360,
-     HLOG = 361,
-     HZYNQMPES1 = 362,
-     HPROCESSBIT = 363,
-     HNONBOOTING = 364,
-     HENCRDUMP = 365,
-     HPOSTPROCESS = 366,
-     HVERIFY = 367,
-     HSECUREDEBUG = 368,
-     HREAD = 369,
-     HVERIFYKDF = 370,
-     HDUMP = 371,
-     HDUMPDIR = 372,
-     HOVLCDO = 373,
-     H_BIF_INIT = 374,
-     H_BIF_UDFBH = 375,
-     H_BIF_AES = 376,
-     H_BIF_PPK = 377,
-     H_BIF_PSK = 378,
-     H_BIF_SPK = 379,
-     H_BIF_SSK = 380,
-     H_BIF_SPKSIGN = 381,
-     H_BIF_HIVEC = 382,
-     H_BIF_HDRSIGN = 383,
-     H_BIF_BOOTIMAGE = 384,
-     H_BIF_BL = 385,
-     H_BIF_PID = 386,
-     H_BIF_ENCR = 387,
-     H_BIF_AUTH = 388,
-     H_BIF_CHKSM = 389,
-     H_BIF_ELYHNDOFF = 390,
-     H_BIF_BHSIGN = 391,
-     H_BIF_POWNER = 392,
-     H_BIF_PRESIGN = 393,
-     H_BIF_UDF = 394,
-     H_BIF_XIP = 395,
-     H_BIF_ALIGN = 396,
-     H_BIF_OFFSET = 397,
-     H_BIF_RES = 398,
-     H_BIF_LOAD = 399,
-     H_BIF_TZ = 400,
-     H_BIF_STARTUP = 401,
-     H_BIF_KEYSRC = 402,
-     H_BIF_FSBLCFG = 403,
-     H_BIF_BOOTDEV = 404,
-     H_BIF_DESTCPU = 405,
-     H_BIF_DESTDEV = 406,
-     H_BIF_EL = 407,
-     H_SPLIT = 408,
-     H_BIF_AUTHPARAM = 409,
-     H_BIF_BHKEY = 410,
-     H_BIF_PFW = 411,
-     H_BIF_BLOCKS = 412,
-     H_BIF_METAL = 413,
-     H_BIF_BHIV = 414,
-     H_BIF_BOOTVEC = 415,
-     H_BIF_PUFDATA = 416,
-     H_BIF_PTYPE = 417,
-     H_BIF_IMAGECFG = 418,
-     H_BIF_PMCCONFIG = 419,
-     H_BIF_AARCH32 = 420,
-     H_BIF_BIGENDIAN = 421,
-     H_BIF_BOOTCONFIG = 422,
-     H_BIF_COPY = 423,
-     H_BIF_CORE = 424,
-     H_BIF_DELAY_HANDOFF = 425,
-     H_BIF_DELAY_LOAD = 426,
-     H_BIF_FILE = 427,
-     H_BIF_ID = 428,
-     H_BIF_IMAGE = 429,
-     H_BIF_METAHDR = 430,
-     H_BIF_NAME = 431,
-     H_BIF_PARTITION = 432,
-     H_BIF_SLR = 433,
-     H_BIF_TYPE = 434,
-     H_BIF_KEYSRCENCR = 435,
-     H_BIF_PARENTID = 436,
-     H_DPACM_ENABLE = 437,
-     H_BIF_USERKEYS = 438
+     _OUT_TYPE = 278,
+     _DUMP = 279,
+     DUMP_PLM = 280,
+     DUMP_PMC_CDO = 281,
+     DUMP_BOOT_FILES = 282,
+     _DUMP_DIR = 283,
+     DUMP_SLAVE_PDIS = 284,
+     _ARCH = 285,
+     ZYNQ = 286,
+     ZYNQMP = 287,
+     VERSAL = 288,
+     _R = 289,
+     FPGA = 290,
+     VERSALNET = 291,
+     _DUAL_QSPI_MODE = 292,
+     _DUAL_OSPI_MODE = 293,
+     PARALLEL = 294,
+     STACKED = 295,
+     _W = 296,
+     ON = 297,
+     OFF = 298,
+     _NOAUTHBLOCKS = 299,
+     _ZYNQMPES1 = 300,
+     _OVERLAYCDO = 301,
+     _EFUSEPPKBITS = 302,
+     _GENERATE_HASHES = 303,
+     _PADIMAGEHEADER = 304,
+     _SPKSIGNATURE = 305,
+     _GENERATE_KEYS = 306,
+     PEM = 307,
+     RSA = 308,
+     ECDSAP521 = 309,
+     AUTH = 310,
+     GREY = 311,
+     METAL = 312,
+     _SECUREDEBUG = 313,
+     ECDSA = 314,
+     _AUTHJTAG = 315,
+     _ENCRYPT = 316,
+     BBRAM = 317,
+     EFUSE = 318,
+     _P_TOK = 319,
+     _INTERFACE = 320,
+     SMAPx8 = 321,
+     SMAPx16 = 322,
+     SMAPx32 = 323,
+     SPI = 324,
+     BPIx8 = 325,
+     BPIx16 = 326,
+     _READ = 327,
+     READ_BH = 328,
+     READ_IHT = 329,
+     READ_IH = 330,
+     READ_PHT = 331,
+     READ_AC = 332,
+     _VERIFY = 333,
+     _VERIFYKDF = 334,
+     _AUTH_OPTIMIZATION = 335,
+     _ZYNQMPENCRDUMP = 336,
+     HEXVALUE = 337,
+     IDENTIFIER = 338,
+     FILENAME = 339,
+     QFILENAME = 340,
+     HEXSTRING = 341,
+     EQUALS = 342,
+     HMAC = 343,
+     STARTCBC = 344,
+     KEY0 = 345,
+     COMMA = 346,
+     HBIFHELP = 347,
+     HARCH = 348,
+     HIMAGE = 349,
+     HFILL = 350,
+     HO = 351,
+     HP = 352,
+     HW = 353,
+     HEFUSEPPKBITS = 354,
+     HGENHASHES = 355,
+     HLEGACY = 356,
+     HPADHDR = 357,
+     H_SPKSIGN = 358,
+     HAUTHOPT = 359,
+     HPACKAGE = 360,
+     HENCRYPT = 361,
+     HGENKEYS = 362,
+     HDQSPI = 363,
+     HLOG = 364,
+     HZYNQMPES1 = 365,
+     HPROCESSBIT = 366,
+     HNONBOOTING = 367,
+     HENCRDUMP = 368,
+     HPOSTPROCESS = 369,
+     HVERIFY = 370,
+     HSECUREDEBUG = 371,
+     HREAD = 372,
+     HVERIFYKDF = 373,
+     HDUMP = 374,
+     HDUMPDIR = 375,
+     HOVLCDO = 376,
+     HOUTTYPE = 377,
+     H_BIF_INIT = 378,
+     H_BIF_UDFBH = 379,
+     H_BIF_AES = 380,
+     H_BIF_PPK = 381,
+     H_BIF_PSK = 382,
+     H_BIF_SPK = 383,
+     H_BIF_SSK = 384,
+     H_BIF_SPKSIGN = 385,
+     H_BIF_HIVEC = 386,
+     H_BIF_HDRSIGN = 387,
+     H_BIF_BOOTIMAGE = 388,
+     H_BIF_BL = 389,
+     H_BIF_PID = 390,
+     H_BIF_ENCR = 391,
+     H_BIF_AUTH = 392,
+     H_BIF_CHKSM = 393,
+     H_BIF_ELYHNDOFF = 394,
+     H_BIF_BHSIGN = 395,
+     H_BIF_TCMBOOT = 396,
+     H_BIF_OPTIONALDATA = 397,
+     H_BIF_POWNER = 398,
+     H_BIF_PRESIGN = 399,
+     H_BIF_UDF = 400,
+     H_BIF_XIP = 401,
+     H_BIF_ALIGN = 402,
+     H_BIF_OFFSET = 403,
+     H_BIF_RES = 404,
+     H_BIF_LOAD = 405,
+     H_BIF_TZ = 406,
+     H_BIF_STARTUP = 407,
+     H_BIF_KEYSRC = 408,
+     H_BIF_FSBLCFG = 409,
+     H_BIF_BOOTDEV = 410,
+     H_BIF_DESTCPU = 411,
+     H_BIF_DESTDEV = 412,
+     H_BIF_EL = 413,
+     H_SPLIT = 414,
+     H_BIF_AUTHPARAM = 415,
+     H_BIF_BHKEY = 416,
+     H_BIF_PFW = 417,
+     H_BIF_BLOCKS = 418,
+     H_BIF_METAL = 419,
+     H_BIF_BHIV = 420,
+     H_BIF_BOOTVEC = 421,
+     H_BIF_PUFDATA = 422,
+     H_BIF_PTYPE = 423,
+     H_BIF_IMAGECFG = 424,
+     H_BIF_PMCCONFIG = 425,
+     H_BIF_AARCH32 = 426,
+     H_BIF_BIGENDIAN = 427,
+     H_BIF_BOOTCONFIG = 428,
+     H_BIF_COPY = 429,
+     H_BIF_CORE = 430,
+     H_BIF_DELAY_HANDOFF = 431,
+     H_BIF_DELAY_LOAD = 432,
+     H_BIF_FILE = 433,
+     H_BIF_ID = 434,
+     H_BIF_IMAGE = 435,
+     H_BIF_METAHDR = 436,
+     H_BIF_NAME = 437,
+     H_BIF_PARTITION = 438,
+     H_BIF_SLR = 439,
+     H_BIF_TYPE = 440,
+     H_BIF_KEYSRCENCR = 441,
+     H_BIF_PARENTID = 442,
+     H_DPACM_ENABLE = 443,
+     H_BIF_USERKEYS = 444,
+     HVN_BIF_PCR = 445,
+     HVN_BIF_PCR_MINDEX = 446,
+     HV_BIF_IMAGESTORE = 447
    };
 
     };
@@ -384,7 +393,7 @@ namespace CO {
     /// - if positive, shift that token.
     /// - if negative, reduce the rule which number is the opposite.
     /// - if zero, do what YYDEFACT says.
-    static const unsigned char yytable_[];
+    static const unsigned short int yytable_[];
     static const signed char yytable_ninf_;
 
     static const short int yycheck_[];
@@ -456,10 +465,10 @@ namespace CO {
     Options& options;
   };
 /* Line 33 of lalr1.cc  */
-#line 23 "../../s/cmdoptions.y"
+#line 24 "../../s/cmdoptions.y"
 } // CO
 /* Line 33 of lalr1.cc  */
-#line 463 "cmdoptions.tab.hpp"
+#line 472 "cmdoptions.tab.hpp"
 
 
 
