@@ -1,6 +1,6 @@
-
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -27,6 +27,7 @@
 #include "imageheadertable-versal.h"
 #include "bootheader-versal.h"
 #include "partitionheadertable-versal.h"
+#include "authentication-versal.h"
 #include <openssl/rand.h>
 
 /*
@@ -489,7 +490,7 @@ void VersalEncryptionContext::ChunkifyAndProcess(BootImage& bi, PartitionHeader*
 
         LOG_INFO("Encrypting Bootloader");
 
-        if (partHdr->imageHeader->GetAuthenticationType() != Authentication::None)
+        if ((partHdr->imageHeader->GetAuthenticationType() != Authentication::None) || (partHdr->imageHeader->GetDelayAuthFlag()))
         {
             Binary::Length_t dataChunksCount = 0;
             // PLM
@@ -526,13 +527,14 @@ void VersalEncryptionContext::ChunkifyAndProcess(BootImage& bi, PartitionHeader*
 
         uint32_t* tmpIvPMCDATA = new uint32_t[WORDS_PER_IV];
         memset(tmpIvPMCDATA, 0, WORDS_PER_IV);
+
         // PMC Data Encryption
-        if (options.bifOptions->pmcDataAesFile != "")
+        if (bi.pmcDataAesFile != "")
         {
             //aesKey = NULL;
             aesSeedexits = false;
             fixedInputDataExits = false;
-            SetAesFileName(options.bifOptions->pmcDataAesFile);
+            SetAesFileName(bi.pmcDataAesFile);
             LOG_INFO("Key file - %s", aesFilename.c_str());
             std::ifstream keyFile(aesFilename);
             bool fileExists = keyFile.good();
@@ -593,7 +595,11 @@ void VersalEncryptionContext::ChunkifyAndProcess(BootImage& bi, PartitionHeader*
         VersalBootHeaderStructure* bh = (VersalBootHeaderStructure*)bi.bootHeader->section->Data;
         bh->plmLength = partHdr->imageHeader->GetFsblFwSizeIh();
         bh->pmcCdoLength = partHdr->imageHeader->GetTotalPmcFwSizeIh();
-        bh->totalPlmLength = estimatedTotalFsblLength + partHdr->imageHeader->GetAuthContext()->GetCertificateSize();
+        bh->totalPlmLength = estimatedTotalFsblLength;
+        if ((bi.partitionHeaderList.front()->imageHeader->GetAuthenticationType() != Authentication::None) || (partHdr->imageHeader->GetDelayAuthFlag()))
+        {
+            bh->totalPlmLength += sizeof(AuthCertificate4096Sha3PaddingStructure);
+        }
         bh->totalPmcCdoLength = estimatedtotalPmcCdoLength;
 
         memcpy_be((uint8_t*)bh->plmSecureHdrIv, tmpIv, BYTES_PER_IV);
@@ -610,14 +616,17 @@ void VersalEncryptionContext::ChunkifyAndProcess(BootImage& bi, PartitionHeader*
             bh->bhAttributes |= bi.bifOptions->GetBhRsa() << BH_RSA_BIT_SHIFT;
             bh->bhAttributes |= bi.bifOptions->GetPufMode() << BH_PUF_MODE_BIT_SHIFT;
 
-            if (bi.partitionHeaderList.front()->imageHeader->GetAuthenticationType() != Authentication::None)
+            if ((bi.partitionHeaderList.front()->imageHeader->GetAuthenticationType() != Authentication::None) || (partHdr->imageHeader->GetDelayAuthFlag()))
             {
                 bh->bhAttributes |= BH_RSA_SINGED_BIT_MASK << BH_RSA_SINGED_BIT_SHIFT;
             }
             bh->bhAttributes |= bi.bifOptions->GetDice() << BH_DICE_BIT_SHIFT;
         }
-
-        bh->sourceOffset = sizeof(VersalBootHeaderStructure) + partHdr->imageHeader->GetAuthContext()->GetCertificateSize();
+        bh->sourceOffset = sizeof(VersalBootHeaderStructure);
+        if ((bi.partitionHeaderList.front()->imageHeader->GetAuthenticationType() != Authentication::None) || (partHdr->imageHeader->GetDelayAuthFlag()))
+        {
+            bh->sourceOffset += sizeof(AuthCertificate4096Sha3PaddingStructure);
+        }
         bh->imageHeaderByteOffset = sizeof(VersalBootHeaderStructure) + bh->totalPlmLength + bh->totalPmcCdoLength;
         bh->headerChecksum = bi.bootHeader->ComputeWordChecksum(&bh->widthDetectionWord, 3872);
 
@@ -644,11 +653,11 @@ void VersalEncryptionContext::ChunkifyAndProcess(BootImage& bi, PartitionHeader*
             options.bifOptions->InsertEncryptionBlock(options.bifOptions->pmcdataBlocks[itr]);
         }
 
-        if (options.bifOptions->pmcDataAesFile != "")
+        if (bi.pmcDataAesFile != "")
         {
             aesKey = aesIv = aesSeed = NULL;
             fixedInputData = NULL;
-            SetAesFileName(options.bifOptions->pmcDataAesFile);
+            SetAesFileName(bi.pmcDataAesFile);
             LOG_INFO("Key file - %s", aesFilename.c_str());
             std::ifstream keyFile(aesFilename);
             bool fileExists = keyFile.good();
@@ -773,7 +782,7 @@ void VersalEncryptionContext::ChunkifyAndProcess(BootImage& bi, PartitionHeader*
 
 
         uint32_t estimatedTotalFsblLength = estimatedEncrLength;
-        if (partHdr->imageHeader->GetAuthenticationType() != Authentication::None)
+        if ((partHdr->imageHeader->GetAuthenticationType() != Authentication::None) || (partHdr->imageHeader->GetDelayAuthFlag()))
         {
             Binary::Length_t dataChunksCount = 0;
 
@@ -792,7 +801,11 @@ void VersalEncryptionContext::ChunkifyAndProcess(BootImage& bi, PartitionHeader*
         VersalBootHeaderStructure* bh = (VersalBootHeaderStructure*)bi.bootHeader->section->Data;
         bh->plmLength = partHdr->imageHeader->GetFsblFwSizeIh();
         bh->pmcCdoLength = 0;
-        bh->totalPlmLength = estimatedTotalFsblLength + partHdr->imageHeader->GetAuthContext()->GetCertificateSize();
+        bh->totalPlmLength = estimatedTotalFsblLength;
+        if ((bi.partitionHeaderList.front()->imageHeader->GetAuthenticationType() != Authentication::None) || (partHdr->imageHeader->GetDelayAuthFlag()))
+        {
+            bh->totalPlmLength += sizeof(AuthCertificate4096Sha3PaddingStructure);
+        }
         bh->totalPmcCdoLength = 0;
 
         memcpy_be((uint8_t*)bh->plmSecureHdrIv, tmpIv, BYTES_PER_IV);
@@ -809,13 +822,17 @@ void VersalEncryptionContext::ChunkifyAndProcess(BootImage& bi, PartitionHeader*
             bh->bhAttributes |= bi.bifOptions->GetBhRsa() << BH_RSA_BIT_SHIFT;
             bh->bhAttributes |= bi.bifOptions->GetPufMode() << BH_PUF_MODE_BIT_SHIFT;
 
-            if (bi.partitionHeaderList.front()->imageHeader->GetAuthenticationType() != Authentication::None)
+            if ((bi.partitionHeaderList.front()->imageHeader->GetAuthenticationType() != Authentication::None) || (partHdr->imageHeader->GetDelayAuthFlag()))
             {
                 bh->bhAttributes |= BH_RSA_SINGED_BIT_MASK << BH_RSA_SINGED_BIT_SHIFT;
             }
             bh->bhAttributes |= bi.bifOptions->GetDice() << BH_DICE_BIT_SHIFT;
         }
-        bh->sourceOffset = sizeof(VersalBootHeaderStructure) + partHdr->imageHeader->GetAuthContext()->GetCertificateSize();
+        bh->sourceOffset = sizeof(VersalBootHeaderStructure);
+        if ((bi.partitionHeaderList.front()->imageHeader->GetAuthenticationType() != Authentication::None) || (partHdr->imageHeader->GetDelayAuthFlag()))
+        {
+            bh->sourceOffset += sizeof(AuthCertificate4096Sha3PaddingStructure);
+        }
         bh->imageHeaderByteOffset = sizeof(VersalBootHeaderStructure) + bh->totalPlmLength + bh->totalPmcCdoLength;
         bh->headerChecksum = bi.bootHeader->ComputeWordChecksum(&bh->widthDetectionWord, 3872);
 
