@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -219,6 +220,7 @@ ImageHeader::ImageHeader(std::string& filename)
     , lockstep(false)
     , cluster(0)
     , delayAuth(false)
+    , tcmBoot(false)
 { }
 
 /******************************************************************************/
@@ -290,6 +292,7 @@ ImageHeader::ImageHeader(std::ifstream& ifs)
     , lockstep(false)
     , cluster(0)
     , delayAuth(false)
+    , tcmBoot(false)
 { }
 
 /******************************************************************************/
@@ -361,6 +364,7 @@ ImageHeader::ImageHeader(uint8_t* data, uint64_t len)
     , lockstep(false)
     , cluster(0)
     , delayAuth(false)
+    , tcmBoot(false)
 {
 }
 
@@ -619,7 +623,7 @@ void ImageHeader::SetDestCpu(DestinationCPU::Type type)
     }
     if (destCpu == DestinationCPU::PMU && destCpuPmuExists)
     {
-        LOG_ERROR("Bif attribute Error!!!'core=psm/pmu' cannot be specified on multiple partitions.");
+        //LOG_ERROR("Bif attribute Error!!!'core=psm/pmu' cannot be specified on multiple partitions.");
     }
     else if (destCpu == DestinationCPU::PMU)
     {
@@ -677,4 +681,15 @@ uint32_t ImageHeader::GetFunctionId(void)
 bool ImageHeader::IsUidInfoFoundInCdo(void)
 {
     return uidInfoFoundInCdo;
-}
\ No newline at end of file
+}
+
+/******************************************************************************/
+bool ImageHeader::IsElf(std::string line)
+{
+    bool status = false;
+    if ((line.at(0) == 0x7f) && (line.at(1) == 0x45) && (line.at(2) == 0x4C) && (line.at(3) == 0x46))
+    {
+        status = true;
+    }
+    return status;
+}
