@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -58,6 +59,12 @@ typedef struct
     SlrPdiType type;
 } SlrPdiInfo;
 
+typedef struct
+{
+    std::string file;
+    uint32_t id;
+} ImageStorePdiInfo;
+
 /* Forward Class References */
 class BaseThing;
 class Options;
@@ -149,12 +156,14 @@ public:
     virtual void SetClusterNum(uint8_t num) { cluster = num; }
     virtual void SetLockStepFlag(bool flag) { lockstep = flag; }
     virtual void SetDelayAuthFlag (bool flag) { delayAuth = flag; }
+    virtual void SetTcmBoot(bool flag) { tcmBoot = flag; }
 
     virtual void SetEncryptionKeySrc(KeySource::Type type) { };
     virtual void SetPartitionRevocationId(uint32_t id) { };
     virtual uint32_t GetPartitionRevocationId() { return 0; }
     virtual void SetSlrBootPartitions(std::list<SlrPdiInfo*>) { };
     virtual void SetSlrConfigPartitions(std::list<SlrPdiInfo*>) { };
+    virtual void SetWriteImageStorePartitions(ImageStorePdiInfo*) { };
     void SetMemCopyAddress(uint64_t addr) { ihMemCpyAddr = addr; }
     void SetDelayLoadHandOffFlags(bool load_flag, bool handoff_flag) { ihDelayLoad = load_flag; ihDelayHandoff = handoff_flag; }
 
@@ -209,6 +218,7 @@ public:
     virtual uint8_t GetClusterNum(void) { return cluster; }
     virtual bool GetLockStepFlag(void) { return lockstep; }
     virtual bool GetDelayAuthFlag(void) { return delayAuth; }
+    virtual bool GetTcmBootFlag(void) { return tcmBoot; }
 
     // For multiple key files and auth parameters
     void SetAesKeyFile(std::string filename) { aesKeyFile = filename; }
@@ -342,6 +352,7 @@ protected:
     DpaCM::Type dpacm;
     PufHdLoc::Type pufHdLoc;
     std::string kekIvFile;
+    bool tcmBoot;
 
     std::list<PartitionHeader*> partitionHeaderList;
 
@@ -354,6 +365,7 @@ protected:
     virtual void ImportElf(BootImage& bi);
     virtual void ImportBit(BootImage& bi) { };
     virtual void ImportBin(BootImage& bi) { };
+    virtual bool IsElf(std::string line);
     virtual void ImportFpgaDataFile(BootImage& bi) { };
     uint8_t* CombineElfSections(ElfFormat* elf, Binary::Length_t* size, Binary::Address_t* load_addr);
     uint8_t* GetElfSections(ElfFormat* elf, Binary::Length_t* size, Binary::Address_t* load_addr, uint32_t iprog);
@@ -364,6 +376,7 @@ protected:
     uint32_t allHdrSize;
     std::list<SlrPdiInfo*> slrBootPdiInfo;
     std::list<SlrPdiInfo*> slrConfigPdiInfo;
+    ImageStorePdiInfo* imageStorePdiInfo;
     uint32_t imageId;
     uint32_t uniqueId;
     uint32_t parentUniqueId;
@@ -385,6 +398,8 @@ public:
         , encrypt(NULL)
         , iht_optional_data(NULL)
         , iht_optional_data_length(0)
+        , idCodeCheck(0)
+        , nonApuSubSysCnt(0)
     { }
 
     virtual ~ImageHeaderTable() {}
@@ -411,7 +426,9 @@ public:
     virtual void SetBootDeviceAddress(uint32_t address) {};
     virtual void ValidateSecurityCombinations(Authentication::Type, Encryption::Type, Checksum::Type) = 0;
     virtual void SetOptionalData(uint32_t*, uint32_t) {};
+    virtual void SetUserOptionalData(std::vector<std::pair<std::string, uint32_t>> optionalDataInfo, uint32_t hashTableSize) {};
     virtual void SetXplmModulesData(BootImage& bi, uint32_t*, uint32_t) {};
+    virtual void SetOptionalDataSize(void) {};
 
     virtual uint32_t GetImageHeaderTableVersion(void) { return 0; }
     virtual uint32_t GetPartitionCount(void) { return 0; }
@@ -428,8 +445,11 @@ public:
     uint32_t metaHeaderLength;
     KeySource::Type metaHdrKeySrc;
     uint8_t* metaHdrSecHdrIv;
-    uint32_t* iht_optional_data;
+    void* iht_optional_data;
     uint32_t iht_optional_data_length;
+    uint32_t copied_iht_optional_data_length;
+    uint8_t idCodeCheck;
+    int nonApuSubSysCnt;
 
 protected:
     bool slaveBootSplitMode;
