@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -31,6 +32,15 @@
 #include "authentication-versal.h"
 #include "Keccak-compact-versal.h"
 
+extern "C" {
+#include "cdo-npi.h"
+#include "cdo-source.h"
+#include "cdo-command.h"
+#include "cdo-binary.h"
+#include "cdo-load.h"
+#include "cdo-overlay.h"
+};
+
 /*
 -------------------------------------------------------------------------------
 *****************************************************   F U N C T I O N S   ***
@@ -175,13 +185,11 @@ size_t VersalPartition::GetBootloaderTotalDataChunks(Binary::Length_t partitionS
                 {
                     dataChunks.push_back(secureChunkSize + SECURE_HDR_SZ + AES_GCM_TAG_SZ);
                     firstChunkSize = secureChunkSize + SECURE_HDR_SZ + AES_GCM_TAG_SZ;
-                    LOG_TRACE("length of pmc data remaining length 0x%X:", secureChunkSize + SECURE_HDR_SZ + AES_GCM_TAG_SZ);
                 }
                 else
                 {
                     dataChunks.push_back(secureChunkSize);
                     firstChunkSize = secureChunkSize;
-                    LOG_TRACE("length of pmc data remaining length 0x%X:", secureChunkSize);
                 }
             }
         }
@@ -190,12 +198,10 @@ size_t VersalPartition::GetBootloaderTotalDataChunks(Binary::Length_t partitionS
             if (chunkOnlength % secureChunkSize != 0)
             {
                 dataChunks.push_back(((chunkOnlength)-((dataChunksCount - 1) * secureChunkSize)));
-                LOG_TRACE("length of pmc data remaining length %d:", ((chunkOnlength)-((dataChunksCount - 1) * secureChunkSize)));
             }
             else
             {
                 dataChunks.push_back(secureChunkSize);
-                LOG_TRACE("length of pmc data remaining length %d:", secureChunkSize);
             }
 
             for (uint32_t itr = 0; itr < dataChunksCount - 1; itr++)
@@ -357,7 +363,7 @@ void VersalPartition::ChunkifyAndHash(Section* section, bool encryptionFlag)
     if (newLength <length)
     {
         LOG_DEBUG(DEBUG_STAMP, "Resize length is less than original length");
-        LOG_ERROR("Section resize issue for authentication..");
+        LOG_ERROR("Section resize issue for authentication");
     }
 
     uint8_t* dataPtr = new uint8_t[length];
@@ -526,6 +532,57 @@ void VersalPartition::Build(BootImage& bi, Binary& cache)
     {
         LOG_ERROR("Cannot reencrypt a partition that is already encrypted for %s", section->Name.c_str());
     }
+
+    /*******************************************************************************/
+    if ((imageHeader.GetPartitionType() == PartitionType::CONFIG_DATA_OBJ) && !header->preencrypted && !header->presigned)
+    {
+        size_t buffer_size = 0;
+        uint32_t* syncpt_offsets = NULL;
+        uint8_t num_of_sync_points = 0;
+        std::vector<uint32_t> sync_offsets;
+
+        CdoSequence * cdo_seq;
+        cdo_seq = decode_cdo_binary(header->partition->section->Data, header->partition->section->Length);
+
+        /* Enable the search for sync points - only needs to be done for SSIT devices */
+        search_for_sync_points();
+
+        uint8_t* buffer = (uint8_t*)cdoseq_to_binary(cdo_seq, &buffer_size, 0);
+
+        /* Get no. of sync points and sync points offsets */
+        num_of_sync_points = get_num_of_sync_points();
+        syncpt_offsets = get_slr_sync_point_offsets();
+
+        for (int i = 0; i < num_of_sync_points; i++)
+        {
+            size_t offset = (*(syncpt_offsets + i) * 4);
+            bi.sync_offsets.push_back(offset);
+            sync_offsets.push_back(offset);
+        }
+        delete syncpt_offsets;
+        delete buffer;
+
+        if (num_of_sync_points != 0)
+        {
+            std::string sync_addresses_filename = StringUtils::RemoveExtension(bi.options.GetOutputFileNames().front()) +
+                "_" + bi.Name + "_" + std::to_string(imageHeader.GetPartitionUid()) + "_sync_offsets.txt";
+            std::ofstream f(sync_addresses_filename.c_str(), std::ios_base::out | std::ios_base::binary);
+            f << "sync_offsets" << "\n";
+            for (size_t i = 0; i < sync_offsets.size(); i++)
+            {
+                f << sync_offsets[i] << "\n";
+            }
+            f.close();
+
+            if (f.fail())
+            {
+                LOG_ERROR("Failed to write sync addresses to the file: %s", sync_addresses_filename.c_str());
+            }
+            LOG_TRACE("Sync addresses written to file %s successfully", sync_addresses_filename.c_str());
+        }
+    }
+    /*******************************************************************************/
+
     if (versalNetSeries)
     {
         encryptCtx->ChunkifyAndProcess(bi, header);
@@ -753,6 +810,7 @@ void VersalPartition::Link(BootImage &bi)
     {
         if ((*acs))
         {
+            header->partition->section->partitionNum = header->partitionNum;
             (*acs)->Link(bi, header->partition->section);
         }
     }
