@@ -1,6 +1,6 @@
-
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -63,6 +63,7 @@ VersalPartitionHeader::VersalPartitionHeader(ImageHeader* imageheader, int index
     , kekIvFile("")
     , lockstep(Lockstep::LockstepDisable)
     , cluster(0)
+    , tcmBoot(TcmBoot::TcmBootDisable)
 {
     std::string name;
     slr = 0;
@@ -157,6 +158,9 @@ void VersalPartitionHeader::ReadHeader(std::ifstream& ifs)
     early_handoff = GetEarlyHandoff();
     hivec = GetHivec();
     partitionUid = GetPartitionUid();
+    lockstep = GetLockStepFlag();
+    cluster = GetDestinationCluster();
+    tcmBoot = GetTcmBootFlag();
 
     presigned = (authCertPresent != 0);
     if (presigned)
@@ -228,7 +232,7 @@ void VersalPartitionHeader::Link(BootImage &bi, PartitionHeader* next_part_hdr)
         addr = next_part_hdr->section->Address;
     }
     SetNextPartitionHeaderOffset(addr);
-    SetLoadAddress(loadAddress);
+    SetLoadAddress(loadAddress, bi.options.IsVersalNetSeries());
     SetPartitionWordOffset((uint32_t)partition->section->Address);
     SetPartitionAttributes();
     SetChecksumOffset();
@@ -483,7 +487,45 @@ void VersalPartitionHeader::SetExecAddress(uint64_t addr)
 }
 
 /******************************************************************************/
-void VersalPartitionHeader::SetLoadAddress(uint64_t addr)
+#define R52_0A_TCMA_BASE_ADDR 	0xEBA00000 /* R52_0A TCMA base address */
+#define R52_1A_TCMA_BASE_ADDR 	0xEBA40000 /* R52_1A TCMA base address */
+#define R52_0B_TCMA_BASE_ADDR 	0xEBA80000 /* R52_0B TCMA base address */
+#define R52_1B_TCMA_BASE_ADDR 	0xEBAC0000 /* R52_1B TCMA base address */
+#define R52_TCM_CLUSTER_OFFSET	0x00080000 /* R52_TCM TCM cluster offset */
+#define R52_TCMA_LOAD_ADDRESS	0x0        /* R52 TCMA load address */
+#define R52_TCM_TOTAL_LENGTH	0x30000    /* R52 TCMA total length */
+
+uint64_t VersalPartitionHeader::GetR52LoadAddr(uint64_t loadAddr)
+{
+    uint64_t address = loadAddr;
+
+    if ((imageHeader->GetDestCpu() == DestinationCPU::R5_0) && ((address < (R52_TCMA_LOAD_ADDRESS + R52_TCM_TOTAL_LENGTH))))
+    {
+        if (((address % R52_TCM_TOTAL_LENGTH) + GetEncryptedPartitionLength()) > R52_TCM_TOTAL_LENGTH)
+        {
+            //Status = XPlmi_UpdateStatus(XLOADER_ERR_TCM_ADDR_OUTOF_RANGE, 0);
+        }
+        address += R52_0A_TCMA_BASE_ADDR + (imageHeader->GetClusterNum() * R52_TCM_CLUSTER_OFFSET);
+    }
+    else if ((imageHeader->GetDestCpu() == DestinationCPU::R5_1) && ((address < (R52_TCMA_LOAD_ADDRESS + R52_TCM_TOTAL_LENGTH))))
+    {
+        if (((address % R52_TCM_TOTAL_LENGTH) + GetEncryptedPartitionLength()) > R52_TCM_TOTAL_LENGTH)
+        {
+            //Status = XPlmi_UpdateStatus(XLOADER_ERR_TCM_ADDR_OUTOF_RANGE, 0);
+        }
+        address += R52_1A_TCMA_BASE_ADDR + (imageHeader->GetClusterNum() * R52_TCM_CLUSTER_OFFSET);
+    }
+    else
+    {
+        /* Do nothing */
+    }
+
+    /* Update the load address */
+    return address;
+}
+
+/******************************************************************************/
+void VersalPartitionHeader::SetLoadAddress(uint64_t addr, bool versalNetSeries)
 {
     if ((imageHeader->GetPartitionType() == PartitionType::CONFIG_DATA_OBJ) ||
         (imageHeader->GetPartitionType() == PartitionType::CFI) || 
@@ -492,6 +534,12 @@ void VersalPartitionHeader::SetLoadAddress(uint64_t addr)
     {
         addr = 0xFFFFFFFFFFFFFFFF;
     }
+    if (versalNetSeries)
+    {
+        if((imageHeader->GetDestCpu() == DestinationCPU::R5_0) || (imageHeader->GetDestCpu() == DestinationCPU::R5_1))
+            addr = GetR52LoadAddr(addr);
+    }
+
     pHTable->destinationLoadAddress = addr;
 }
 
@@ -542,6 +590,10 @@ void VersalPartitionHeader::SetPartitionAttributes(void)
     {
         lockstep = Lockstep::LockstepEnable;
     }
+    if (imageHeader->GetTcmBootFlag() == true)
+    {
+        tcmBoot = TcmBoot::TcmBootEnable;
+    }
 
     if (hivec) 
     {
@@ -610,7 +662,8 @@ void VersalPartitionHeader::SetPartitionAttributes(void)
                                    (hivec << vphtHivecShift) |
                                    (dpaCM << vphtDpaCMShift) |
                                    (cluster << vNetphtClusterShift) |
-                                   (lockstep << vNetphtlockStepShift);
+                                   (lockstep << vNetphtlockStepShift) |
+                                   (tcmBoot << vNetphtTcmBootShift);
 }
 
 /******************************************************************************/
@@ -781,6 +834,12 @@ uint8_t VersalPartitionHeader::GetDestinationCpu(void)
     return ((pHTable->partitionAttributes >> vphtDestCpuShift) & vphtDestCpuMask);
 }
 
+/******************************************************************************/
+uint8_t VersalPartitionHeader::GetDestinationCluster(void)
+{
+    return ((pHTable->partitionAttributes >> vNetphtClusterShift) & vNetphtClusterMask);
+}
+
 /******************************************************************************/
 uint8_t VersalPartitionHeader::GetProcessorExecState(void)
 {
@@ -854,6 +913,12 @@ PufHdLoc::Type VersalPartitionHeader::GetPufHdLocation(void)
     return (PufHdLoc::Type)((pHTable->partitionAttributes >> vphtPufHDLocationShift) & vphtPufHDLocationMask);
 }
 
+/******************************************************************************/
+Lockstep::Type VersalPartitionHeader::GetLockStepFlag (void)
+{
+    return (Lockstep::Type)((pHTable->partitionAttributes >> vNetphtlockStepShift) & vNetphtlockStepMask);
+}
+
 /******************************************************************************/
 uint64_t VersalPartitionHeader::GetLQspiExecAddrForXip(uint64_t execAddr)
 {
@@ -910,6 +975,12 @@ uint8_t VersalPartitionHeader::GetAuthCertFlag(void)
     }
 }
 
+/******************************************************************************/
+TcmBoot::Type VersalPartitionHeader::GetTcmBootFlag(void)
+{
+    return (TcmBoot::Type)((pHTable->partitionAttributes >> vNetphtTcmBootShift) & vNetphtTcmBootMask);
+}
+
 /******************************************************************************/
 void VersalPartitionHeaderTable::Build(BootImage & bi, Binary & cache)
 {
@@ -921,7 +992,7 @@ void VersalPartitionHeaderTable::Build(BootImage & bi, Binary & cache)
         {
             if (bi.subSysImageList.size() > MAX_NUM_IMAGES_VERSAL)
             {
-                LOG_ERROR("The maximum number of images supported for Versal is %d.\n           No. of images found : %d", MAX_NUM_IMAGES_VERSAL, bi.imageList.size());
+                LOG_ERROR("The maximum number of images supported for Versal is %d.\n           No. of images found : %d", MAX_NUM_IMAGES_VERSAL, bi.subSysImageList.size());
             }
             if (bi.partitionHeaderList.size() > MAX_NUM_PARTITIONS_VERSAL)
             {
