@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -31,7 +32,6 @@
 #include "options.h"
 #include "logger.h"
 #include "encryptutils.h"
-#include "imageheadertable-versal.h"
 #include <string.h>
 #include <fstream>
 
@@ -76,6 +76,9 @@ typedef enum
     vphtEndiannessShift = 18,
     vphtEndiannessMask = 0x1,
 
+    vNetphtTcmBootShift = 19,
+    vNetphtTcmBootMask = 0x3,
+
     vphtHivecShift = 23,
     vphtHivecMask = 0x1,
 
@@ -166,8 +169,9 @@ public:
     void SetEncryptedPartitionLength(uint32_t len);
     void SetUnencryptedPartitionLength(uint32_t len);
     void SetTotalPartitionLength(uint32_t len);
-    void SetLoadAddress(uint64_t addr);
+    void SetLoadAddress(uint64_t addr, bool versalNetSeries);
     void SetExecAddress(uint64_t addr);
+    uint64_t GetR52LoadAddr(uint64_t loadAddr);
     void SetPartitionWordOffset(uint32_t addr);
     void SetPartitionAttributes(void);
     void SetSectionCount(uint32_t cnt);
@@ -189,6 +193,7 @@ public:
     uint8_t GetChecksumType(void);
     uint8_t GetOwnerType(void);
     uint8_t GetDestinationCpu(void);
+    uint8_t GetDestinationCluster(void);
     uint8_t GetProcessorExecState(void);
     uint8_t GetElfEndianess(void);
     uint8_t GetExceptionLevel(void);
@@ -207,9 +212,11 @@ public:
     PartitionType::Type GetPartitionType(void);
     DpaCM::Type GetDpaCMFlag(void);
     PufHdLoc::Type GetPufHdLocation(void);
+    Lockstep::Type GetLockStepFlag(void);
     uint32_t GetPartitionUid(void);
     uint64_t GetLQspiExecAddrForXip(uint64_t addr);
     uint32_t GetSectionCount(void);
+    TcmBoot::Type GetTcmBootFlag(void);
 
 private:
     uint8_t partitionEncrypted;
@@ -222,6 +229,7 @@ private:
     std::string kekIvFile;
     DpaCM::Type dpaCM;
     PufHdLoc::Type pufHdLoc;
+    TcmBoot::Type tcmBoot;
     VersalPartitionHeaderTableStructure* pHTable;
 };
 
@@ -232,6 +240,7 @@ public:
     void Build(BootImage& bi, Binary& cache);
     void ConfigureMetaHdrAuthenticationContext(BootImage& bi);
     void UpdateAtfHandoffParams(BootImage& bi);
+    void SetPartitionHashinOptionalData(BootImage & bi);
     void Link(BootImage& bi);
     Section* firstSection;
 };
