@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -74,7 +75,7 @@ public:
     virtual void SetEncryptedPartitionLength(uint32_t len) {}
     virtual void SetUnencryptedPartitionLength(uint32_t len) {}
     virtual void SetTotalPartitionLength(uint32_t len) {}
-    virtual void SetLoadAddress(uint64_t addr) {}
+    virtual void SetLoadAddress(uint64_t addr, bool arch) {}
     virtual void SetExecAddress(uint64_t addr) {}
     virtual void SetPartitionWordOffset(uint32_t addr) {}
     virtual void SetPartitionAttributes(void) {}
