@@ -1,6 +1,6 @@
-
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -69,6 +69,134 @@ VersalReadImage::~VersalReadImage()
     }
 }
 
+/**********************************************************************************************/
+void VersalReadImage::DisplayOptionalData()
+{
+    //Return without displaying anything if optional data size in IHT is 0
+    if(iHT->optionalDataSize == 0){
+        LOG_INFO("Optional data does not exist");
+        return;
+    }
+
+    size_t result;
+    uint32_t read_opData = 0;
+    uint32_t opData_total_length = 0;
+    uint32_t opData_length = 0;
+    uint32_t opData_id = 0;
+    uint32_t opData_hdr_length = sizeof(uint32_t);
+    uint32_t opData_checksum_length = sizeof(uint32_t);
+    bool smap_header_found = false;
+    uint32_t offset;
+    int cnt_opData = 0;
+
+    //Opening PDI to read optional data
+    FILE *binFile = fopen(binFilename.c_str(), "rb");
+    if (!binFile)
+    {
+        fclose(binFile);
+        LOG_ERROR("Cannot read file %s", binFilename.c_str());
+    }
+
+    //Creating text file to dump optional data
+    FILE* filePtr;
+    std::string fName = StringUtils::FolderPath(binFilename);
+    if (dumpPath != "")
+    {
+        fName = dumpPath;
+    }
+    fName = fName + "/" + "optional_data.txt";
+    filePtr = fopen(fName.c_str(), "wb");
+    if (!filePtr)
+    {
+        fclose(filePtr);
+        LOG_ERROR("Cannot create file %s", fName.c_str());
+    }
+
+    if (bH && ((bH->smapWords[0] == 0xDD000000) || (bH->smapWords[0] == 0x00DD0000) || (bH->smapWords[0] == 0x000000DD)))
+    {
+        smap_header_found = true;
+    }
+
+    if(bH)
+    {
+        offset = bH->imageHeaderByteOffset;
+    }
+    else
+    {
+        if(smap_header_found)
+        {
+            offset = sizeof(VersalSmapWidthTable);
+        }
+        else
+        {
+            offset = 0;
+        }
+    }
+    offset += sizeof(VersalImageHeaderTableStructure);
+
+    while(!feof(binFile))
+    {
+        if(read_opData == (iHT->optionalDataSize * 4))
+            break;
+
+        //Extracting optional data header
+        if (!(fseek(binFile, offset, SEEK_SET)))
+        {
+            uint32_t* buffer = new uint32_t[opData_hdr_length];
+            size_t result = fread(buffer, 1, opData_hdr_length, binFile);
+            if (result != opData_hdr_length)
+            {
+                LOG_ERROR("Error parsing Optional Data Header from PDI file");
+            }
+
+            opData_total_length = (((*buffer) & 0xFFFF0000) >> 16) * 4;
+            opData_id = (*buffer) & 0x0000FFFF;
+            opData_length = opData_total_length - opData_hdr_length - opData_checksum_length;
+        }
+        else
+        {
+            LOG_ERROR("Error parsing Optional Data Header from PDI file");
+        }
+
+        if(0x00 <= opData_id && opData_id <= 0x20){
+            read_opData += opData_total_length;
+            offset += opData_total_length;
+            continue;
+        }
+
+        if(opData_id == 0xFFFF){
+            break;
+        }
+
+        fprintf(filePtr, "Optional Data %d\n", ++cnt_opData);
+        fprintf(filePtr, "Length %d\n", opData_length);
+        fprintf(filePtr, "ID %d\n", opData_id);
+
+        fprintf(filePtr, "Data\n");
+        char* opData_buffer = new char[opData_length];
+        result = fread(opData_buffer, 1, opData_length, binFile);
+        if (result != opData_length)
+        {
+                LOG_ERROR("Error parsing Optional Data from PDI file");
+        }
+
+        for(uint32_t i=0; i < opData_length; i++)
+        {
+            char c1 = (*opData_buffer);
+            opData_buffer++;
+            fprintf(filePtr, "%x", c1);
+        }
+
+        offset += opData_total_length;
+        read_opData += opData_total_length;
+        fprintf(filePtr, "\n\n");
+    }
+
+    fclose(filePtr);
+    LOG_INFO("%s generated successfully", StringUtils::BaseName(fName).c_str());
+    fclose(binFile);
+}
+
 /**********************************************************************************************/
 void VersalReadImage::ReadPartitions()
 {
@@ -85,15 +213,24 @@ void VersalReadImage::ReadPartitions()
     std::list<VersalPartitionHeaderTableStructure*>::iterator pHT = pHTs.begin();
     for (std::list<VersalImageHeaderStructure*>::iterator iH = iHs.begin(); iH != iHs.end(); iH++)
     {
-        uint32_t prev_id = 0xffffffff;
-        uint32_t section_count = 0;
+        uint32_t part_sec_index = 0;
+        uint32_t part_sec_count = 0;
+        uint32_t part_index = 0;
 
         for (cnt_index = 0; cnt_index < (*iH)->dataSectionCount; cnt_index++)
         {
             uint32_t length = (*pHT)->encryptedPartitionLength * 4;
             uint8_t* buffer = new uint8_t[length];
-            uint32_t id = (*pHT)->puid & 0xFFFF;
             offset = (*pHT)->partitionWordOffset * 4;
+            if ((*pHT)->dataSectionCount > 0)
+            {
+               part_sec_index = 0;
+               part_sec_count = (*pHT)->dataSectionCount;
+            }
+            if (((*pHT)->dataSectionCount > 0) && (cnt_index != 0))
+            {
+               part_index++;
+            }
             if (!(fseek(binFile, offset, SEEK_SET)))
             {
                 result = fread(buffer, 1, length, binFile);
@@ -107,11 +244,11 @@ void VersalReadImage::ReadPartitions()
                     {
                         length = bH->totalPlmLength;
                     }
-                    if (prev_id == id)
+                    if((part_sec_count > 1) && ((*pHT)->dataSectionCount == 0))
                     {
-                        section_count++;
+                        part_sec_index++;
                     }
-                    DumpPartitions(buffer, length, (*iH)->imageName, id, section_count);
+                    DumpPartitions(buffer, length, (*iH)->imageName, part_index, part_sec_index);
                 }
                 else
                 {
@@ -122,7 +259,7 @@ void VersalReadImage::ReadPartitions()
                     }
                     if ((dumpType == DumpOption::PLM) || (dumpType == DumpOption::BOOT_FILES))
                     {
-                        DumpPartitions(buffer, length, "plm", id, section_count);
+                        DumpPartitions(buffer, length, "plm", part_index, part_sec_index);
                         if (dumpType == DumpOption::PLM)
                         {
                             delete[] buffer;
@@ -159,7 +296,6 @@ void VersalReadImage::ReadPartitions()
                 LOG_ERROR("Error parsing Partition Headers from bin file");
             }
             pHT++;
-            prev_id = id;
             delete[] buffer;
         }
     }
@@ -179,7 +315,7 @@ void VersalReadImage::ReadHeaderTableDetails()
 
     if (!binFile)
     {
-        fclose(binFile);
+        //fclose(binFile);
         LOG_ERROR("Cannot read file %s", binFilename.c_str());
     }
 
@@ -354,8 +490,13 @@ void VersalReadImage::ReadBinaryFile(DumpOption::Type dump, std::string path)
     dumpPath = path;
 
     ReadHeaderTableDetails();
+    if (dumpType == DumpOption::PARTITIONS)
+    {
+        DisplayImageInfo();
+    }
     if (readType != ReadImageOption::NONE)
     {
+        DisplayImageInfo();
         DisplayHeaderTableDetails(readType);
     }
     ReadPartitions();
@@ -390,6 +531,7 @@ void VersalReadImage::DisplayHeaderTableDetails(ReadImageOption::Type type)
 
     case ReadImageOption::IHT:
         DisplayImageHeaderTable();
+        DisplayOptionalData();
         break;
 
     case ReadImageOption::IH:
@@ -415,6 +557,7 @@ void VersalReadImage::DisplayHeaderTableDetails(ReadImageOption::Type type)
     default:
         DisplayBootHeader();
         DisplayImageHeaderTable();
+        DisplayOptionalData();
         if (iHT->metaHdrKeySource == KeySource::None)
         {
             DisplayImageHeaders();
@@ -499,9 +642,12 @@ void VersalReadImage::DisplayBootHeader(void)
     DisplayIV("plm_sec_hdr_iv (0x64) : ", bH->plmSecureHdrIv);
     DisplayValue("puf_shutter (0x70) : ", bH->shutterValue);
     DisplayIV("pmccdo_sec_hdr_iv (0x74) : ", bH->pmcCdoSecureHdrIv);
-    //VersalNet
-    DisplayValue("puf_ro_swap (0x80) : ", bH->pufRoSwapConfigVal);
-    DisplayValue("revoke_id (0x84) : ", bH->revokeId);
+
+    if (versalNetSeries)
+    {
+        DisplayValue("puf_ro_swap (0x80) : ", bH->pufRoSwapConfigVal);
+        DisplayValue("revoke_id (0x84) : ", bH->revokeId);
+    }
     DisplayValue("metahdr_offset (0xc4) : ", bH->imageHeaderByteOffset);
     DisplayKey("puf_data (0x928) : ", bH->puf);
     DisplayValue("checksum (0xf30) : ", bH->headerChecksum);
@@ -524,11 +670,8 @@ void VersalReadImage::DumpPartitions(uint8_t* buffer, uint32_t length, std::stri
     fName = fName + "/" + name;
     if (dumpType == DumpOption::PARTITIONS)
     {
-        fName += StringUtils::Format("_%x", id);
-        if (index != 0)
-        {
-            fName += StringUtils::Format(".%x", index);
-        }
+        fName += StringUtils::Format(".%x", id);
+        fName += StringUtils::Format(".%x", index);
     }
     fName += extension;
     filePtr = fopen(fName.c_str(), "wb");
@@ -586,6 +729,10 @@ void VersalReadImage::DisplayImageHeaders(void)
         DisplayValue("id (0x18) : ", (*iH)->imageId, "unique_id (0x24) : ", (*iH)->uniqueId);
         DisplayValue("parent_unique_id (0x28) : ", (*iH)->parentUniqueId, "function_id (0x2c) : ", (*iH)->functionId);
         DisplayValue("memcpy_address_lo (0x30) : ", (*iH)->memcpyAddressLo, "memcpy_address_hi (0x34) : ", (*iH)->memcpyAddressHi);
+        if (versalNetSeries)
+        {
+            DisplayValue("pcr_value (0x38) : ", (*iH)->pcrNumber, "pcr_mindex (0x3A) : ", (*iH)->pcrMeasurementIndex);
+        }
         DisplayValue("checksum (0x3c) : ", (*iH)->ihChecksum);
         std::cout << " attribute list -" << std::endl;
         DisplayIhAttributes((*iH)->imageAttributes);
@@ -599,10 +746,26 @@ void VersalReadImage::DisplayPartitionHeaderTable(void)
     std::list<VersalPartitionHeaderTableStructure*>::iterator pHT = pHTs.begin();
     for (std::list<VersalImageHeaderStructure*>::iterator iH = iHs.begin(); iH != iHs.end(); iH++)
     {
+        uint32_t part_sec_index = 0;
+        uint32_t part_sec_count = 0;
+        uint32_t part_index = 0;
         for (cnt_index = 0; cnt_index < (*iH)->dataSectionCount; cnt_index++)
         {
+            if ((*pHT)->dataSectionCount > 0)
+            {
+               part_sec_index = 0;
+               part_sec_count = (*pHT)->dataSectionCount;
+            }
+            if (((*pHT)->dataSectionCount > 0) && (cnt_index != 0))
+            {
+               part_index++;
+            }
+            if((part_sec_count > 1) && ((*pHT)->dataSectionCount == 0))
+            {
+                part_sec_index++;
+            }
             Separator();
-            std::cout << "   PARTITION HEADER TABLE " << "(" << (*iH)->imageName << "." << std::dec << cnt_index << ")" << std::endl;
+            std::cout << "   PARTITION HEADER TABLE " << "(" << (*iH)->imageName << "." << std::hex << part_index << "." << std::hex << part_sec_index << ")" << std::endl;
             Separator();
             DisplayValue("encrypted_length (0x00) : ", (*pHT)->encryptedPartitionLength, "unencrypted_length (0x04) : ", (*pHT)->unencryptedPartitionLength);
             DisplayValue("total_length (0x08) : ", (*pHT)->totalPartitionLength, "next_pht (0x0c) : ", (*pHT)->nextPartitionHeaderOffset);
@@ -750,6 +913,51 @@ void VersalReadImage::DisplaySmapVectors(void)
     std::cout << std::setfill('0') << "0x" << std::setw(8) << std::hex << bH->smapWords[0] << std::setw(8) << std::hex << bH->smapWords[1] << std::setw(8) << std::hex << bH->smapWords[2] << std::setw(8) << std::hex << bH->smapWords[3] << std::endl;
 }
 
+/******************************************************************************/
+void VersalReadImage::DisplayImageInfo()
+{
+    Separator();
+    std::cout << "   BOOTIMAGE COMPONENTS" << std::endl;
+    Separator();
+
+    uint32_t cnt_index = 0;
+    std::list<VersalPartitionHeaderTableStructure*>::iterator pHT = pHTs.begin();
+    for (std::list<VersalImageHeaderStructure*>::iterator iH = iHs.begin(); iH != iHs.end(); iH++)
+    {
+        uint32_t part_sec_index = 0;
+        uint32_t part_sec_count = 0;
+        uint32_t part_index = 0;
+        if ((dumpType == DumpOption::PARTITIONS) || readType != ReadImageOption::NONE)
+        {
+           LOG_MSG("+---Image: %s [id:0x%x]",(*iH)->imageName, (*iH)->imageId);
+        }
+        for (cnt_index = 0; cnt_index < (*iH)->dataSectionCount; cnt_index++)
+        {
+            std::string partitionType = GetPartitionType((*pHT)->partitionAttributes);
+            std::string partitionCore = GetPartitionCore((*pHT)->partitionAttributes);
+            if ((*pHT)->dataSectionCount > 0)
+            {
+               part_sec_index = 0;
+               part_sec_count = (*pHT)->dataSectionCount;
+            }
+            if (((*pHT)->dataSectionCount > 0) && (cnt_index != 0))
+            {
+               part_index++;
+            }
+            if((part_sec_count > 1) && ((*pHT)->dataSectionCount == 0))
+            {
+                part_sec_index++;
+            }
+            std::string fName = (*iH)->imageName;
+            fName += StringUtils::Format(".%x", part_index);
+            fName += StringUtils::Format(".%x", part_sec_index);
+            LOG_MSG("    |__ %s [core: %s, type: %s]",fName.c_str(), partitionCore.c_str(), partitionType.c_str());
+            pHT++;
+        }
+        LOG_MSG("");
+    }
+}
+
 /******************************************************************************/
 void VersalReadImage::DisplayBhAttributes(uint32_t value)
 {
@@ -761,7 +969,14 @@ void VersalReadImage::DisplayBhAttributes(uint32_t value)
         default: val = "[efuse]";       break;
     }
     val1 = val;
-    DisplayAttributes("puf_hd_source ", val1,"","");
+
+    switch ((value >> BH_PUF_MODE_BIT_SHIFT) & BH_PUF_MODE_BIT_MASK)
+    {
+        case 0: val = "[puf-12k]";      break;
+        case 3: val = "[puf-4k]";       break;
+        default: val = "[invalid]";     break;
+    }
+    DisplayAttributes("puf_hd_source ", val1, "puf_mode ", val);
 
     switch ((value >> BI_HASH_BIT_SHIFT) & BI_HASH_BIT_MASK)
     {
@@ -777,35 +992,32 @@ void VersalReadImage::DisplayBhAttributes(uint32_t value)
     }
     DisplayAttributes("integrity ", val1, "dpa_cm ", val);
 
-    switch ((value >> BH_RSA_BIT_SHIFT) & BH_RSA_BIT_MASK)
+    if (!versalNetSeries)
     {
-        case 3: val = "[enabled]";      break;
-        default: val = "[disabled]";    break;
+        switch ((value >> BH_RSA_BIT_SHIFT) & BH_RSA_BIT_MASK)
+        {
+            case 3: val = "[enabled]";      break;
+            default: val = "[disabled]";    break;
+        }
+        DisplayAttributes("bh_auth ", val, "", "");
     }
-    val1 = val;
-    
-    switch ((value >> BH_PUF_MODE_BIT_SHIFT) & BH_PUF_MODE_BIT_MASK)
-    {
-        case 0: val = "[puf-12k]";      break;
-        case 3: val = "[puf-4k]";       break;
-        default: val = "[invalid]";     break;
-    }
-    DisplayAttributes("bh_auth ", val1, "puf_mode ", val);
 
-    //VersalNet
-    switch ((value >> BH_RSA_SINGED_BIT_SHIFT) & BH_RSA_SINGED_BIT_MASK)
+    if (versalNetSeries)
     {
-    case 3: val = "[yes]";      break;
-    default: val = "[no]";    break;
-    }
-    val1 = val;
+        switch ((value >> BH_RSA_SINGED_BIT_SHIFT) & BH_RSA_SINGED_BIT_MASK)
+        {
+            case 3: val = "[yes]";      break;
+            default: val = "[no]";    break;
+        }
+        val1 = val;
 
-    switch ((value >> BH_DICE_BIT_SHIFT) & BH_DICE_BIT_MASK)
-    {
-    case 3: val = "[enabled]";      break;
-    default: val = "[disabled]";    break;
+        switch ((value >> BH_DICE_BIT_SHIFT) & BH_DICE_BIT_MASK)
+        {
+            case 3: val = "[enabled]";      break;
+            default: val = "[disabled]";    break;
+        }
+        DisplayAttributes("rsa_signed ", val1, "dice ", val);
     }
-    DisplayAttributes("rsa_signed ", val1, "dice ", val);
 }
 
 /*********************************************************************************/
@@ -813,7 +1025,7 @@ void VersalReadImage::DisplayIhtAttributes(uint32_t value)
 {
     std::string val, val1;
 
-    switch ((value >> vihtIdCodeCheckShift) & vihtIdCodeCheckMask)
+    switch ((value >> vihtSiliconRevisionIdCodeCheckShift) & vihtSiliconRevisionIdCodeCheckMask)
     {
         case 1: val = "[true]";         break;
         default: val = "[false]";       break;
@@ -825,7 +1037,7 @@ void VersalReadImage::DisplayIhtAttributes(uint32_t value)
         case 2: val = "[sdk]";          break;
         default: val = "[rsvd]";        break;
     }
-    DisplayAttributes("id_code_check ", val1, "image_creator ", val);
+    DisplayAttributes("silicon_rev_id_code_check ", val1, "image_creator ", val);
 
     switch ((value >> vihtSecBootDeviceShift) & vihtSecBootDeviceMask)
     {
@@ -850,6 +1062,7 @@ void VersalReadImage::DisplayIhtAttributes(uint32_t value)
         case 18: val = "[mmc-raw]";     break;
         case 19: val = "[mmc0]";        break;
         case 20: val = "[mmc0-raw]";    break;
+        case 21: val = "[imagestore]";  break;
         default: val = "[n/a]";         break;
     }
     val1 = val;
@@ -867,6 +1080,14 @@ void VersalReadImage::DisplayIhtAttributes(uint32_t value)
     }
     val1 = val;
     DisplayAttributes("puf_hd_source", val1, "", "");
+
+    switch ((value >> vihtIdCodeCheckShift) & vihtIdCodeCheckMask)
+    {
+        case 3: val = "[true]";           break;
+        default: val = "[false]";         break;
+    }
+    val1 = val;
+    DisplayAttributes("skip_id_code_check", val1, "", "");
 }
 
 /************************************************************************************************/
@@ -903,26 +1124,26 @@ void VersalReadImage::DisplayIhAttributes(uint32_t value)
     std::string powerDomains = "";
     switch ((value >> vihLowPowerDomainShift) & vihLowPowerDomainMask)
     {
-    case 1: powerDomains += "[lpd]";    break;
-    default:                            break;
+        case 1: powerDomains += "[lpd]";    break;
+        default:                            break;
     }
 
     switch ((value >> vihFullPowerDomainShift) & vihFullPowerDomainMask)
     {
-    case 1: powerDomains += "[fpd]";    break;
-    default:                            break;
+        case 1: powerDomains += "[fpd]";    break;
+        default:                            break;
     }
 
     switch ((value >> vihSystemPowerDomainShift) & vihSystemPowerDomainMask)
     {
-    case 1: powerDomains += "[spd]";    break;
-    default:                            break;
+        case 1: powerDomains += "[spd]";    break;
+        default:                            break;
     }
     val1 = val;
     switch ((value >> vihPLPowerDomainShift) & vihPLPowerDomainMask)
     {
-    case 1: powerDomains += "[pld]";    break;
-    default:                            break;
+        case 1: powerDomains += "[pld]";    break;
+        default:                            break;
     }
     if (powerDomains == "") powerDomains = "[none]";
     DisplayAttributes("dependentPowerDomains ", powerDomains, " ", "");
@@ -977,8 +1198,17 @@ void VersalReadImage::DisplayPhtAttributes(uint32_t value)
     switch ((value >> vphtDestCpuShift) & vphtDestCpuMask)
     {
         case 0: val = "[none]";           break;
-        case 1: val = "[a72-0]";          break;
-        case 2: val = "[a72-1]";          break;
+        case 1: 
+            if (versalNetSeries) val = "[a78-0]"; 
+            else val = "[a72-0]";         break;
+        case 2: 
+            if (versalNetSeries) val = "[a78-1]";
+            else val = "[a72-1]";         break;
+        if (versalNetSeries)
+        {
+            case 3: val = "[a78-2]";      break;
+            case 4: val = "[a78-3]";      break;
+        }
         case 5: val = "[r5-0]";           break;
         case 6: val = "[r5-1]";           break;
         case 7: val = "[r5-lockstep]";    break;
@@ -993,25 +1223,32 @@ void VersalReadImage::DisplayPhtAttributes(uint32_t value)
 
     DisplayAttributes("exec_state ", val1, "core ", val);
 
-    //VersalNet
-    switch ((value >> vNetphtlockStepShift) & vNetphtlockStepMask)
+    if (versalNetSeries)
     {
-        case 0: val = "[disabled]";      break;
-        case 3: val = "[enabled]";       break;
-        default: val = "[invalid]";      break;
+        switch ((value >> vNetphtlockStepShift) & vNetphtlockStepMask)
+        {
+            case 0: val = "[disabled]";      break;
+            case 3: val = "[enabled]";       break;
+            default: val = "[invalid]";      break;
+        }
+        val1 = val;
+        switch ((value >> vNetphtClusterShift) & vNetphtClusterMask)
+        {
+            case 0: val = "[0]";            break;
+            case 1: val = "[1]";            break;
+            case 2: val = "[2]";            break;
+            case 3: val = "[3]";            break;
+            default: val = "[invalid]";     break;
+        }
+        DisplayAttributes("lockstep ", val1, "cluster", val);
+        switch ((value >> vNetphtTcmBootShift) & vNetphtTcmBootMask)
+        {
+            case 3: val = "[enabled]";      break;
+            default: val = "[disabled]";    break;
+        }
+        val1 = val;
+        DisplayAttributes("tcmboot ", val1, " ", "");
     }
-    val1 = val;
-    switch ((value >> vNetphtClusterShift) & vNetphtClusterMask)
-    {
-        case 0: val = "[0]";            break;
-        case 1: val = "[1]";            break;
-        case 2: val = "[2]";            break;
-        case 3: val = "[3]";            break;
-        default: val = "[invalid]";     break;
-    }
-
-    DisplayAttributes("lockstep ", val1, "cluster", val);
-
     switch ((value >> vphtChecksumTypeShift) & vphtChecksumTypeMask)
     {
         case 0: val = "[none]";          break;
@@ -1082,3 +1319,56 @@ void VersalReadImage::DisplayPhtAttributes(uint32_t value)
     val1 = val;
     DisplayAttributes("dpacm ", val1, " ", "");
 }
+
+/*******************************************************************************************************/
+std::string VersalReadImage::GetPartitionCore(uint32_t value)
+{
+    std::string val;
+    bool is_elf = (((value >> vphtPartitionTypeShift) & vphtPartitionTypeMask) == 1);
+
+    switch ((value >> vphtDestCpuShift) & vphtDestCpuMask)
+    {
+        case 0: val = "none";           break;
+        case 1:
+            if (versalNetSeries) val = "a78-0";
+            else val = "a72-0";         break;
+        case 2:
+            if (versalNetSeries) val = "a78-1";
+            else val = "a72-1";         break;
+        if (versalNetSeries)
+        {
+            case 3: val = "a78-2";      break;
+            case 4: val = "a78-3";      break;
+        }
+        case 5: val = "r5-0";           break;
+        case 6: val = "r5-1";           break;
+        case 7: val = "r5-lockstep";    break;
+        case 8: val = "psm";            break;
+        case 9: val = "aie";            break;
+        default: val = "invalid";       break;
+    }
+    if (!is_elf)
+    {
+        val = "n/a";
+    }
+    return val;
+}
+
+/*******************************************************************************************************/
+std::string VersalReadImage::GetPartitionType(uint32_t value)
+{
+    std::string val;
+    switch ((value >> vphtPartitionTypeShift) & vphtPartitionTypeMask)
+    {
+        case 0: val = "none";             break;
+        case 1: val = "elf";              break;
+        case 2: val = "cdo";              break;
+        case 3: val = "cfi";              break;
+        case 4: val = "raw";              break;
+        case 5: val = "raw-elf";          break;
+        case 6: val = "cfi-gsc-mask";     break;
+        case 7: val = "cfi-gsc-unmask";   break;
+        default: val = "invalid";         break;
+    }
+    return val;
+}
