@@ -1,5 +1,6 @@
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -58,19 +59,19 @@ public:
         binFilename = filename;
         bH = NULL;
         iHT = NULL;
-        dumpType = DumpOption::NONE;
-        readType = ReadImageOption::NONE;
         iHs.clear();
         pHTs.clear();
         partitionBuffers.clear();
         aCs.clear();
         authenticationVerified = true;
+        versalNetSeries = false;
     }
     ~VersalReadImage();
 
     void ReadBinaryFile(DumpOption::Type dump=DumpOption::NONE, std::string path="");
     void ReadHeaderTableDetails();
     void ReadPartitions();
+    void DisplayOptionalData();
     void DisplayImageDetails(ReadImageOption::Type type, DumpOption::Type dump, std::string path="");
     void DisplayHeaderTableDetails(ReadImageOption::Type type);
     void DisplayBootHeader(void);
@@ -92,17 +93,16 @@ public:
     bool VerifyECDSASignature(bool nist, uint8_t * data, size_t dataLength,  ACKeyECDSA *eckey, uint8_t* signature);
     bool VerifyECDSAP521Signature(bool nist, uint8_t * data, size_t dataLength, ACKeyECDSAP521 *eckey, uint8_t* signature);
     void Separator(void);
-    void DumpBootHeader(void);
     void DumpPartitions(uint8_t* buffer, uint32_t length, std::string name, uint32_t id = 0, uint32_t index = 0);
+    void DisplayImageInfo();
     uint32_t GetPdiId(void);
     uint32_t GetParentId(void);
     uint8_t GetCreatorId(void);
+    std::string GetPartitionType(uint32_t);
+    std::string GetPartitionCore(uint32_t);
 
 protected:
     std::string binFilename;
-    DumpOption::Type dumpType;
-    ReadImageOption::Type readType;
-    std::string dumpPath;
     VersalBootHeaderStructure* bH;
     VersalImageHeaderTableStructure* iHT;
     VersalImageHeaderStructure *iH;
