@@ -1,6 +1,6 @@
-
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -44,8 +44,65 @@ ZynqReadImage::~ZynqReadImage()
     }
 }
 
+/**********************************************************************************************/
+void ZynqReadImage::ReadPartitions()
+{
+    size_t result;
+    uint64_t offset = 0;
+    FILE *binFile = fopen(binFilename.c_str(), "rb");
+
+    if (!binFile)
+    {
+        fclose(binFile);
+        LOG_ERROR("Cannot read file %s", binFilename.c_str());
+    }
+    std::list<std::string>::iterator partitionName = pHTNames.begin();
+    for (std::list<ZynqPartitionHeaderTableStructure*>::iterator partitionHdr = pHTs.begin(); partitionHdr != pHTs.end(); partitionHdr++, partitionName++)
+    {
+        uint32_t bufferLength = ((*partitionHdr)->encryptedPartitionLength * 4);
+        uint8_t* tempBuffer = new uint8_t[bufferLength];
+        memset(tempBuffer, 0, bufferLength);
+
+        offset = (*partitionHdr)->partitionWordOffset * 4;
+        if (!(fseek(binFile, offset, SEEK_SET)))
+        {
+            result = fread(tempBuffer, 1, bufferLength, binFile);
+            if (result != bufferLength)
+            {
+                LOG_ERROR("Error reading partition for hash calculation");
+            }
+            if (bH && bH->sourceOffset == offset)
+            {
+                bufferLength = bH->totalFsblLength;
+            }
+            if (dumpType == DumpOption::BOOT_FILES)
+            {
+                DumpPartitions(tempBuffer, bufferLength, *partitionName);
+            }
+            if (dumpType == DumpOption::PARTITIONS)
+            {
+                DumpPartitions(tempBuffer, bufferLength, *partitionName);
+            }
+        }
+        else
+        {
+            LOG_ERROR("Error parsing Partitions from BootImage file %s",binFilename.c_str());
+        }
+    }
+    fclose(binFile);
+}
+
 /*******************************************************************************/
 void ZynqReadImage::ReadBinaryFile(DumpOption::Type dump, std::string path)
+{
+    if (StringUtils::GetExtension(binFilename) == ".mcs")
+    {
+        LOG_ERROR("The option '-read/-dump' is not supported on mcs format file : %s", binFilename.c_str());
+    }
+    ReadPartitions();
+}
+/*******************************************************************************/
+void ZynqReadImage::ReadHeaderTableDetails()
 {
     size_t result;
     uint64_t offset = 0;
@@ -76,6 +133,29 @@ void ZynqReadImage::ReadBinaryFile(DumpOption::Type dump, std::string path)
         LOG_ERROR("Boot Header not found in %s", binFilename.c_str());
     }
 
+    if ((dumpType == DumpOption::BH) || (dumpType == DumpOption::BOOT_FILES))
+    {
+        FILE* filePtr;
+        std::string fName = binFilename;
+        if (dumpPath != "")
+        {
+            fName = dumpPath + "/" + StringUtils::BaseName(binFilename);
+        }
+        fName = StringUtils::RemoveExtension(fName);
+        fName += "_bh.bin";
+        filePtr = fopen(fName.c_str(), "wb");
+        if (filePtr != NULL)
+        {
+            result = fwrite(bH, 1, size_t(sizeof(ZynqBootHeaderStructure)), filePtr);
+            if (result != sizeof(ZynqBootHeaderStructure))
+            {
+                LOG_ERROR("Error dumping Boot Header to a file");
+            }
+            fclose(filePtr);
+            LOG_INFO("%s generated successfully", StringUtils::BaseName(fName).c_str());
+        }
+    }
+
     /* Image Header Table Extraction */
     offset = bH->imageHeaderByteOffset;
     if (!(fseek(binFile, offset, SEEK_SET)))
@@ -108,6 +188,7 @@ void ZynqReadImage::ReadBinaryFile(DumpOption::Type dump, std::string path)
     do
     {
         iH = new ZynqImageHeaderStructure;
+        memset(iH, 0, sizeof(ZynqImageHeaderStructure));
         if (!(fseek(binFile, offset, SEEK_SET)))
         {
             result = fread(iH, 1, 4 * sizeof(uint32_t), binFile);
@@ -167,6 +248,19 @@ void ZynqReadImage::ReadBinaryFile(DumpOption::Type dump, std::string path)
         offset += sizeof(ZynqPartitionHeaderTableStructure);
     }
 
+    /* Insert Partition Names from Image Header based on Section Count */
+    std::list<std::string>::iterator iHName = iHNames.begin();
+    for (std::list<ZynqImageHeaderStructure*>::iterator iH = iHs.begin(); iH != iHs.end(); iH++, iHName++)
+    {
+        for (uint32_t sectionIndex = 0; sectionIndex < (*iH)->imageNameLength; sectionIndex++)
+        {
+            std::stringstream ss;
+            ss << sectionIndex;
+            pHTNames.push_back(*iHName + "." + ss.str());
+        }
+    }
+
+
     /* Authentication Certificates Extraction */
     uint8_t*  header_ac = NULL;
     if (iHT->headerAuthCertificateWordOffset != 0)
@@ -210,6 +304,35 @@ void ZynqReadImage::ReadBinaryFile(DumpOption::Type dump, std::string path)
     fclose(binFile);
 }
 
+/******************************************************************************/
+void ZynqReadImage::DumpPartitions(uint8_t* buffer, uint32_t length, std::string name)
+{
+    FILE* filePtr;
+    size_t result;
+    std::string extension = ".bin";
+
+    std::string fName = StringUtils::FolderPath(binFilename);
+    if (dumpPath != "")
+    {
+        fName = dumpPath;
+    }
+    fName = fName + "/" + name;
+    fName += extension;
+    filePtr = fopen(fName.c_str(), "wb");
+
+    if (filePtr != NULL)
+    {
+        result = fwrite(buffer, 1, length, filePtr);
+        if (result != length)
+        {
+            LOG_ERROR("Error dumping partition %s to a file", fName.c_str());
+        }
+        fclose(filePtr);
+        LOG_INFO("%s generated successfully", StringUtils::BaseName(fName).c_str());
+    }
+    return;
+}
+
 /*******************************************************************************/
 void ZynqReadImage::DisplayBootHeader(void)
 {
