@@ -1,6 +1,6 @@
-
 /******************************************************************************
-* Copyright 2015-2021 Xilinx, Inc.
+* Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -54,10 +54,13 @@ public:
         iHs.clear();
         pHTs.clear();
         authenticationVerified = true;
+        plAcCount = 0;
     }
     ~ZynqMpReadImage();
 
     void ReadBinaryFile(DumpOption::Type dump = DumpOption::NONE, std::string path = "");
+    void ReadHeaderTableDetails();
+    void ReadPartitions();
     void DisplayBootHeader(void);
     void DisplayImageHeaderTable(void);
     void DisplayImageHeaders(void);
@@ -67,6 +70,7 @@ public:
     void DisplayBhAttributes(uint32_t value);
     void DisplayPhtAttributes(uint32_t value);
     void DisplayBootVectors(void);
+    void DumpPartitions(uint8_t* buffer, uint32_t length, std::string name);
 
     void VerifyAuthentication(bool);
     void VerifyHeaderTableSignature();
