@@ -1,6 +1,6 @@
-
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -255,7 +255,7 @@ void VersalReadImage::VerifyHeaderTableSignature()
     VerifySPKSignature(*auth_cert);
 
     /* Verifying IHT Signature */
-    uint32_t iHTLength = sizeof(VersalImageHeaderTableStructure);
+    uint32_t iHTLength = sizeof(VersalImageHeaderTableStructure) + (iHT->optionalDataSize * 4);
     uint8_t* tempIHBuffer = new uint8_t[iHTLength];
     memset(tempIHBuffer, 0, iHTLength);
     bool signatureVerified = false;
@@ -348,6 +348,7 @@ void VersalReadImage::VerifyHeaderTableSignature()
             offset = sizeof(VersalImageHeaderTableStructure);
         }
     }
+    offset += (iHT->optionalDataSize * 4);
     if (!(fseek(binFile, offset, SEEK_SET)))
     {
         result = fread(tempBuffer, 1, headersAcDataSize, binFile);
@@ -529,9 +530,16 @@ void VersalReadImage::VerifyPartitionSignature(void)
                 VerifySPKSignature(*auth_cert);
 
                 uint32_t actualSecureChunkSize = SECURE_32K_CHUNK - SHA3_LENGTH_BYTES;
+                if(versalNetSeries && isItBootloader)
+                    actualSecureChunkSize = SECURE_16K_CHUNK - SHA3_LENGTH_BYTES;
+
                 bool nist = true;
                 /* Partition Signature should not be included for hash calculation. */
                 uint32_t encryptedSize = ((*partitionHdr)->encryptedPartitionLength * 4);
+                if (isItBootloader && versalNetSeries)
+                {
+                    encryptedSize = bH->plmLength;
+                }
                 uint32_t dataBufferLength = ((*partitionHdr)->totalPartitionLength * 4) - SIGN_LENGTH_VERSAL;
                 uint32_t acBufferLength = sizeof(AuthCertificate4096Sha3PaddingStructure) - SIGN_LENGTH_VERSAL;
                 uint8_t* tempBuffer = new uint8_t[dataBufferLength];
@@ -564,7 +572,7 @@ void VersalReadImage::VerifyPartitionSignature(void)
                 {
                     LOG_ERROR("Error parsing Partitions from BootImage file %s", binFilename.c_str());
                 }
-                if (encryptedSize <= actualSecureChunkSize || isItBootloader)
+                if (encryptedSize <= actualSecureChunkSize || (isItBootloader && !versalNetSeries))
                 {
                     if (isItBootloader)
                     {
@@ -620,7 +628,7 @@ void VersalReadImage::VerifyPartitionSignature(void)
                         {
                             encryptedSize -= (SECURE_HDR_SZ + AES_GCM_TAG_SZ);
                         }
-                        int count = encryptedSize / actualSecureChunkSize;
+                        int count = (encryptedSize / actualSecureChunkSize);
                         if (encryptedSize % actualSecureChunkSize != 0)
                         {
                             count = count + 1;
@@ -634,8 +642,8 @@ void VersalReadImage::VerifyPartitionSignature(void)
                                 dataSize = lastChunkSize;
                             }
                             uint8_t* dataBuffer = new uint8_t[dataSize];
-                            memcpy(hashBuffer, tempBuffer + chunk0Size - SHA3_LENGTH_BYTES + (SECURE_32K_CHUNK) * (i-1), SHA3_LENGTH_BYTES);
-                            memcpy(dataBuffer, tempBuffer + chunk0Size + (SECURE_32K_CHUNK) * (i-1), dataSize);
+                            memcpy(hashBuffer, tempBuffer + chunk0Size - SHA3_LENGTH_BYTES + (actualSecureChunkSize + SHA3_LENGTH_BYTES) * (i-1), SHA3_LENGTH_BYTES);
+                            memcpy(dataBuffer, tempBuffer + chunk0Size + (actualSecureChunkSize + SHA3_LENGTH_BYTES) * (i-1), dataSize);
                             uint8_t* shaHash = new uint8_t[SHA3_LENGTH_BYTES];
                             Versalcrypto_hash(shaHash, dataBuffer, dataSize, true);
                             int compare = memcmp(shaHash, hashBuffer, SHA3_LENGTH_BYTES);
