@@ -1,6 +1,6 @@
-
 /******************************************************************************
 * Copyright 2015-2022 Xilinx, Inc.
+* Copyright 2022-2023 Advanced Micro Devices, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,11 +16,11 @@
 ******************************************************************************/
 
 #ifndef RDI_VERSION
-#define RDI_VERSION "2022.2"
+#define RDI_VERSION "2024.2"
 #endif
 
-#define PROGRAMNAME     "Xilinx Bootgen"
-#define COPYRIGHT       "Copyright (c) 1986-2022 Xilinx, Inc. All rights reserved.\n"
+#define PROGRAMNAME     "Bootgen"
+#define COPYRIGHT       "Copyright (c) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.\n"
 
 
 /******************************************************************************
@@ -30,9 +30,9 @@
 ------------------------------------------------------------------------------+\n\
                        COMMAND LINE OPTIONS                                   |\n\
 ------------------------------------------------------------------------------+\n\
- arch              - Xilinx Architecture                                      |\n\
+ arch              - Device Architecture                                      |\n\
  bif_help          - Print the BIF help summary                               |\n\
- dual_ospi_mode    - Generate 2 output files for Dual QSPI                    |\n\
+ dual_ospi_mode    - Generate 2 output files for Dual OSPI                    |\n\
  dual_qspi_mode    - Generate 2 output files for Dual QSPI                    |\n\
  dump              - Dumps components from PDI as bin files                   |\n\
  dump_dir          - Dumps components in specified directory                  |\n\
@@ -59,6 +59,7 @@
  verify_kdf        - Verify AES key generation                                |\n\
  w                 - Overwrite mode                                           |\n\
  zynqmpes1         - Generate boot image for ZynqMP ES1                       |\n\
+ enable_auth_opt   - Enables authentication optimization                      |\n\
 ------------------------------------------------------------------------------+\n\
  Note    : Some options are valid only for specific architectures.            |\n\
            Help for such options include a list of supported architectures.   |\n\
@@ -81,7 +82,7 @@
  aarch32_mode         - 32-bit mode execution for binary partitions           |\n\
  alignment            - Alignment for the partition                           |\n\
  big_endian           - Big endian partition                                  |\n\
- bootimage            - Boot Image File (in Xilinx Boot Image format)         |\n\
+ bootimage            - BootImage File(Bootgen generated BootImage/PDI Format)|\n\
  bootloader           - First Stage Boot Loader                               |\n\
  bootvectors          - Vector table for XIP                                  |\n\
  boot_device          - Secondary Boot Device                                 |\n\
@@ -119,6 +120,10 @@
  type                 - Type of partition                                     |\n\
  udf_bh               - User Defined Feild of boot header                     |\n\
  xip_mode             - eXecute In Place                                      |\n\
+ pcr                  - PCR register number                                   |\n\
+ pcr_mindex           - PCR measurement index                                 |\n\
+ imagestore           - ID corresponding to Image Store PDI                   |\n\
+ optionaldata         - Adds optional metadata                                |\n\
 ------------------------------------------------------------------------------+\n\
  Encryption attributes                                                        |\n\
 ------------------------------------------------------------------------------+\n\
@@ -136,7 +141,7 @@
  keysrc_encryption    - Encryption Key Source                                 |\n\
  puf_file             - PUF Helper Data File                                  |\n\
  blocks               - Blocks for Key Rolling Encryption                     |\n\
- dpacm_enable          - Enables DPA Counter Measure for Encryption           |\n\
+ dpacm_enable         - Enables DPA Counter Measure for Encryption            |\n\
 ------------------------------------------------------------------------------+\n\
  Authentication attributes                                                    |\n\
 ------------------------------------------------------------------------------+\n\
